@@ -24,17 +24,12 @@
 ;;;
 
 (use-modules (gnutls)
-             (gnutls build tests)
              (srfi srfi-4))
 
 
 ;; TLS session settings.
-(define %protos  (list protocol/tls-1.0))
-(define %certs   '())
-(define %ciphers (list cipher/null cipher/arcfour cipher/aes-128-cbc
-                       cipher/aes-256-cbc))
-(define %kx      (list kx/anon-dh))
-(define %macs    (list mac/sha1 mac/rmd160 mac/md5))
+(define priorities
+  "NONE:+VERS-TLS-ALL:+CIPHER-ALL:+MAC-ALL:+SIGN-ALL:+COMP-ALL:+ANON-DH")
 
 ;; Message sent by the client.
 (define %message (apply u8vector (iota 256)))
@@ -55,7 +50,10 @@
 ;; (set-log-procedure! (lambda (level str)
 ;;                       (format #t "[~a|~a] ~a" (getpid) level str)))
 
-(run-test
+(dynamic-wind
+    (lambda ()
+      #t)
+
     (lambda ()
       (let ((socket-pair (socketpair PF_UNIX SOCK_STREAM 0))
             (pid         (primitive-fork)))
@@ -63,12 +61,7 @@
 
             (let ((client (make-session connection-end/client)))
               ;; client-side (child process)
-              (set-session-default-priority! client)
-              (set-session-certificate-type-priority! client %certs)
-              (set-session-kx-priority! client %kx)
-              (set-session-protocol-priority! client %protos)
-              (set-session-cipher-priority! client %ciphers)
-              (set-session-mac-priority! client %macs)
+              (set-session-priorities! client priorities)
 
               (set-session-transport-fd! client (fileno (car socket-pair)))
               (set-session-credentials! client (make-anonymous-client-credentials))
@@ -78,16 +71,11 @@
               (record-send client %message)
               (bye client close-request/rdwr)
 
-              (primitive-exit))
+              (exit))
 
             (let ((server (make-session connection-end/server)))
               ;; server-side
-              (set-session-default-priority! server)
-              (set-session-certificate-type-priority! server %certs)
-              (set-session-kx-priority! server %kx)
-              (set-session-protocol-priority! server %protos)
-              (set-session-cipher-priority! server %ciphers)
-              (set-session-mac-priority! server %macs)
+              (set-session-priorities! server priorities)
 
               (set-session-transport-fd! server (fileno (cdr socket-pair)))
               (let ((cred (make-anonymous-server-credentials))
@@ -101,7 +89,11 @@
               (let* ((buf (make-u8vector (u8vector-length %message)))
                      (amount (record-receive! server buf)))
                 (bye server close-request/rdwr)
-                (and (= amount (u8vector-length %message))
-                     (equal? buf %message))))))))
+                (exit (= amount (u8vector-length %message))
+                      (equal? buf %message)))))))
+
+    (lambda ()
+      ;; failure
+      (exit 1)))
 
 ;;; arch-tag: 8c98de24-0a53-4290-974e-4b071ad162a0
