@@ -62,10 +62,9 @@ static gnutls_cert *alloc_and_load_x509_certs (gnutls_x509_crt_t * certs,
 static gnutls_privkey_t alloc_and_load_x509_key (gnutls_x509_privkey_t key,
                                                  int deinit);
 
-#ifdef ENABLE_PKCS11
 static gnutls_privkey_t alloc_and_load_pkcs11_key (gnutls_pkcs11_privkey_t
                                                    key, int deinit);
-#endif
+
 
 /* Copies data from a internal certificate struct (gnutls_cert) to 
  * exported certificate struct (cert_auth_info_t)
@@ -118,6 +117,7 @@ _gnutls_copy_certificate_auth_info (cert_auth_info_t info,
   info->ncerts = ncerts;
 
   info->cert_type = cert[0].cert_type;
+  info->sign_algo = cert[0].sign_algo;
 
 #ifdef ENABLE_OPENPGP
   if (cert[0].cert_type == GNUTLS_CRT_OPENPGP)
@@ -568,10 +568,9 @@ call_get_cert_callback (gnutls_session_t session,
               goto cleanup;
             }
         }
-#endif
       break;
+#endif
     case GNUTLS_PRIVKEY_PKCS11:
-#ifdef ENABLE_PKCS11
       if (st2.key.pkcs11 != NULL)
         {
           local_key =
@@ -583,7 +582,6 @@ call_get_cert_callback (gnutls_session_t session,
               goto cleanup;
             }
         }
-#endif
       break;
     case GNUTLS_PRIVKEY_X509:
       if (st2.key.x509 != NULL)
@@ -615,7 +613,6 @@ cleanup:
             {
               gnutls_x509_crt_deinit (st2.cert.x509[i]);
             }
-          gnutls_free(st2.cert.x509);
         }
     }
   else
@@ -756,10 +753,9 @@ cleanup:
  */
 
 static int
-_gnutls_gen_x509_crt (gnutls_session_t session, opaque ** data)
+_gnutls_gen_x509_crt (gnutls_session_t session, gnutls_buffer_st * data)
 {
   int ret, i;
-  opaque *pdata;
   gnutls_cert *apr_cert_list;
   gnutls_privkey_t apr_pkey;
   int apr_cert_list_length;
@@ -791,23 +787,19 @@ _gnutls_gen_x509_crt (gnutls_session_t session, opaque ** data)
    * the one produced here )
    */
 
-  (*data) = gnutls_malloc (ret);
-  pdata = (*data);
+  ret = _gnutls_buffer_append_prefix(data, 24, ret - 3);
+  if (ret < 0)
+    return gnutls_assert_val(ret);
 
-  if (pdata == NULL)
-    {
-      gnutls_assert ();
-      return GNUTLS_E_MEMORY_ERROR;
-    }
-  _gnutls_write_uint24 (ret - 3, pdata);
-  pdata += 3;
   for (i = 0; i < apr_cert_list_length; i++)
     {
-      _gnutls_write_datum24 (pdata, apr_cert_list[i].raw);
-      pdata += (3 + apr_cert_list[i].raw.size);
+      ret = _gnutls_buffer_append_data_prefix( data, 24, apr_cert_list[i].raw.data,
+        apr_cert_list[i].raw.size);
+      if (ret < 0)
+        return gnutls_assert_val(ret);
     }
 
-  return ret;
+  return data->length;
 }
 
 enum PGPKeyDescriptorType
@@ -815,13 +807,13 @@ enum PGPKeyDescriptorType
 
 #ifdef ENABLE_OPENPGP
 static int
-_gnutls_gen_openpgp_certificate (gnutls_session_t session, opaque ** data)
+_gnutls_gen_openpgp_certificate (gnutls_session_t session, gnutls_buffer_st * data)
 {
   int ret;
-  opaque *pdata;
   gnutls_cert *apr_cert_list;
   gnutls_privkey_t apr_pkey;
   int apr_cert_list_length;
+  uint8_t type;
 
   /* find the appropriate certificate */
   if ((ret =
@@ -843,56 +835,61 @@ _gnutls_gen_openpgp_certificate (gnutls_session_t session, opaque ** data)
       ret += apr_cert_list[0].raw.size;
     }
 
-  (*data) = gnutls_malloc (ret);
-  pdata = (*data);
-
-  if (pdata == NULL)
-    {
-      gnutls_assert ();
-      return GNUTLS_E_MEMORY_ERROR;
-    }
-
-  _gnutls_write_uint24 (ret - 3, pdata);
-  pdata += 3;
+  ret = _gnutls_buffer_append_prefix( data, 24, ret - 3);
+  if (ret < 0)
+    return gnutls_assert_val(ret);
 
 
   if (apr_cert_list_length > 0)
     {
       if (apr_cert_list[0].use_subkey != 0)
         {
-          *pdata = PGP_KEY_SUBKEY;
-          pdata++;
-          *pdata = sizeof (apr_cert_list[0].subkey_id);
-          pdata++;
-          memcpy (pdata, apr_cert_list[0].subkey_id,
-                  sizeof (apr_cert_list[0].subkey_id));
-          pdata += sizeof (apr_cert_list[0].subkey_id);
+          type = PGP_KEY_SUBKEY;
+
+          ret = _gnutls_buffer_append_data( data, &type, 1);
+          if (ret < 0)
+            return gnutls_assert_val(ret);
+
+          ret = _gnutls_buffer_append_data_prefix( data, 8, apr_cert_list[0].subkey_id,
+              sizeof (apr_cert_list[0].subkey_id));
+          if (ret < 0)
+            return gnutls_assert_val(ret);
         }
       else
         {
-          *pdata = PGP_KEY;
-          pdata++;
+          type = PGP_KEY;
+          ret = _gnutls_buffer_append_data( data, &type, 1);
+          if (ret < 0)
+            return gnutls_assert_val(ret);
         }
 
-      _gnutls_write_datum24 (pdata, apr_cert_list[0].raw);
-      pdata += (3 + apr_cert_list[0].raw.size);
+      ret = _gnutls_buffer_append_data_prefix( data, 24, apr_cert_list[0].raw.data,
+        apr_cert_list[0].raw.size);
+      if (ret < 0)
+        return gnutls_assert_val(ret);
     }
   else                          /* empty - no certificate */
     {
-      *pdata = PGP_KEY;
-      pdata++;
-      _gnutls_write_uint24 (0, pdata);
+      type = PGP_KEY;
+
+      ret = _gnutls_buffer_append_data( data, &type, 1);
+      if (ret < 0)
+        return gnutls_assert_val(ret);
+
+      ret = _gnutls_buffer_append_prefix( data, 24, 0);
+      if (ret < 0)
+        return gnutls_assert_val(ret);
     }
 
-  return ret;
+  return data->length;
 }
 
 static int
-_gnutls_gen_openpgp_certificate_fpr (gnutls_session_t session, opaque ** data)
+_gnutls_gen_openpgp_certificate_fpr (gnutls_session_t session, gnutls_buffer_st * data)
 {
   int ret, packet_size;
+  uint8_t type, fpr[20];
   size_t fpr_size;
-  opaque *pdata;
   gnutls_cert *apr_cert_list;
   gnutls_privkey_t apr_pkey;
   int apr_cert_list_length;
@@ -918,54 +915,50 @@ _gnutls_gen_openpgp_certificate_fpr (gnutls_session_t session, opaque ** data)
   else                          /* empty certificate case */
     return _gnutls_gen_openpgp_certificate (session, data);
 
-  (*data) = gnutls_malloc (packet_size);
-  pdata = (*data);
-
-  if (pdata == NULL)
-    {
-      gnutls_assert ();
-      return GNUTLS_E_MEMORY_ERROR;
-    }
-
-  _gnutls_write_uint24 (packet_size - 3, pdata);
-  pdata += 3;
+  ret = _gnutls_buffer_append_prefix( data, 24, packet_size - 3);
+  if (ret < 0)
+    return gnutls_assert_val(ret);
 
   if (apr_cert_list[0].use_subkey)
     {
-      *pdata = PGP_KEY_FINGERPRINT_SUBKEY;
-      pdata++;
-      *pdata = sizeof (apr_cert_list[0].subkey_id);
-      pdata++;
-      memcpy (pdata, apr_cert_list[0].subkey_id,
-              sizeof (apr_cert_list[0].subkey_id));
-      pdata += sizeof (apr_cert_list[0].subkey_id);
+      type = PGP_KEY_FINGERPRINT_SUBKEY;
+      ret = _gnutls_buffer_append_data( data, &type, 1);
+      if (ret < 0)
+        return gnutls_assert_val(ret);
+      
+      ret = _gnutls_buffer_append_data_prefix( data, 8, 
+        apr_cert_list[0].subkey_id, sizeof(apr_cert_list[0].subkey_id));
+      if (ret < 0)
+        return gnutls_assert_val(ret);
     }
   else
     {
-      *pdata = PGP_KEY_FINGERPRINT;     /* key fingerprint */
-      pdata++;
+      type = PGP_KEY_FINGERPRINT;     /* key fingerprint */
+      ret = _gnutls_buffer_append_data( data, &type, 1);
+      if (ret < 0)
+        return gnutls_assert_val(ret);
     }
 
-  *pdata = 20;
-  pdata++;
-
-  fpr_size = 20;
-
+  fpr_size = sizeof(fpr);
   if ((ret =
-       _gnutls_openpgp_fingerprint (&apr_cert_list[0].raw, pdata,
+       _gnutls_openpgp_fingerprint (&apr_cert_list[0].raw, fpr,
                                     &fpr_size)) < 0)
     {
       gnutls_assert ();
       return ret;
     }
 
-  return packet_size;
+  ret = _gnutls_buffer_append_data_prefix( data, 8, fpr, fpr_size);
+  if (ret < 0)
+    return gnutls_assert_val(ret);
+
+  return data->length;
 }
 #endif
 
 
 int
-_gnutls_gen_cert_client_certificate (gnutls_session_t session, opaque ** data)
+_gnutls_gen_cert_client_certificate (gnutls_session_t session, gnutls_buffer_st * data)
 {
   switch (session->security_parameters.cert_type)
     {
@@ -986,7 +979,7 @@ _gnutls_gen_cert_client_certificate (gnutls_session_t session, opaque ** data)
 }
 
 int
-_gnutls_gen_cert_server_certificate (gnutls_session_t session, opaque ** data)
+_gnutls_gen_cert_server_certificate (gnutls_session_t session, gnutls_buffer_st * data)
 {
   switch (session->security_parameters.cert_type)
     {
@@ -1173,7 +1166,7 @@ _gnutls_proc_openpgp_server_certificate (gnutls_session_t session,
   gnutls_cert *peer_certificate_list = NULL;
   int peer_certificate_list_size = 0;
   gnutls_datum_t tmp, akey = { NULL, 0 };
-  uint8_t subkey_id[GNUTLS_OPENPGP_KEYID_SIZE];
+  gnutls_openpgp_keyid_t subkey_id;
   unsigned int subkey_id_set = 0;
 
   cred = (gnutls_certificate_credentials_t)
@@ -1529,20 +1522,17 @@ _gnutls_proc_cert_cert_req (gnutls_session_t session, opaque * data,
 }
 
 int
-_gnutls_gen_cert_client_cert_vrfy (gnutls_session_t session, opaque ** data)
+_gnutls_gen_cert_client_cert_vrfy (gnutls_session_t session, gnutls_buffer_st * data)
 {
   int ret;
   gnutls_cert *apr_cert_list;
   gnutls_privkey_t apr_pkey;
-  int apr_cert_list_length, size;
+  int apr_cert_list_length;
   gnutls_datum_t signature = { NULL, 0 };
   int total_data;
-  opaque *p;
   gnutls_sign_algorithm_t sign_algo;
   gnutls_protocol_t ver = gnutls_protocol_get_version (session);
 
-  *data = NULL;
-
   /* find the appropriate certificate */
   if ((ret =
        _gnutls_get_selected_cert (session, &apr_cert_list,
@@ -1577,43 +1567,36 @@ _gnutls_gen_cert_client_cert_vrfy (gnutls_session_t session, opaque ** data)
       total_data += 2;
     }
 
-  *data = gnutls_malloc (total_data);
-  if (*data == NULL)
-    {
-      _gnutls_free_datum (&signature);
-      return GNUTLS_E_MEMORY_ERROR;
-    }
-
-  p = *data;
   if (_gnutls_version_has_selectable_sighash (ver))
     {
       const sign_algorithm_st *aid;
+      uint8_t p[2];
       /* error checking is not needed here since we have used those algorithms */
       aid = _gnutls_sign_to_tls_aid (sign_algo);
       if (aid == NULL)
-        {
-          ret = GNUTLS_E_UNKNOWN_ALGORITHM;
-          goto cleanup;
-        }
+        return gnutls_assert_val(GNUTLS_E_UNKNOWN_ALGORITHM);
 
       p[0] = aid->hash_algorithm;
       p[1] = aid->sign_algorithm;
-      p += 2;
+      ret = _gnutls_buffer_append_data(data, p, 2);
+      if (ret < 0)
+        {
+          gnutls_assert();
+          goto cleanup;
+        }
     }
 
-  size = signature.size;
-  _gnutls_write_uint16 (size, p);
+  ret = _gnutls_buffer_append_data_prefix(data, 16, signature.data, signature.size);
+  if (ret < 0)
+    {
+      gnutls_assert();
+      goto cleanup;
+    }
 
-  p += 2;
-  memcpy (p, signature.data, size);
-
-  _gnutls_free_datum (&signature);
-
-  return total_data;
+  ret = data->length;
 
 cleanup:
   _gnutls_free_datum (&signature);
-  gnutls_free(*data);
   return ret;
 }
 
@@ -1693,16 +1676,14 @@ _gnutls_proc_cert_client_cert_vrfy (gnutls_session_t session,
   return 0;
 }
 
-
 #define CERTTYPE_SIZE 3
 int
-_gnutls_gen_cert_server_cert_req (gnutls_session_t session, opaque ** data)
+_gnutls_gen_cert_server_cert_req (gnutls_session_t session, gnutls_buffer_st * data)
 {
   gnutls_certificate_credentials_t cred;
   int size, ret;
-  opaque *pdata;
+  uint8_t tmp_data[CERTTYPE_SIZE];
   gnutls_protocol_t ver = gnutls_protocol_get_version (session);
-  const int signalgosize = 2 + MAX_SIGNATURE_ALGORITHMS * 2;
 
   /* Now we need to generate the RDN sequence. This is
    * already in the CERTIFICATE_CRED structure, to improve
@@ -1727,27 +1708,22 @@ _gnutls_gen_cert_server_cert_req (gnutls_session_t session, opaque ** data)
   if (_gnutls_version_has_selectable_sighash (ver))
     /* Need two bytes to announce the number of supported hash
        functions (see below).  */
-    size += signalgosize;
+    size += MAX_SIGN_ALGO_SIZE;
 
-  (*data) = gnutls_malloc (size);
-  pdata = (*data);
+  tmp_data[0] = CERTTYPE_SIZE - 1;
+  tmp_data[1] = RSA_SIGN;
+  tmp_data[2] = DSA_SIGN;          /* only these for now */
 
-  if (pdata == NULL)
-    {
-      gnutls_assert ();
-      return GNUTLS_E_MEMORY_ERROR;
-    }
-
-  pdata[0] = CERTTYPE_SIZE - 1;
-
-  pdata[1] = RSA_SIGN;
-  pdata[2] = DSA_SIGN;          /* only these for now */
-  pdata += CERTTYPE_SIZE;
+  ret = _gnutls_buffer_append_data( data, tmp_data, CERTTYPE_SIZE);
+  if (ret < 0)
+    return gnutls_assert_val(ret);
 
   if (_gnutls_version_has_selectable_sighash (ver))
     {
+      uint8_t p[MAX_SIGN_ALGO_SIZE];
+
       ret =
-        _gnutls_sign_algorithm_write_params (session, pdata, signalgosize);
+        _gnutls_sign_algorithm_write_params (session, p, MAX_SIGN_ALGO_SIZE);
       if (ret < 0)
         {
           gnutls_assert ();
@@ -1755,23 +1731,29 @@ _gnutls_gen_cert_server_cert_req (gnutls_session_t session, opaque ** data)
         }
 
       /* recalculate size */
-      size = size - signalgosize + ret;
-      pdata += ret;
+      size -= MAX_SIGN_ALGO_SIZE + ret;
+
+      ret = _gnutls_buffer_append_data( data, p, ret);
+      if (ret < 0)
+        return gnutls_assert_val(ret);
     }
 
   if (session->security_parameters.cert_type == GNUTLS_CRT_X509 &&
       session->internals.ignore_rdn_sequence == 0)
     {
-      _gnutls_write_datum16 (pdata, cred->x509_rdn_sequence);
-      /* pdata += cred->x509_rdn_sequence.size + 2; */
+      ret = _gnutls_buffer_append_data_prefix( data, 16, cred->x509_rdn_sequence.data,
+        cred->x509_rdn_sequence.size);
+      if (ret < 0)
+        return gnutls_assert_val(ret);
     }
   else
     {
-      _gnutls_write_uint16 (0, pdata);
-      /* pdata+=2; */
+      ret = _gnutls_buffer_append_prefix( data, 16, 0);
+      if (ret < 0)
+        return gnutls_assert_val(ret);
     }
 
-  return size;
+  return data->length;
 }
 
 
@@ -1974,8 +1956,6 @@ alloc_and_load_pgp_key (gnutls_openpgp_privkey_t key, int deinit)
 }
 #endif
 
-#ifdef ENABLE_PKCS11
-
 /* converts the given raw key to gnutls_privkey* and allocates
  * space for it.
  */
@@ -2009,8 +1989,6 @@ alloc_and_load_pkcs11_key (gnutls_pkcs11_privkey_t key, int deinit)
   return local_key;
 }
 
-#endif
-
 void
 _gnutls_selected_certs_deinit (gnutls_session_t session)
 {
@@ -2026,7 +2004,6 @@ _gnutls_selected_certs_deinit (gnutls_session_t session)
       session->internals.selected_cert_list = NULL;
       session->internals.selected_cert_list_length = 0;
 
-      gnutls_privkey_deinit(session->internals.selected_key);
       session->internals.selected_key = NULL;
     }
 
@@ -2063,7 +2040,7 @@ _gnutls_server_select_cert (gnutls_session_t session,
                             gnutls_pk_algorithm_t requested_algo)
 {
   unsigned i;
-  int idx;
+  int idx, ret;
   gnutls_certificate_credentials_t cred;
 
   cred = (gnutls_certificate_credentials_t)
@@ -2077,19 +2054,31 @@ _gnutls_server_select_cert (gnutls_session_t session,
   /* If the callback which retrieves certificate has been set,
    * use it and leave.
    */
-  if (cred->server_get_cert_callback != NULL ||
-    cred->get_cert_callback != NULL)
-    return call_get_cert_callback (session, NULL, 0, NULL, 0);
+  if (cred->server_get_cert_callback != NULL)
+    {
+      ret = call_get_cert_callback (session, NULL, 0, NULL, 0);
+      if (ret < 0)
+        return gnutls_assert_val(ret);
+      return ret;
+    }
 
   /* Otherwise... */
 
   idx = -1;                     /* default is use no certificate */
 
-
+  _gnutls_handshake_log("HSK[%p]: Requested PK algorithm: %s (%d) -- ctype: %s (%d)\n", session, 
+    gnutls_pk_get_name(requested_algo), requested_algo, 
+    gnutls_certificate_type_get_name(session->security_parameters.cert_type), session->security_parameters.cert_type);
+  
   for (i = 0; i < cred->ncerts; i++)
     {
       /* find one compatible certificate
        */
+      _gnutls_handshake_log("HSK[%p]: certificate[%d] PK algorithm: %s (%d) - ctype: %s (%d) - sig: %s (%d)\n", session, i,
+        gnutls_pk_get_name(cred->cert_list[i][0].subject_pk_algorithm), cred->cert_list[i][0].subject_pk_algorithm,
+        gnutls_certificate_type_get_name(cred->cert_list[i][0].cert_type), cred->cert_list[i][0].cert_type,
+        gnutls_sign_get_name(cred->cert_list[i][0].sign_algo), cred->cert_list[i][0].sign_algo);
+
       if (requested_algo == GNUTLS_PK_ANY ||
           requested_algo == cred->cert_list[i][0].subject_pk_algorithm)
         {
@@ -2101,9 +2090,6 @@ _gnutls_server_select_cert (gnutls_session_t session,
 	      && (cred->cert_list[i][0].cert_type == GNUTLS_CRT_OPENPGP
 		  ||	/* FIXME: make this a check for certificate
 			   type capabilities */
-		  !_gnutls_version_has_selectable_sighash
-		  (gnutls_protocol_get_version (session))
-		  ||
 		  _gnutls_session_sign_algo_requested
 		  (session, cred->cert_list[i][0].sign_algo) == 0))
 	    {
@@ -2125,8 +2111,11 @@ _gnutls_server_select_cert (gnutls_session_t session,
                                   cred->pkey[idx], 0);
     }
   else
-    /* Certificate does not support REQUESTED_ALGO.  */
-    return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
+    {
+      gnutls_assert();
+      /* Certificate does not support REQUESTED_ALGO.  */
+      return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
+    }
 
   return 0;
 }
