@@ -71,15 +71,7 @@ typedef struct gnutls_certificate_credentials_st
 #endif
 
   /* X509 specific stuff */
-
-  gnutls_x509_crt_t *x509_ca_list;
-  unsigned x509_ncas;           /* number of CAs in the ca_list 
-                                 */
-
-  gnutls_x509_crl_t *x509_crl_list;
-  unsigned x509_ncrls;          /* number of CRLs in the crl_list 
-                                 */
-
+  gnutls_x509_trust_list_t tlist;
   unsigned int verify_flags;    /* flags to be used at 
                                  * certificate verification.
                                  */
@@ -123,9 +115,10 @@ typedef struct cert_auth_info_st
   unsigned int ncerts;          /* holds the size of the list above */
 
   gnutls_certificate_type_t cert_type;
+  gnutls_sign_algorithm_t sign_algo;
 #ifdef ENABLE_OPENPGP
   int use_subkey;
-  uint8_t subkey_id[GNUTLS_OPENPGP_KEYID_SIZE];
+  gnutls_openpgp_keyid_t subkey_id;
 #endif
 } *cert_auth_info_t;
 
@@ -134,10 +127,10 @@ typedef struct cert_auth_info_st cert_auth_info_st;
 void _gnutls_free_rsa_info (rsa_info_st * rsa);
 
 /* AUTH X509 functions */
-int _gnutls_gen_cert_server_certificate (gnutls_session_t, opaque **);
-int _gnutls_gen_cert_client_certificate (gnutls_session_t, opaque **);
-int _gnutls_gen_cert_client_cert_vrfy (gnutls_session_t, opaque **);
-int _gnutls_gen_cert_server_cert_req (gnutls_session_t, opaque **);
+int _gnutls_gen_cert_server_certificate (gnutls_session_t, gnutls_buffer_st *);
+int _gnutls_gen_cert_client_certificate (gnutls_session_t, gnutls_buffer_st *);
+int _gnutls_gen_cert_client_cert_vrfy (gnutls_session_t, gnutls_buffer_st *);
+int _gnutls_gen_cert_server_cert_req (gnutls_session_t, gnutls_buffer_st *);
 int _gnutls_proc_cert_cert_req (gnutls_session_t, opaque *, size_t);
 int _gnutls_proc_cert_client_cert_vrfy (gnutls_session_t, opaque *, size_t);
 int _gnutls_proc_cert_server_certificate (gnutls_session_t, opaque *, size_t);
