@@ -120,20 +120,17 @@ _gnutls_proc_dh_common_client_kx (gnutls_session_t session,
   return 0;
 }
 
-int _gnutls_gen_dh_common_client_kx (gnutls_session_t session, opaque** data)
+int _gnutls_gen_dh_common_client_kx (gnutls_session_t session, gnutls_buffer_st* data)
 {
   return _gnutls_gen_dh_common_client_kx_int(session, data, NULL);
 }
 
 int
-_gnutls_gen_dh_common_client_kx_int (gnutls_session_t session, opaque ** data, gnutls_datum_t* pskkey)
+_gnutls_gen_dh_common_client_kx_int (gnutls_session_t session, gnutls_buffer_st* data, gnutls_datum_t* pskkey)
 {
   bigint_t x = NULL, X = NULL;
-  size_t n_X;
   int ret;
 
-  *data = NULL;
-
   X = gnutls_calc_dh_secret (&x, session->key->client_g,
                              session->key->client_p);
   if (X == NULL || x == NULL)
@@ -145,19 +142,13 @@ _gnutls_gen_dh_common_client_kx_int (gnutls_session_t session, opaque ** data, g
 
   _gnutls_dh_set_secret_bits (session, _gnutls_mpi_get_nbits (x));
 
-  _gnutls_mpi_print (X, NULL, &n_X);
-  (*data) = gnutls_malloc (n_X + 2);
-  if (*data == NULL)
+  ret = _gnutls_buffer_append_mpi( data, 16, X, 0);
+  if (ret < 0)
     {
-      ret = GNUTLS_E_MEMORY_ERROR;
+      gnutls_assert();
       goto error;
     }
 
-  _gnutls_mpi_print (X, &(*data)[2], &n_X);
-  _gnutls_mpi_release (&X);
-
-  _gnutls_write_uint16 (n_X, &(*data)[0]);
-
   /* calculate the key after calculating the message */
   session->key->KEY =
     gnutls_calc_dh_key (session->key->client_Y, x, session->key->client_p);
@@ -203,17 +194,11 @@ _gnutls_gen_dh_common_client_kx_int (gnutls_session_t session, opaque ** data, g
       goto error;
     }
 
-  ret = n_X + 2;
+  ret = data->length;
 
 error:
   _gnutls_mpi_release (&x);
   _gnutls_mpi_release (&X);
-  
-  if (ret < 0)
-    {
-      gnutls_free (*data);
-      *data = NULL;
-    }
   return ret;
 }
 
@@ -314,13 +299,11 @@ _gnutls_proc_dh_common_server_kx (gnutls_session_t session,
  * be inserted */
 int
 _gnutls_dh_common_print_server_kx (gnutls_session_t session,
-                                   bigint_t g, bigint_t p, opaque ** data,
+                                   bigint_t g, bigint_t p, gnutls_buffer_st* data,
                                    int psk)
 {
   bigint_t x, X;
-  size_t n_X, n_g, n_p;
-  int ret, data_size, pos;
-  uint8_t *pdata;
+  int ret;
 
   X = gnutls_calc_dh_secret (&x, g, p);
   if (X == NULL || x == NULL)
@@ -332,51 +315,39 @@ _gnutls_dh_common_print_server_kx (gnutls_session_t session,
   session->key->dh_secret = x;
   _gnutls_dh_set_secret_bits (session, _gnutls_mpi_get_nbits (x));
 
-  _gnutls_mpi_print (g, NULL, &n_g);
-  _gnutls_mpi_print (p, NULL, &n_p);
-  _gnutls_mpi_print (X, NULL, &n_X);
-
-  data_size = n_g + n_p + n_X + 6;
-  if (psk != 0)
-    data_size += 2;
-
-  (*data) = gnutls_malloc (data_size);
-  if (*data == NULL)
-    {
-      _gnutls_mpi_release (&X);
-      return GNUTLS_E_MEMORY_ERROR;
-    }
-
-  pos = 0;
-  pdata = *data;
-
   if (psk != 0)
     {
-      _gnutls_write_uint16 (0, &pdata[pos]);
-      pos += 2;
+      ret = _gnutls_buffer_append_prefix(data, 16, 0);
+      if (ret < 0)
+        {
+          ret = gnutls_assert_val(ret);
+          goto cleanup;
+        }
     }
 
-  _gnutls_mpi_print (p, &pdata[pos + 2], &n_p);
-  _gnutls_write_uint16 (n_p, &pdata[pos]);
+  ret = _gnutls_buffer_append_mpi(data, 16, p, 0);
+  if (ret < 0)
+    {
+      ret = gnutls_assert_val(ret);
+      goto cleanup;
+    }
 
-  pos += n_p + 2;
+  ret = _gnutls_buffer_append_mpi(data, 16, g, 0);
+  if (ret < 0)
+    {
+      ret = gnutls_assert_val(ret);
+      goto cleanup;
+    }
 
-  _gnutls_mpi_print (g, &pdata[pos + 2], &n_g);
-  _gnutls_write_uint16 (n_g, &pdata[pos]);
+  ret = _gnutls_buffer_append_mpi(data, 16, X, 0);
+  if (ret < 0)
+    {
+      ret = gnutls_assert_val(ret);
+      goto cleanup;
+    }
 
-  pos += n_g + 2;
-
-  _gnutls_mpi_print (X, &pdata[pos + 2], &n_X);
+cleanup:
   _gnutls_mpi_release (&X);
 
-  _gnutls_write_uint16 (n_X, &pdata[pos]);
-
-  /* do not use data_size. _gnutls_mpi_print() might
-   * have been pessimist and might have returned initially
-   * more data */
-  ret = n_g + n_p + n_X + 6;
-  if (psk != 0)
-    ret += 2;
-
-  return ret;
+  return data->length;
 }
