@@ -40,8 +40,8 @@
 #include <auth_dh_common.h>
 #include <gnutls_datum.h>
 
-static int gen_psk_server_kx (gnutls_session_t, opaque **);
-static int gen_psk_client_kx (gnutls_session_t, opaque **);
+static int gen_psk_server_kx (gnutls_session_t, gnutls_buffer_st*);
+static int gen_psk_client_kx (gnutls_session_t, gnutls_buffer_st*);
 static int proc_psk_client_kx (gnutls_session_t, opaque *, size_t);
 static int proc_psk_server_kx (gnutls_session_t, opaque *, size_t);
 
@@ -63,11 +63,9 @@ const mod_auth_st dhe_psk_auth_struct = {
 };
 
 static int
-gen_psk_client_kx (gnutls_session_t session, opaque ** data)
+gen_psk_client_kx (gnutls_session_t session, gnutls_buffer_st* data)
 {
   int ret, free;
-  opaque *tmp_data = NULL;
-  int data_size, tmp_data_size;
   gnutls_psk_client_credentials_t cred;
   gnutls_datum_t username, key;
 
@@ -77,37 +75,28 @@ gen_psk_client_kx (gnutls_session_t session, opaque ** data)
   if (cred == NULL)
     return gnutls_assert_val(GNUTLS_E_INSUFFICIENT_CREDENTIALS);
 
-
   ret = _gnutls_find_psk_key( session, cred, &username, &key, &free);
   if (ret < 0)
     return gnutls_assert_val(ret);
 
+  ret = _gnutls_buffer_append_data_prefix(data, 16, username.data, username.size);
+  if (ret < 0)
+    {
+      gnutls_assert();
+      goto cleanup;
+    }
+
   /* The PSK key is set in there */
-  ret = _gnutls_gen_dh_common_client_kx_int (session, &tmp_data, &key);
+  ret = _gnutls_gen_dh_common_client_kx_int (session, data, &key);
   if (ret < 0)
     {
       gnutls_assert ();
       goto cleanup;
     }
 
-  tmp_data_size = ret;
-  data_size = tmp_data_size + username.size + 2;
-
-  (*data) = gnutls_malloc (data_size);
-  if ((*data) == NULL)
-    {
-      gnutls_assert ();
-      ret = GNUTLS_E_MEMORY_ERROR;
-      goto cleanup;
-    }
-
-  _gnutls_write_datum16 (*data, username);
-  memcpy (&(*data)[username.size + 2], tmp_data, tmp_data_size);
-
-  ret = data_size;
+  ret = data->length;
 
 cleanup:
-  gnutls_free (tmp_data);
   if (free)
     {
       _gnutls_free_datum(&username);
@@ -119,7 +108,7 @@ cleanup:
 }
 
 static int
-gen_psk_server_kx (gnutls_session_t session, opaque ** data)
+gen_psk_server_kx (gnutls_session_t session, gnutls_buffer_st* data)
 {
   bigint_t g, p;
   const bigint_t *mpis;
