@@ -35,7 +35,6 @@
 #include <gnutls_mbuffers.h>
 #include <gnutls_extensions.h>
 #include <gnutls_constate.h>
-#include <system.h>
 
 #ifdef ENABLE_SESSION_TICKET
 
@@ -49,7 +48,7 @@
 static int session_ticket_recv_params (gnutls_session_t session,
                                        const opaque * data, size_t data_size);
 static int session_ticket_send_params (gnutls_session_t session,
-                                       opaque * data, size_t data_size);
+                                       gnutls_buffer_st* extdata);
 static int session_ticket_unpack (gnutls_buffer_st * ps,
                                   extension_priv_data_t * _priv);
 static int session_ticket_pack (extension_priv_data_t _priv,
@@ -68,12 +67,10 @@ extension_entry_st ext_mod_session_ticket = {
   .deinit_func = session_ticket_deinit_data,
 };
 
-struct gnutls_session_ticket_key_st
-{
-  opaque key_name[SESSION_TICKET_KEY_NAME_SIZE];
-  opaque key[SESSION_TICKET_KEY_SIZE];
-  opaque mac_secret[SESSION_TICKET_MAC_SECRET_SIZE];
-};
+#define SESSION_KEY_SIZE (SESSION_TICKET_KEY_NAME_SIZE+SESSION_TICKET_KEY_SIZE+SESSION_TICKET_MAC_SECRET_SIZE)
+#define NAME_POS (0)
+#define KEY_POS (SESSION_TICKET_KEY_NAME_SIZE)
+#define MAC_SECRET_POS (SESSION_TICKET_KEY_NAME_SIZE+SESSION_TICKET_KEY_SIZE)
 
 typedef struct
 {
@@ -84,7 +81,7 @@ typedef struct
   opaque *session_ticket;
   int session_ticket_len;
 
-  struct gnutls_session_ticket_key_st key;
+  opaque key[SESSION_KEY_SIZE];
 } session_ticket_ext_st;
 
 struct ticket
@@ -129,11 +126,11 @@ decrypt_ticket (gnutls_session_t session, session_ticket_ext_st * priv,
   cipher_hd_st cipher_hd;
   gnutls_datum_t key, IV, mac_secret, state;
   opaque final[MAC_SECRET_SIZE];
-  time_t timestamp = gnutls_time (0);
+  time_t timestamp = time (0);
   int ret;
 
   /* Check the integrity of ticket using HMAC-SHA-256. */
-  mac_secret.data = (void *) priv->key.mac_secret;
+  mac_secret.data = (void *) &priv->key[MAC_SECRET_POS];
   mac_secret.size = MAC_SECRET_SIZE;
   ret = digest_ticket (&mac_secret, ticket, final);
   if (ret < 0)
@@ -149,7 +146,7 @@ decrypt_ticket (gnutls_session_t session, session_ticket_ext_st * priv,
     }
 
   /* Decrypt encrypted_state using 128-bit AES in CBC mode. */
-  key.data = (void *) priv->key.key;
+  key.data = (void *) &priv->key[KEY_POS];
   key.size = KEY_SIZE;
   IV.data = ticket->IV;
   IV.size = IV_SIZE;
@@ -224,7 +221,7 @@ encrypt_ticket (gnutls_session_t session, session_ticket_ext_st * priv,
   _gnutls_free_datum (&state);
 
   /* Encrypt state using 128-bit AES in CBC mode. */
-  key.data = (void *) priv->key.key;
+  key.data = (void *) &priv->key[KEY_POS];
   key.size = KEY_SIZE;
   IV.data = priv->session_ticket_IV;
   IV.size = IV_SIZE;
@@ -248,12 +245,12 @@ encrypt_ticket (gnutls_session_t session, session_ticket_ext_st * priv,
     }
 
   /* Fill the ticket structure to compute MAC. */
-  memcpy (ticket->key_name, priv->key.key_name, KEY_NAME_SIZE);
+  memcpy (ticket->key_name, &priv->key[NAME_POS], KEY_NAME_SIZE);
   memcpy (ticket->IV, IV.data, IV.size);
   ticket->encrypted_state_len = encrypted_state.size;
   ticket->encrypted_state = encrypted_state.data;
 
-  mac_secret.data = priv->key.mac_secret;
+  mac_secret.data = &priv->key[MAC_SECRET_POS];
   mac_secret.size = MAC_SECRET_SIZE;
   ret = digest_ticket (&mac_secret, ticket, ticket->mac);
   if (ret < 0)
@@ -306,7 +303,7 @@ session_ticket_recv_params (gnutls_session_t session,
 
       /* If the key name of the ticket does not match the one that we
          hold, issue a new ticket. */
-      if (memcmp (ticket.key_name, priv->key.key_name, KEY_NAME_SIZE))
+      if (memcmp (ticket.key_name, &priv->key[NAME_POS], KEY_NAME_SIZE))
         {
           priv->session_ticket_renew = 1;
           return 0;
@@ -362,9 +359,8 @@ session_ticket_recv_params (gnutls_session_t session,
  */
 static int
 session_ticket_send_params (gnutls_session_t session,
-                            opaque * data, size_t _data_size)
+                            gnutls_buffer_st * extdata)
 {
-  ssize_t data_size = _data_size;
   session_ticket_ext_st *priv = NULL;
   extension_priv_data_t epriv;
   int ret;
@@ -404,9 +400,9 @@ session_ticket_send_params (gnutls_session_t session,
 
       if (priv->session_ticket_len > 0)
         {
-          DECR_LENGTH_RET (data_size, priv->session_ticket_len,
-                           GNUTLS_E_SHORT_MEMORY_BUFFER);
-          memcpy (data, priv->session_ticket, priv->session_ticket_len);
+          ret = _gnutls_buffer_append_data( extdata, priv->session_ticket, priv->session_ticket_len);
+          if (ret < 0)
+            return gnutls_assert_val(ret);
 
           return priv->session_ticket_len;
         }
@@ -442,7 +438,7 @@ session_ticket_unpack (gnutls_buffer_st * ps, extension_priv_data_t * _priv)
   session_ticket_ext_st *priv = NULL;
   int ret;
   extension_priv_data_t epriv;
-  gnutls_datum ticket;
+  gnutls_datum_t ticket;
 
   priv = gnutls_calloc (1, sizeof (*priv));
   if (priv == NULL)
@@ -484,25 +480,7 @@ error:
 int
 gnutls_session_ticket_key_generate (gnutls_datum_t * key)
 {
-  int ret;
-
-  key->size = sizeof (struct gnutls_session_ticket_key_st);
-  key->data = gnutls_malloc (key->size);
-  if (!key->data)
-    {
-      gnutls_assert ();
-      return GNUTLS_E_MEMORY_ERROR;
-    }
-
-  ret = _gnutls_rnd (GNUTLS_RND_RANDOM, key->data, key->size);
-  if (ret < 0)
-    {
-      gnutls_assert ();
-      _gnutls_free_datum (key);
-      return ret;
-    }
-
-  return 0;
+  return gnutls_key_generate(key, SESSION_KEY_SIZE);
 }
 
 /**
@@ -567,7 +545,7 @@ gnutls_session_ticket_enable_server (gnutls_session_t session,
   extension_priv_data_t epriv;
 
   if (!session || !key
-      || key->size != sizeof (struct gnutls_session_ticket_key_st))
+      || key->size != SESSION_KEY_SIZE)
     {
       gnutls_assert ();
       return GNUTLS_E_INVALID_REQUEST;
@@ -581,7 +559,7 @@ gnutls_session_ticket_enable_server (gnutls_session_t session,
     }
   epriv.ptr = priv;
 
-  ret = _gnutls_rnd (GNUTLS_RND_RANDOM, priv->session_ticket_IV, IV_SIZE);
+  ret = _gnutls_rnd (GNUTLS_RND_NONCE, priv->session_ticket_IV, IV_SIZE);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -650,8 +628,7 @@ _gnutls_send_new_session_ticket (gnutls_session_t session, int again)
       ticket_len = KEY_NAME_SIZE + IV_SIZE + 2 + ticket.encrypted_state_len
         + MAC_SIZE;
 
-      bufel =
-        _gnutls_handshake_alloc (4 + 2 + ticket_len, 4 + 2 + ticket_len);
+      bufel = _gnutls_handshake_alloc (session, 4 + 2 + ticket_len, 4+2+ticket_len);
       if (!bufel)
         {
           gnutls_assert ();
@@ -686,17 +663,17 @@ _gnutls_send_new_session_ticket (gnutls_session_t session, int again)
 
       data_size = p - data;
     }
-  ret = _gnutls_send_handshake (session, data_size ? bufel : NULL,
+  return _gnutls_send_handshake (session, data_size ? bufel : NULL,
                                 GNUTLS_HANDSHAKE_NEW_SESSION_TICKET);
-
-  return ret;
 }
 
 int
 _gnutls_recv_new_session_ticket (gnutls_session_t session)
 {
-  uint8_t *data = NULL, *p;
+  uint8_t *p;
   int data_size;
+  gnutls_buffer_st buf;
+  uint32_t lifetime_hint;
   uint16_t ticket_len;
   int ret;
   session_ticket_ext_st *priv = NULL;
@@ -715,34 +692,32 @@ _gnutls_recv_new_session_ticket (gnutls_session_t session)
   if (!priv->session_ticket_renew)
     return 0;
 
-  ret = _gnutls_recv_handshake (session, &data, &data_size,
+  ret = _gnutls_recv_handshake (session, 
                                 GNUTLS_HANDSHAKE_NEW_SESSION_TICKET,
-                                MANDATORY_PACKET);
+                                MANDATORY_PACKET, &buf);
   if (ret < 0)
-    {
-      gnutls_assert ();
-      return ret;
-    }
+    return gnutls_assert_val_fatal(ret);
 
-  p = data;
-  DECR_LENGTH_COM (data_size, 4, goto error);
-  _gnutls_read_uint32 (p);
+  p = buf.data;
+  data_size = buf.length;
+
+  DECR_LENGTH_COM (data_size, 4, ret = GNUTLS_E_UNEXPECTED_PACKET_LENGTH; goto error);
+  lifetime_hint = _gnutls_read_uint32 (p);
   p += 4;
 
-  DECR_LENGTH_COM (data_size, 2, goto error);
+  DECR_LENGTH_COM (data_size, 2, ret = GNUTLS_E_UNEXPECTED_PACKET_LENGTH; goto error);
   ticket_len = _gnutls_read_uint16 (p);
   p += 2;
 
-  DECR_LENGTH_COM (data_size, ticket_len, goto error);
+  DECR_LENGTH_COM (data_size, ticket_len, ret = GNUTLS_E_UNEXPECTED_PACKET_LENGTH; goto error);
   priv->session_ticket = gnutls_realloc (priv->session_ticket, ticket_len);
   if (!priv->session_ticket)
     {
       gnutls_assert ();
-      gnutls_free (data);
-      return GNUTLS_E_MEMORY_ERROR;
+      ret = GNUTLS_E_MEMORY_ERROR;
+      goto error;
     }
   memcpy (priv->session_ticket, p, ticket_len);
-  gnutls_free (data);
   priv->session_ticket_len = ticket_len;
 
   /* Discard the current session ID.  (RFC5077 3.4) */
@@ -754,13 +729,15 @@ _gnutls_recv_new_session_ticket (gnutls_session_t session)
       gnutls_assert ();
       gnutls_free (priv->session_ticket);
       priv->session_ticket = NULL;
-      return GNUTLS_E_INTERNAL_ERROR;
+      ret = GNUTLS_E_INTERNAL_ERROR;
+      goto error;
     }
-  return 0;
+  ret = 0;
 
 error:
-  gnutls_free (data);
-  return GNUTLS_E_UNEXPECTED_PACKET_LENGTH;
+  _gnutls_buffer_clear (&buf);
+  
+  return ret;
 }
 
 #endif
