@@ -86,37 +86,6 @@ gnutls_certificate_free_keys (gnutls_certificate_credentials_t sc)
 
 }
 
-/**
- * gnutls_certificate_get_issuer:
- * @sc: is a #gnutls_certificate_credentials_t structure.
- * @cert: is the certificate to find issuer for
- * @issuer: Will hold the issuer if any. Should be treated as constant.
- * @flags: Use zero.
- *
- * This function will return the issuer of a given certificate.
- *
- * Returns: On success, %GNUTLS_E_SUCCESS is returned, otherwise a
- *   negative error value.
- **/
-int
-gnutls_certificate_get_issuer (gnutls_certificate_credentials_t sc,
-  gnutls_x509_crt_t cert, gnutls_x509_crt_t* issuer, unsigned int flags)
-{
-int i, ret;
-
-  for (i=0;i<sc->x509_ncas;i++)
-    {
-      ret = gnutls_x509_crt_check_issuer(cert, sc->x509_ca_list[i]);
-      if (ret > 0)
-        {
-          *issuer = sc->x509_ca_list[i];
-          return 0;
-        }
-    }
-  
-  return GNUTLS_E_REQUESTED_DATA_NOT_AVAILABLE;
-}
-
 /**
  * gnutls_certificate_free_cas:
  * @sc: is a #gnutls_certificate_credentials_t structure.
@@ -129,81 +98,10 @@ int i, ret;
 void
 gnutls_certificate_free_cas (gnutls_certificate_credentials_t sc)
 {
-  unsigned j;
-
-  for (j = 0; j < sc->x509_ncas; j++)
-    {
-      gnutls_x509_crt_deinit (sc->x509_ca_list[j]);
-    }
-
-  sc->x509_ncas = 0;
-
-  gnutls_free (sc->x509_ca_list);
-  sc->x509_ca_list = NULL;
-
+  /* FIXME: do nothing for now */
+  return;
 }
 
-/**
- * gnutls_certificate_get_x509_cas:
- * @sc: is a #gnutls_certificate_credentials_t structure.
- * @x509_ca_list: will point to the CA list. Should be treated as constant
- * @ncas: the number of CAs
- *
- * This function will export all the CAs associated with the given
- * credentials.
- *
- * Since: 2.4.0
- **/
-void
-gnutls_certificate_get_x509_cas (gnutls_certificate_credentials_t sc,
-                                 gnutls_x509_crt_t ** x509_ca_list,
-                                 unsigned int *ncas)
-{
-  *x509_ca_list = sc->x509_ca_list;
-  *ncas = sc->x509_ncas;
-}
-
-/**
- * gnutls_certificate_get_x509_crls:
- * @sc: is a #gnutls_certificate_credentials_t structure.
- * @x509_crl_list: the exported CRL list. Should be treated as constant
- * @ncrls: the number of exported CRLs
- *
- * This function will export all the CRLs associated with the given
- * credentials.
- *
- * Since: 2.4.0
- **/
-void
-gnutls_certificate_get_x509_crls (gnutls_certificate_credentials_t sc,
-                                  gnutls_x509_crl_t ** x509_crl_list,
-                                  unsigned int *ncrls)
-{
-  *x509_crl_list = sc->x509_crl_list;
-  *ncrls = sc->x509_ncrls;
-}
-
-#ifdef ENABLE_OPENPGP
-
-/**
- * gnutls_certificate_get_openpgp_keyring:
- * @sc: is a #gnutls_certificate_credentials_t structure.
- * @keyring: the exported keyring. Should be treated as constant
- *
- * This function will export the OpenPGP keyring associated with the
- * given credentials.
- *
- * Since: 2.4.0
- **/
-void
-gnutls_certificate_get_openpgp_keyring (gnutls_certificate_credentials_t sc,
-                                        gnutls_openpgp_keyring_t * keyring)
-{
-  *keyring = sc->keyring;
-}
-
-#endif
-
 /**
  * gnutls_certificate_free_ca_names:
  * @sc: is a #gnutls_certificate_credentials_t structure.
@@ -277,12 +175,9 @@ _gnutls_certificate_get_rsa_params (gnutls_rsa_params_t rsa_params,
 void
 gnutls_certificate_free_credentials (gnutls_certificate_credentials_t sc)
 {
+  gnutls_x509_trust_list_deinit(sc->tlist, 1);
   gnutls_certificate_free_keys (sc);
-  gnutls_certificate_free_cas (sc);
   gnutls_certificate_free_ca_names (sc);
-#ifdef ENABLE_PKI
-  gnutls_certificate_free_crls (sc);
-#endif
 
 #ifdef ENABLE_OPENPGP
   gnutls_openpgp_keyring_deinit (sc->keyring);
@@ -305,11 +200,20 @@ int
 gnutls_certificate_allocate_credentials (gnutls_certificate_credentials_t *
                                          res)
 {
+int ret;
+
   *res = gnutls_calloc (1, sizeof (certificate_credentials_st));
 
   if (*res == NULL)
     return GNUTLS_E_MEMORY_ERROR;
 
+  ret = gnutls_x509_trust_list_init( &(*res)->tlist, 0);
+  if (ret < 0)
+    {
+      gnutls_assert();
+      gnutls_free(*res);
+      return GNUTLS_E_MEMORY_ERROR;
+    }
   (*res)->verify_bits = DEFAULT_VERIFY_BITS;
   (*res)->verify_depth = DEFAULT_VERIFY_DEPTH;
 
@@ -712,39 +616,6 @@ gnutls_certificate_verify_peers2 (gnutls_session_t session,
     }
 }
 
-/**
- * gnutls_certificate_verify_peers:
- * @session: is a gnutls session
- *
- * This function will try to verify the peer's certificate and return
- * its status (trusted, invalid etc.).  However you must also check
- * the peer's name in order to check if the verified certificate
- * belongs to the actual peer.
- *
- * This function uses gnutls_x509_crt_list_verify().
- *
- * Returns: one or more of the #gnutls_certificate_status_t
- * enumerated elements bitwise or'd, or a negative value on error.
- *
- * Deprecated: Use gnutls_certificate_verify_peers2() instead.
- **/
-int
-gnutls_certificate_verify_peers (gnutls_session_t session)
-{
-  unsigned int status;
-  int ret;
-
-  ret = gnutls_certificate_verify_peers2 (session, &status);
-
-  if (ret < 0)
-    {
-      gnutls_assert ();
-      return ret;
-    }
-
-  return status;
-}
-
 /**
  * gnutls_certificate_expiration_time_peers:
  * @session: is a gnutls session
