@@ -48,7 +48,7 @@ _gnutls_m_plaintext2compressed (gnutls_session_t session,
   size =
     _gnutls_compress (params->write.compression_state,
                       plaintext->data, plaintext->size, &data,
-                      MAX_RECORD_SEND_SIZE + EXTRA_COMP_SIZE);
+                      MAX_RECORD_SEND_SIZE(session) + EXTRA_COMP_SIZE);
   if (size < 0)
     {
       gnutls_assert ();
@@ -72,7 +72,7 @@ _gnutls_m_compressed2plaintext (gnutls_session_t session,
   size =
     _gnutls_decompress (params->read.compression_state,
                         compressed->data, compressed->size, &data,
-                        MAX_RECORD_RECV_SIZE);
+                        MAX_RECORD_RECV_SIZE(session));
   if (size < 0)
     {
       gnutls_assert ();
@@ -421,10 +421,12 @@ _gnutls_comp_deinit (comp_hd_t handle, int d)
 #ifdef HAVE_LIBZ
         case GNUTLS_COMP_DEFLATE:
           {
+            int err;
+
             if (d)
-              inflateEnd (handle->handle);
+              err = inflateEnd (handle->handle);
             else
-              deflateEnd (handle->handle);
+              err = deflateEnd (handle->handle);
             break;
           }
 #endif
