@@ -291,19 +291,25 @@ _gnutls_set_keys (gnutls_session_t session, record_parameters_st * params,
 }
 
 static int
-_gnutls_init_record_state (record_parameters_st * params, int read,
+_gnutls_init_record_state (record_parameters_st * params, gnutls_protocol_t ver, int d,
                            record_state_st * state)
 {
   int ret;
+  gnutls_datum_t * iv = NULL;
 
-  ret = _gnutls_cipher_init (&state->cipher_state,
-                             params->cipher_algorithm,
-                             &state->key, &state->IV);
+  if (!_gnutls_version_has_explicit_iv(ver))
+    {
+      iv = &state->IV;
+    }
+
+  ret = _gnutls_auth_cipher_init (&state->cipher_state,
+    params->cipher_algorithm, &state->key, iv,
+    params->mac_algorithm, &state->mac_secret, (ver==GNUTLS_SSL3)?1:0);
   if (ret < 0 && params->cipher_algorithm != GNUTLS_CIPHER_NULL)
     return gnutls_assert_val (ret);
 
   state->compression_state =
-    _gnutls_comp_init (params->compression_algorithm, read);
+    _gnutls_comp_init (params->compression_algorithm, d);
 
   if (state->compression_state == GNUTLS_COMP_FAILED)
     return gnutls_assert_val (GNUTLS_E_UNKNOWN_COMPRESSION_ALGORITHM);
@@ -396,6 +402,7 @@ _gnutls_epoch_set_keys (gnutls_session_t session, uint16_t epoch)
   gnutls_compression_method_t comp_algo;
   record_parameters_st *params;
   int ret;
+  gnutls_protocol_t ver = gnutls_protocol_get_version (session);
 
   ret = _gnutls_epoch_get (session, epoch, &params);
   if (ret < 0)
@@ -428,11 +435,11 @@ _gnutls_epoch_set_keys (gnutls_session_t session, uint16_t epoch)
   if (ret < 0)
     return gnutls_assert_val (ret);
 
-  ret = _gnutls_init_record_state (params, 1, &params->read);
+  ret = _gnutls_init_record_state (params, ver, 1, &params->read);
   if (ret < 0)
     return gnutls_assert_val (ret);
 
-  ret = _gnutls_init_record_state (params, 0, &params->write);
+  ret = _gnutls_init_record_state (params, ver, 0, &params->write);
   if (ret < 0)
     return gnutls_assert_val (ret);
 
@@ -519,6 +526,18 @@ _gnutls_check_algos (gnutls_session_t session,
   return 0;
 }
 
+int _gnutls_epoch_get_compression(gnutls_session_t session, int epoch)
+{
+record_parameters_st *params;
+int ret;
+
+  ret = _gnutls_epoch_get (session, epoch, &params);
+  if (ret < 0)
+    return GNUTLS_COMP_UNKNOWN;
+
+  return params->compression_algorithm;
+}
+
 /* Initializes the read connection session
  * (read encrypted data)
  */
@@ -533,10 +552,11 @@ _gnutls_read_connection_state_init (gnutls_session_t session)
    */
   if (session->internals.resumed == RESUME_FALSE)
     {
+
       ret = _gnutls_check_algos (session,
                                  &session->
                                  security_parameters.current_cipher_suite,
-                                 session->internals.compression_method);
+                                 _gnutls_epoch_get_compression(session, epoch_next));
       if (ret < 0)
         return ret;
 
@@ -561,7 +581,6 @@ _gnutls_read_connection_state_init (gnutls_session_t session)
                           security_parameters.current_cipher_suite));
 
   session->security_parameters.epoch_read = epoch_next;
-  _gnutls_epoch_gc (session);
 
   return 0;
 }
@@ -585,7 +604,7 @@ _gnutls_write_connection_state_init (gnutls_session_t session)
       ret = _gnutls_check_algos (session,
                                  &session->
                                  security_parameters.current_cipher_suite,
-                                 session->internals.compression_method);
+                                 _gnutls_epoch_get_compression(session, epoch_next));
       if (ret < 0)
         return ret;
 
@@ -612,7 +631,6 @@ _gnutls_write_connection_state_init (gnutls_session_t session)
     ("HSK[%p]: Initializing internal [write] cipher sessions\n", session);
 
   session->security_parameters.epoch_write = epoch_next;
-  _gnutls_epoch_gc (session);
 
   return 0;
 }
@@ -670,10 +688,10 @@ epoch_get_slot (gnutls_session_t session, uint16_t epoch)
 
   if (epoch_index >= MAX_EPOCH_INDEX)
     {
+      _gnutls_handshake_log("Epoch %d out of range (idx: %d, max: %d)\n", (int)epoch, (int)epoch_index, MAX_EPOCH_INDEX);
       gnutls_assert ();
       return NULL;
     }
-
   /* The slot may still be empty (NULL) */
   return &session->record_parameters[epoch_index];
 }
@@ -725,6 +743,9 @@ _gnutls_epoch_alloc (gnutls_session_t session, uint16_t epoch,
   (*slot)->mac_algorithm = GNUTLS_MAC_UNKNOWN;
   (*slot)->compression_algorithm = GNUTLS_COMP_UNKNOWN;
 
+  if (IS_DTLS (session))
+    _gnutls_write_uint16 (epoch, UINT64DATA((*slot)->write.sequence_number));
+
   if (out != NULL)
     *out = *slot;
 
@@ -737,9 +758,19 @@ epoch_alive (gnutls_session_t session, record_parameters_st * params)
   const security_parameters_st *sp = &session->security_parameters;
 
   /* DTLS will, in addition, need to check the epoch timeout value. */
-  return (params->epoch == sp->epoch_read
-          || params->epoch == sp->epoch_write
-          || params->epoch == sp->epoch_next);
+  if (params->usage_cnt > 0)
+    return 1;
+
+  if (params->epoch == sp->epoch_read)
+    return 1;
+  
+  if (params->epoch == sp->epoch_write)
+    return 1;
+  
+  if (params->epoch == sp->epoch_next)
+    return 1;
+
+  return 0;
 }
 
 void
@@ -777,16 +808,16 @@ _gnutls_epoch_gc (gnutls_session_t session)
 }
 
 static inline void
-free_record_state (record_state_st * state, int read)
+free_record_state (record_state_st * state, int d)
 {
   _gnutls_free_datum (&state->mac_secret);
   _gnutls_free_datum (&state->IV);
   _gnutls_free_datum (&state->key);
 
-  _gnutls_cipher_deinit (&state->cipher_state);
+  _gnutls_auth_cipher_deinit (&state->cipher_state);
 
   if (state->compression_state != NULL)
-    _gnutls_comp_deinit (state->compression_state, read);
+    _gnutls_comp_deinit (state->compression_state, d);
 }
 
 void
