@@ -74,6 +74,8 @@ static const gnutls_error_entry error_algorithms[] = {
                GNUTLS_E_DH_PRIME_UNACCEPTABLE, 1),
   ERROR_ENTRY (N_("A TLS packet with unexpected length was received."),
                GNUTLS_E_UNEXPECTED_PACKET_LENGTH, 1),
+  ERROR_ENTRY (N_("The TLS connection was non-properly terminated."),
+               GNUTLS_E_PREMATURE_TERMINATION, 1),
   ERROR_ENTRY (N_
                ("The specified session has been invalidated for some reason."),
                GNUTLS_E_INVALID_SESSION, 1),
@@ -173,6 +175,7 @@ static const gnutls_error_entry error_algorithms[] = {
   ERROR_ENTRY (N_("The given memory buffer is too short to hold parameters."),
                GNUTLS_E_SHORT_MEMORY_BUFFER, 1),
   ERROR_ENTRY (N_("The request is invalid."), GNUTLS_E_INVALID_REQUEST, 1),
+  ERROR_ENTRY (N_("The cookie was bad."), GNUTLS_E_BAD_COOKIE, 1),
   ERROR_ENTRY (N_("An illegal parameter has been received."),
                GNUTLS_E_RECEIVED_ILLEGAL_PARAMETER, 1),
   ERROR_ENTRY (N_("Error while reading file."), GNUTLS_E_FILE_ERROR, 1),
@@ -201,6 +204,8 @@ static const gnutls_error_entry error_algorithms[] = {
 
   ERROR_ENTRY (N_("Too many empty record packets have been received."),
                GNUTLS_E_TOO_MANY_EMPTY_PACKETS, 1),
+  ERROR_ENTRY (N_("Too many handshake packets have been received."),
+               GNUTLS_E_TOO_MANY_HANDSHAKE_PACKETS, 1),
   ERROR_ENTRY (N_("The initialization of GnuTLS-extra has failed."),
                GNUTLS_E_INIT_LIBEXTRA, 1),
   ERROR_ENTRY (N_
@@ -321,8 +326,8 @@ static const gnutls_error_entry error_algorithms[] = {
                GNUTLS_E_PKCS11_TOKEN_ERROR, 1),
   ERROR_ENTRY (N_("PKCS #11 user error"),
                GNUTLS_E_PKCS11_USER_ERROR, 1),
-  ERROR_ENTRY (N_("The provided X.509 certificate list is not sorted (in subject to issuer order)"),
-               GNUTLS_E_CERTIFICATE_LIST_UNSORTED, 1),
+  ERROR_ENTRY (N_("The operation timed out"),
+               GNUTLS_E_TIMEDOUT, 1),
   {NULL, NULL, 0, 0}
 };
 
@@ -491,13 +496,11 @@ _gnutls_mpi_log (const char *prefix, bigint_t a)
   char *hexbuf;
   int res;
 
-  if (_gnutls_log_level < 2) return;
-
   res = _gnutls_mpi_print (a, NULL, &binlen);
-  if (res < 0 && res != GNUTLS_E_SHORT_MEMORY_BUFFER)
+  if (res != 0)
     {
       gnutls_assert ();
-      _gnutls_hard_log ("MPI: %s can't print value (%d/%d)\n", prefix, res,
+      _gnutls_hard_log ("MPI: can't print value (%d/%d)\n", res,
                         (int) binlen);
       return;
     }
@@ -505,7 +508,7 @@ _gnutls_mpi_log (const char *prefix, bigint_t a)
   if (binlen > 1024 * 1024)
     {
       gnutls_assert ();
-      _gnutls_hard_log ("MPI: %s too large mpi (%d)\n", prefix, (int) binlen);
+      _gnutls_hard_log ("MPI: too large mpi (%d)\n", (int) binlen);
       return;
     }
 
@@ -513,7 +516,7 @@ _gnutls_mpi_log (const char *prefix, bigint_t a)
   if (!binbuf)
     {
       gnutls_assert ();
-      _gnutls_hard_log ("MPI: %s out of memory (%d)\n", prefix, (int) binlen);
+      _gnutls_hard_log ("MPI: out of memory (%d)\n", (int) binlen);
       return;
     }
 
@@ -521,7 +524,7 @@ _gnutls_mpi_log (const char *prefix, bigint_t a)
   if (res != 0)
     {
       gnutls_assert ();
-      _gnutls_hard_log ("MPI: %s can't print value (%d/%d)\n", prefix, res,
+      _gnutls_hard_log ("MPI: can't print value (%d/%d)\n", res,
                         (int) binlen);
       gnutls_free (binbuf);
       return;
@@ -533,7 +536,7 @@ _gnutls_mpi_log (const char *prefix, bigint_t a)
   if (!hexbuf)
     {
       gnutls_assert ();
-      _gnutls_hard_log ("MPI: %s out of memory (hex %d)\n", prefix, (int) hexlen);
+      _gnutls_hard_log ("MPI: out of memory (hex %d)\n", (int) hexlen);
       gnutls_free (binbuf);
       return;
     }
