@@ -48,6 +48,7 @@ typedef int ssize_t;
 #include <sys/stat.h>
 #include <sys/socket.h>
 #include <time.h>
+#include <u64.h> /* gnulib for uint64_t */
 
 /* some systems had problems with long long int, thus,
  * it is not used.
@@ -58,7 +59,6 @@ typedef struct
 } uint64;
 
 #include <gnutls/gnutls.h>
-#include <system.h>
 
 /*
  * They are not needed any more. You can simply enable
@@ -93,33 +93,81 @@ typedef struct
 #define SESSION_TICKET_IV_SIZE 16
 #define SESSION_TICKET_MAC_SECRET_SIZE 32
 
+#define AEAD_EXPLICIT_DATA_SIZE 8
+#define AEAD_IMPLICIT_DATA_SIZE 4
+
+#define GNUTLS_MASTER_SIZE 48
+#define GNUTLS_RANDOM_SIZE 32
+
+/* TLS Extensions */
 /* we can receive up to MAX_EXT_TYPES extensions.
  */
 #define MAX_EXT_TYPES 32
 
-/* The initial size of the receive
- * buffer size. This will grow if larger
- * packets are received.
- */
-#define INITIAL_RECV_BUFFER_SIZE 256
+  /**
+   * gnutls_ext_parse_type_t:
+   * @GNUTLS_EXT_NONE: Never parsed
+   * @GNUTLS_EXT_ANY: Any extension type.
+   * @GNUTLS_EXT_APPLICATION: Application extension.
+   * @GNUTLS_EXT_TLS: TLS-internal extension.
+   * @GNUTLS_EXT_MANDATORY: Extension parsed even if resuming (or extensions are disabled).
+   *
+   * Enumeration of different TLS extension types.  This flag
+   * indicates for an extension whether it is useful to application
+   * level or TLS level only.  This is (only) used to parse the
+   * application level extensions before the "client_hello" callback
+   * is called.
+   */
+  typedef enum
+  {
+    GNUTLS_EXT_ANY = 0,
+    GNUTLS_EXT_APPLICATION = 1,
+    GNUTLS_EXT_TLS = 2,
+    GNUTLS_EXT_MANDATORY = 3,
+    GNUTLS_EXT_NONE = 4
+  } gnutls_ext_parse_type_t;
 
-/* the default for TCP */
-#define DEFAULT_LOWAT 0
 
 /* expire time for resuming sessions */
 #define DEFAULT_EXPIRE_TIME 3600
 
+typedef enum transport_t
+{
+  GNUTLS_STREAM,
+  GNUTLS_DGRAM
+} transport_t;
+
 /* the maximum size of encrypted packets */
+#define IS_DTLS(session) (session->internals.transport == GNUTLS_DGRAM)
+
 #define DEFAULT_MAX_RECORD_SIZE 16384
-#define RECORD_HEADER_SIZE 5
-#define MAX_RECORD_SEND_SIZE (size_t)session->security_parameters.max_record_send_size
-#define MAX_RECORD_RECV_SIZE (size_t)session->security_parameters.max_record_recv_size
+#define TLS_RECORD_HEADER_SIZE 5
+#define DTLS_RECORD_HEADER_SIZE (TLS_RECORD_HEADER_SIZE+8)
+#define RECORD_HEADER_SIZE(session) (IS_DTLS(session) ? DTLS_RECORD_HEADER_SIZE : TLS_RECORD_HEADER_SIZE)
+#define MAX_RECORD_HEADER_SIZE DTLS_RECORD_HEADER_SIZE
+
+#define MAX_RECORD_SEND_SIZE(session) (IS_DTLS(session)?((size_t)session->internals.dtls.mtu-DTLS_RECORD_HEADER_SIZE):(size_t)session->security_parameters.max_record_send_size)
+#define MAX_RECORD_RECV_SIZE(session) ((size_t)session->security_parameters.max_record_recv_size)
 #define MAX_PAD_SIZE 255
 #define EXTRA_COMP_SIZE 2048
-#define MAX_RECORD_OVERHEAD (MAX_CIPHER_BLOCK_SIZE/*iv*/+MAX_PAD_SIZE+EXTRA_COMP_SIZE)
-#define MAX_RECV_SIZE (MAX_RECORD_OVERHEAD+MAX_RECORD_RECV_SIZE+RECORD_HEADER_SIZE)
+#define MAX_RECORD_OVERHEAD (MAX_CIPHER_BLOCK_SIZE/*iv*/+MAX_PAD_SIZE+EXTRA_COMP_SIZE+MAX_HASH_SIZE/*MAC*/)
+#define MAX_RECV_SIZE(session) (MAX_RECORD_OVERHEAD+MAX_RECORD_RECV_SIZE(session)+RECORD_HEADER_SIZE(session))
 
-#define HANDSHAKE_HEADER_SIZE 4
+#define TLS_HANDSHAKE_HEADER_SIZE 4
+#define DTLS_HANDSHAKE_HEADER_SIZE (TLS_HANDSHAKE_HEADER_SIZE+8)
+#define HANDSHAKE_HEADER_SIZE(session) (IS_DTLS(session) ? DTLS_HANDSHAKE_HEADER_SIZE : TLS_HANDSHAKE_HEADER_SIZE)
+#define MAX_HANDSHAKE_HEADER_SIZE DTLS_HANDSHAKE_HEADER_SIZE
+
+/* This is the maximum handshake message size we send without
+   fragmentation. This currently ignores record layer overhead. */
+#define DTLS_DEFAULT_MTU 1200
+
+/* the maximum size of the DTLS cookie */
+#define DTLS_MAX_COOKIE_SIZE 32
+
+/* The maximum number of HELLO_VERIFY_REQUEST messages the client
+   processes before aborting. */
+#define MAX_HANDSHAKE_HELLO_VERIFY_REQUESTS 5
 
 /* defaults for verification functions
  */
@@ -147,14 +195,11 @@ typedef struct
 
 #include <gnutls_mpi.h>
 
-typedef enum change_cipher_spec_t
-{ GNUTLS_TYPE_CHANGE_CIPHER_SPEC = 1
-} change_cipher_spec_t;
-
 typedef enum handshake_state_t
 { STATE0 = 0, STATE1, STATE2,
   STATE3, STATE4, STATE5,
-  STATE6, STATE7, STATE8, STATE9, STATE20 = 20, STATE21, STATE22,
+  STATE6, STATE7, STATE8, STATE9, STATE11 = 11,
+  STATE20 = 20, STATE21, STATE22,
   STATE30 = 30, STATE31, STATE40 = 40, STATE41, STATE50 = 50,
   STATE60 = 60, STATE61, STATE62, STATE70, STATE71
 } handshake_state_t;
@@ -164,9 +209,7 @@ typedef enum handshake_state_t
 /* This is the maximum number of algorithms (ciphers or macs etc).
  * keep it synced with GNUTLS_MAX_ALGORITHM_NUM in gnutls.h
  */
-#define MAX_ALGOS 16
-
-#define MAX_CIPHERSUITES 256
+#define MAX_ALGOS GNUTLS_MAX_ALGORITHM_NUM
 
 typedef enum extensions_t
 {
@@ -176,7 +219,6 @@ typedef enum extensions_t
   GNUTLS_EXTENSION_SRP = 12,
   GNUTLS_EXTENSION_SIGNATURE_ALGORITHMS = 13,
   GNUTLS_EXTENSION_SESSION_TICKET = 35,
-  GNUTLS_EXTENSION_INNER_APPLICATION = 37703,
   GNUTLS_EXTENSION_SAFE_RENEGOTIATION = 65281   /* aka: 0xff01 */
 } extensions_t;
 
@@ -191,31 +233,85 @@ typedef enum content_type_t
 {
   GNUTLS_CHANGE_CIPHER_SPEC = 20, GNUTLS_ALERT,
   GNUTLS_HANDSHAKE, GNUTLS_APPLICATION_DATA,
-  GNUTLS_INNER_APPLICATION = 24
 } content_type_t;
 
+
 #define GNUTLS_PK_ANY (gnutls_pk_algorithm_t)-1
 #define GNUTLS_PK_NONE (gnutls_pk_algorithm_t)-2
 
 /* Message buffers (mbuffers) structures */
 
+/* this is actually the maximum number of distinct handshake
+ * messages that can arrive in a single flight
+ */
+#define MAX_HANDSHAKE_MSGS 6
+typedef struct
+{
+  /* Handshake layer type and sequence of message */
+  gnutls_handshake_description_t htype;
+  uint32_t length;
+
+  /* valid in DTLS */
+  uint16_t sequence;
+
+  /* indicate whether that message is complete.
+   * complete means start_offset == 0 and end_offset == length
+   */
+  uint32_t start_offset;
+  uint32_t end_offset;
+  
+  opaque header[MAX_HANDSHAKE_HEADER_SIZE];
+  int header_size;
+
+  gnutls_buffer_st data;
+} handshake_buffer_st;
+
 typedef struct mbuffer_st
 {
+  /* when used in mbuffer_head_st */
   struct mbuffer_st *next;
+  struct mbuffer_st *prev;
 
-  gnutls_datum_t msg;
   /* msg->size - mark = number of bytes left to process in this
      message. Mark should only be non-zero when this buffer is the
      head of the queue. */
   size_t mark;
-  unsigned int user_mark;       /* only used during fill in */
+
+
+  /* the data */
+  gnutls_datum_t msg;
   size_t maximum_size;
+
+  /* used during fill in, to separate header from data
+   * body. */
+  unsigned int user_mark;
+
+  /* Filled in by record layer on recv:
+   * type, record_sequence
+   */
+
+  /* record layer content type */
+  content_type_t type;
+
+  /* record layer sequence */
+  uint64 record_sequence;
+
+  /* Filled in by handshake layer on send:
+   * type, epoch, htype, handshake_sequence
+   */
+
+  /* Record layer epoch of message */
+  uint16_t epoch;
+
+  /* Handshake layer type and sequence of message */
+  gnutls_handshake_description_t htype;
+  uint16_t handshake_sequence;
 } mbuffer_st;
 
 typedef struct mbuffer_head_st
 {
   mbuffer_st *head;
-  mbuffer_st **tail;
+  mbuffer_st *tail;
 
   unsigned int length;
   size_t byte_length;
@@ -317,6 +413,7 @@ typedef struct
  */
 
 #define MAX_SIGNATURE_ALGORITHMS 16
+#define MAX_SIGN_ALGO_SIZE (2 + MAX_SIGNATURE_ALGORITHMS * 2)
 
 #define MAX_VERIFY_DATA_SIZE 36 /* in SSL 3.0, 12 in TLS 1.0 */
 
@@ -340,7 +437,7 @@ typedef struct
  */
 typedef struct
 {
-  gnutls_connection_end_t entity;
+  int entity; /* GNUTLS_SERVER or GNUTLS_CLIENT */
   gnutls_kx_algorithm_t kx_algorithm;
   handshake_mac_type_t handshake_mac_handle_type;       /* one of HANDSHAKE_TYPE_10 and HANDSHAKE_TYPE_12 */
 
@@ -390,7 +487,7 @@ struct record_state_st
   gnutls_datum_t mac_secret;
   gnutls_datum_t IV;
   gnutls_datum_t key;
-  cipher_hd_st cipher_state;
+  auth_cipher_hd_st cipher_state;
   comp_hd_t compression_state;
   uint64 sequence_number;
 };
@@ -414,6 +511,12 @@ struct record_parameters_st
 
   record_state_st read;
   record_state_st write;
+  
+  /* Whether this state is in use, i.e., if there is
+     a pending handshake message waiting to be encrypted
+     under this epoch's parameters.
+   */
+  int usage_cnt;
 };
 
 typedef struct
@@ -467,17 +570,51 @@ typedef struct
   int free_rsa_params;
 } internal_params_st;
 
+#define DTLS_RECORD_WINDOW_SIZE 64
 
-
+/* DTLS session state
+ */
 typedef struct
 {
-  opaque header[HANDSHAKE_HEADER_SIZE];
-  /* this holds the number of bytes in the handshake_header[] */
-  size_t header_size;
-  /* this holds the length of the handshake packet */
-  size_t packet_length;
-  gnutls_handshake_description_t recv_type;
-} handshake_header_buffer_st;
+  /* HelloVerifyRequest DOS prevention cookie */
+  opaque  cookie[DTLS_MAX_COOKIE_SIZE];
+  uint8_t cookie_len;
+
+  /* For DTLS handshake fragmentation and reassembly. */
+  uint16_t hsk_write_seq;
+  unsigned int hsk_read_seq;
+  uint16_t mtu;
+
+  /* a flight transmission is in process */
+  unsigned int flight_init:1;
+  /* whether this is the last flight in the protocol  */
+  unsigned int last_flight:1;
+  unsigned int retrans_timeout;
+  unsigned int total_timeout;
+
+  unsigned int hsk_hello_verify_requests;
+  
+  uint64_t record_sw[DTLS_RECORD_WINDOW_SIZE];
+  unsigned int record_sw_size;
+
+  /* non blocking stuff variables */
+  unsigned int blocking:1;
+  /* starting time of current handshake */
+  time_t handshake_start_time;
+  /* time in seconds of the last handshake call */
+  time_t handshake_last_call;
+  /* The actual retrans_timeout for the next message (e.g. doubled or so) */
+  unsigned int actual_retrans_timeout;
+
+  /* timers to handle async handshake after gnutls_handshake()
+   * has terminated. Required to handle retransmissions.
+   */
+  time_t async_term;
+  
+  /* last retransmission triggered by record layer */
+  time_t last_retransmit;
+} dtls_st;
+
 
 typedef union
 {
@@ -487,7 +624,9 @@ typedef union
 
 typedef struct
 {
-  gnutls_buffer_st application_data_buffer;     /* holds data to be delivered to application layer */
+  /* holds all the data received by the record layer */
+  mbuffer_head_st record_buffer; 
+
   gnutls_buffer_st handshake_hash_buffer;       /* used to keep the last received handshake 
                                                  * message */
   union
@@ -505,8 +644,6 @@ typedef struct
   } handshake_mac_handle;
   int handshake_mac_handle_init;        /* 1 when the previous union and type were initialized */
 
-  gnutls_buffer_st handshake_data_buffer;       /* this is a buffer that holds the current handshake message */
-  gnutls_buffer_st ia_data_buffer;      /* holds inner application data (TLS/IA) */
   int resumable:1;              /* TRUE or FALSE - if we can resume that session */
   handshake_state_t handshake_state;    /* holds
                                          * a number which indicates where
@@ -529,9 +666,6 @@ typedef struct
   int last_handshake_in;
   int last_handshake_out;
 
-  /* this is the compression method we are going to use */
-  gnutls_compression_method_t compression_method;
-
   /* priorities */
   struct gnutls_priority_st priorities;
 
@@ -540,22 +674,17 @@ typedef struct
   security_parameters_st resumed_security_parameters;
   gnutls_compression_method_t resumed_compression_method;
 
-  /* sockets internals */
-  int lowat;
-
   /* These buffers are used in the handshake
    * protocol only. freed using _gnutls_handshake_io_buffer_clear();
    */
   mbuffer_head_st handshake_send_buffer;
-  gnutls_handshake_description_t handshake_send_buffer_htype;
-  content_type_t handshake_recv_buffer_type;
-  gnutls_handshake_description_t handshake_recv_buffer_htype;
-  gnutls_buffer_st handshake_recv_buffer;
+  handshake_buffer_st handshake_recv_buffer[MAX_HANDSHAKE_MSGS];
+  int handshake_recv_buffer_size;
 
   /* this buffer holds a record packet -mostly used for
    * non blocking IO.
    */
-  mbuffer_head_st record_recv_buffer;
+  mbuffer_head_st record_recv_buffer;   /* buffer holding the record that is currently being received */
   mbuffer_head_st record_send_buffer;   /* holds cached data
                                          * for the gnutls_io_write_buffered()
                                          * function.
@@ -565,19 +694,8 @@ typedef struct
                                          * send.
                                          */
 
-
-  /* 0 if no peeked data was kept, 1 otherwise.
-   */
-  int have_peeked_data:1;
-
   int expire_time;              /* after expire_time seconds this session will expire */
   struct mod_auth_st_int *auth_struct;  /* used in handshake packets and KX algorithms */
-  int v2_hello;                 /* 0 if the client hello is v3+.
-                                 * non-zero if we got a v2 hello.
-                                 */
-  /* keeps the headers of the handshake packet 
-   */
-  handshake_header_buffer_st handshake_header_buffer;
 
   /* this is the highest version available
    * to the peer. (advertized version).
@@ -603,6 +721,7 @@ typedef struct
 
   /* PUSH & PULL functions.
    */
+  gnutls_pull_timeout_func pull_timeout_func;
   gnutls_pull_func pull_func;
   gnutls_push_func push_func;
   gnutls_vec_push_func vec_push_func;
@@ -683,11 +802,6 @@ typedef struct
    */
   internal_params_st params;
 
-  /* This buffer is used by the record recv functions,
-   * as a temporary store buffer.
-   */
-  gnutls_datum_t recv_buffer;
-
   /* To avoid using global variables, and especially on Windows where
    * the application may use a different errno variable than GnuTLS,
    * it is possible to use gnutls_transport_set_errno to set a
@@ -704,14 +818,12 @@ typedef struct
   gnutls_sign_func sign_func;
   void *sign_func_userdata;
 
-  /* Callback to extract TLS Finished message. */
-  gnutls_finished_callback_func finished_func;
-
   /* minimum bits to allow for SRP
    * use gnutls_srp_set_prime_bits() to adjust it.
    */
   uint16_t srp_prime_bits;
 
+  /* A handshake process has been completed */
   int initial_negotiation_completed:1;
 
   struct
@@ -727,6 +839,11 @@ typedef struct
     extension_priv_data_t priv;
     int set:1;
   } resumed_extension_int_data[MAX_EXT_TYPES];
+  /* The type of transport protocol; stream or datagram */
+  transport_t transport;
+
+  /* DTLS session state */
+  dtls_st dtls;
 
   unsigned int cb_tls_unique_len;
   unsigned char cb_tls_unique[MAX_VERIFY_DATA_SIZE];
@@ -747,7 +864,6 @@ struct gnutls_session_int
 };
 
 
-
 /* functions 
  */
 void _gnutls_set_current_version (gnutls_session_t session,
