@@ -28,14 +28,18 @@
 #include <gnutls_int.h>
 #include <gnutls_errors.h>
 
-void _mbuffer_init (mbuffer_head_st * buf);
-void _mbuffer_clear (mbuffer_head_st * buf);
+void _mbuffer_head_init (mbuffer_head_st * buf);
+void _mbuffer_head_clear (mbuffer_head_st * buf);
 void _mbuffer_enqueue (mbuffer_head_st * buf, mbuffer_st * bufel);
-int _mbuffer_remove_bytes (mbuffer_head_st * buf, size_t bytes);
+mbuffer_st* _mbuffer_dequeue (mbuffer_head_st * buf, mbuffer_st * bufel);
+int _mbuffer_head_remove_bytes (mbuffer_head_st * buf, size_t bytes);
 mbuffer_st *_mbuffer_alloc (size_t payload_size, size_t maximum_size);
 
-mbuffer_st *_mbuffer_get_first (mbuffer_head_st * buf, gnutls_datum_t * msg);
-mbuffer_st *_mbuffer_get_next (mbuffer_st * cur, gnutls_datum_t * msg);
+mbuffer_st *_mbuffer_head_get_first (mbuffer_head_st * buf, gnutls_datum_t * msg);
+mbuffer_st *_mbuffer_head_get_next (mbuffer_st * cur, gnutls_datum_t * msg);
+
+mbuffer_st *
+_mbuffer_head_pop_first (mbuffer_head_st * buf);
 
 /* This is dangerous since it will replace bufel with a new
  * one.
@@ -51,31 +55,45 @@ int _mbuffer_linearize (mbuffer_head_st * buf);
 inline static void
 _mbuffer_set_udata (mbuffer_st * bufel, void *data, size_t data_size)
 {
-  memcpy (bufel->msg.data + bufel->user_mark, data, data_size);
+  memcpy (bufel->msg.data + bufel->mark + bufel->user_mark, data, data_size);
+  bufel->msg.size = data_size + bufel->user_mark + bufel->mark;
 }
 
 inline static void *
 _mbuffer_get_uhead_ptr (mbuffer_st * bufel)
 {
-  return bufel->msg.data;
+  return bufel->msg.data + bufel->mark;
 }
 
 inline static void *
 _mbuffer_get_udata_ptr (mbuffer_st * bufel)
 {
-  return bufel->msg.data + bufel->user_mark;
+  return bufel->msg.data + bufel->user_mark + bufel->mark;
 }
 
 inline static void
 _mbuffer_set_udata_size (mbuffer_st * bufel, size_t size)
 {
-  bufel->msg.size = size + bufel->user_mark;
+  bufel->msg.size = size + bufel->user_mark + bufel->mark;
 }
 
 inline static size_t
 _mbuffer_get_udata_size (mbuffer_st * bufel)
 {
-  return bufel->msg.size - bufel->user_mark;
+  return bufel->msg.size - bufel->user_mark - bufel->mark;
+}
+
+/* discards size bytes from the begging of the buffer */
+inline static void
+_mbuffer_consume (mbuffer_head_st* buf, mbuffer_st * bufel, size_t size)
+{
+  bufel->user_mark = 0;
+  if (bufel->mark+size < bufel->msg.size)
+    bufel->mark += size;
+  else
+    bufel->mark = bufel->msg.size;
+
+  buf->byte_length -= size;
 }
 
 inline static size_t
@@ -93,15 +111,15 @@ _mbuffer_set_uhead_size (mbuffer_st * bufel, size_t size)
 
 
 inline static mbuffer_st *
-_gnutls_handshake_alloc (size_t size, size_t maximum)
+_gnutls_handshake_alloc (gnutls_session_t session, size_t size, size_t maximum)
 {
-  mbuffer_st *ret = _mbuffer_alloc (HANDSHAKE_HEADER_SIZE + size,
-                                    HANDSHAKE_HEADER_SIZE + maximum);
+  mbuffer_st *ret = _mbuffer_alloc (HANDSHAKE_HEADER_SIZE(session) + size,
+                                    HANDSHAKE_HEADER_SIZE(session) + maximum);
 
   if (!ret)
     return NULL;
 
-  _mbuffer_set_uhead_size (ret, HANDSHAKE_HEADER_SIZE);
+  _mbuffer_set_uhead_size (ret, HANDSHAKE_HEADER_SIZE(session));
 
   return ret;
 }
