@@ -185,8 +185,8 @@ _gnutls_pkcs1_rsa_encrypt (gnutls_datum_t * ciphertext,
   if (ciphertext->data == NULL)
     {
       gnutls_assert ();
-      _gnutls_free_datum (&encrypted);
-      return GNUTLS_E_MEMORY_ERROR;
+      ret = GNUTLS_E_MEMORY_ERROR;
+      goto cleanup;
     }
 
   memcpy (&ciphertext->data[pad], encrypted.data, encrypted.size);
@@ -195,9 +195,12 @@ _gnutls_pkcs1_rsa_encrypt (gnutls_datum_t * ciphertext,
 
   ciphertext->size = k;
 
+  ret = 0;
+
+cleanup:
   _gnutls_free_datum (&encrypted);
 
-  return 0;
+  return ret;
 }
 
 
@@ -216,9 +219,6 @@ _gnutls_pkcs1_rsa_decrypt (gnutls_datum_t * plaintext,
   size_t esize, mod_bits;
   gnutls_pk_params_st pk_params;
 
-  if (params_len > GNUTLS_MAX_PK_PARAMS)
-    return gnutls_assert_val(GNUTLS_E_INTERNAL_ERROR);
-
   for (i = 0; i < params_len; i++)
     pk_params.params[i] = params[i];
   pk_params.params_nr = params_len;
@@ -424,7 +424,7 @@ _gnutls_dsa_sign (gnutls_datum_t * signature,
 
   k = hash->size;
   if (k < 20)
-    {                           /* SHA1 or better only */
+    { /* SHA1 or better only */
       gnutls_assert ();
       return GNUTLS_E_PK_SIGN_FAILED;
     }
@@ -547,7 +547,6 @@ _generate_params (int algo, bigint_t * resarr, unsigned int *resarr_len,
     }
   else
     {
-      gnutls_pk_params_release(&params);
       gnutls_assert ();
       return GNUTLS_E_INVALID_REQUEST;
     }
