@@ -220,6 +220,7 @@ static const int protocol_priority[] = {
   GNUTLS_TLS1_1,
   GNUTLS_TLS1_0,
   GNUTLS_SSL3,
+  GNUTLS_DTLS1_0,
   0
 };
 
@@ -265,7 +266,9 @@ static const int cipher_priority_performance[] = {
 #ifdef	ENABLE_CAMELLIA
   GNUTLS_CIPHER_CAMELLIA_256_CBC,
 #endif
-  /* GNUTLS_CIPHER_ARCFOUR_40: Insecure, don't add! */
+#ifdef NETTLE_GCM
+  GNUTLS_CIPHER_AES_128_GCM,
+#endif
   0
 };
 
@@ -277,10 +280,12 @@ static const int cipher_priority_normal[] = {
   GNUTLS_CIPHER_AES_256_CBC,
 #ifdef	ENABLE_CAMELLIA
   GNUTLS_CIPHER_CAMELLIA_256_CBC,
+#endif
+#ifdef NETTLE_GCM
+  GNUTLS_CIPHER_AES_128_GCM,
 #endif
   GNUTLS_CIPHER_3DES_CBC,
   GNUTLS_CIPHER_ARCFOUR_128,
-  /* GNUTLS_CIPHER_ARCFOUR_40: Insecure, don't add! */
   0
 };
 
@@ -288,10 +293,11 @@ static const int cipher_priority_secure128[] = {
   GNUTLS_CIPHER_AES_128_CBC,
 #ifdef	ENABLE_CAMELLIA
   GNUTLS_CIPHER_CAMELLIA_128_CBC,
+#endif
+#ifdef NETTLE_GCM
+  GNUTLS_CIPHER_AES_128_GCM,
 #endif
   GNUTLS_CIPHER_3DES_CBC,
-  GNUTLS_CIPHER_ARCFOUR_128,
-  /* GNUTLS_CIPHER_ARCFOUR_40: Insecure, don't add! */
   0
 };
 
@@ -304,10 +310,11 @@ static const int cipher_priority_secure256[] = {
   GNUTLS_CIPHER_AES_128_CBC,
 #ifdef	ENABLE_CAMELLIA
   GNUTLS_CIPHER_CAMELLIA_128_CBC,
+#endif
+#ifdef NETTLE_GCM
+  GNUTLS_CIPHER_AES_128_GCM,
 #endif
   GNUTLS_CIPHER_3DES_CBC,
-  GNUTLS_CIPHER_ARCFOUR_128,
-  /* GNUTLS_CIPHER_ARCFOUR_40: Insecure, don't add! */
   0
 };
 
@@ -319,6 +326,7 @@ static const int cipher_priority_export[] = {
   GNUTLS_CIPHER_CAMELLIA_128_CBC,
   GNUTLS_CIPHER_CAMELLIA_256_CBC,
 #endif
+  GNUTLS_CIPHER_AES_128_GCM,
   GNUTLS_CIPHER_3DES_CBC,
   GNUTLS_CIPHER_ARCFOUR_128,
   GNUTLS_CIPHER_ARCFOUR_40,
@@ -355,16 +363,17 @@ static const int sign_priority_secure256[] = {
   0
 };
 
-static const int mac_priority_normal[] = {
+static const int mac_priority_performance[] = {
   GNUTLS_MAC_SHA1,
   GNUTLS_MAC_SHA256,
-  GNUTLS_MAC_MD5,
+  GNUTLS_MAC_AEAD,
   0
 };
 
 
 static const int mac_priority_secure[] = {
   GNUTLS_MAC_SHA256,
+  GNUTLS_MAC_AEAD,
   GNUTLS_MAC_SHA1,
   0
 };
@@ -574,7 +583,7 @@ gnutls_priority_init (gnutls_priority_t * priority_cache,
           _set_priority (&(*priority_cache)->cipher,
                          cipher_priority_performance);
           _set_priority (&(*priority_cache)->kx, kx_priority_performance);
-          _set_priority (&(*priority_cache)->mac, mac_priority_normal);
+          _set_priority (&(*priority_cache)->mac, mac_priority_performance);
           _set_priority (&(*priority_cache)->sign_algo,
                          sign_priority_default);
         }
@@ -582,7 +591,7 @@ gnutls_priority_init (gnutls_priority_t * priority_cache,
         {
           _set_priority (&(*priority_cache)->cipher, cipher_priority_normal);
           _set_priority (&(*priority_cache)->kx, kx_priority_secure);
-          _set_priority (&(*priority_cache)->mac, mac_priority_normal);
+          _set_priority (&(*priority_cache)->mac, mac_priority_secure);
           _set_priority (&(*priority_cache)->sign_algo,
                          sign_priority_default);
         }
