@@ -21,8 +21,10 @@
 */
 
 #include <gnutls_int.h>
+#include <pakchois/pakchois.h>
 #include <gnutls/pkcs11.h>
 #include <stdio.h>
+#include <stdbool.h>
 #include <string.h>
 #include <gnutls_errors.h>
 #include <gnutls_datum.h>
@@ -43,9 +45,7 @@ struct gnutls_privkey_st
   union
   {
     gnutls_x509_privkey_t x509;
-#ifdef ENABLE_PKCS11
     gnutls_pkcs11_privkey_t pkcs11;
-#endif
 #ifdef ENABLE_OPENPGP
     gnutls_openpgp_privkey_t openpgp;
 #endif
@@ -91,10 +91,8 @@ gnutls_privkey_get_pk_algorithm (gnutls_privkey_t key, unsigned int *bits)
     case GNUTLS_PRIVKEY_OPENPGP:
       return gnutls_openpgp_privkey_get_pk_algorithm (key->key.openpgp, bits);
 #endif
-#ifdef ENABLE_PKCS11
     case GNUTLS_PRIVKEY_PKCS11:
       return gnutls_pkcs11_privkey_get_pk_algorithm (key->key.pkcs11, bits);
-#endif
     case GNUTLS_PRIVKEY_X509:
       if (bits)
         *bits = _gnutls_mpi_get_nbits (key->key.x509->params[0]);
@@ -190,7 +188,7 @@ _gnutls_privkey_get_public_mpis (gnutls_privkey_t key,
         bigint_t tmp_params[MAX_PRIV_PARAMS_SIZE];
         int tmp_params_size = MAX_PRIV_PARAMS_SIZE;
         uint32_t kid[2], i;
-        uint8_t keyid[GNUTLS_OPENPGP_KEYID_SIZE];
+        gnutls_openpgp_keyid_t keyid;
 
         ret =
           gnutls_openpgp_privkey_get_preferred_key_id (key->key.openpgp,
@@ -269,8 +267,6 @@ gnutls_privkey_init (gnutls_privkey_t * key)
 void
 gnutls_privkey_deinit (gnutls_privkey_t key)
 {
-  if (key == NULL) return;
-
   if (key->flags & GNUTLS_PRIVKEY_IMPORT_AUTO_RELEASE)
     switch (key->type)
       {
@@ -279,11 +275,9 @@ gnutls_privkey_deinit (gnutls_privkey_t key)
         gnutls_openpgp_privkey_deinit (key->key.openpgp);
         break;
 #endif
-#ifdef ENABLE_PKCS11
       case GNUTLS_PRIVKEY_PKCS11:
         gnutls_pkcs11_privkey_deinit (key->key.pkcs11);
         break;
-#endif
       case GNUTLS_PRIVKEY_X509:
         gnutls_x509_privkey_deinit (key->key.x509);
         break;
@@ -301,8 +295,6 @@ static int check_if_clean(gnutls_privkey_t key)
   return 0;
 }
 
-#ifdef ENABLE_PKCS11
-
 /**
  * gnutls_privkey_import_pkcs11:
  * @pkey: The private key
@@ -339,8 +331,6 @@ int ret;
   return 0;
 }
 
-#endif /* ENABLE_PKCS11 */
-
 /**
  * gnutls_privkey_import_x509:
  * @pkey: The private key
@@ -400,7 +390,7 @@ gnutls_privkey_import_openpgp (gnutls_privkey_t pkey,
                                unsigned int flags)
 {
 int ret, idx;
-uint8_t keyid[GNUTLS_OPENPGP_KEYID_SIZE];
+gnutls_openpgp_keyid_t keyid;
 
   ret = check_if_clean(pkey);
   if (ret < 0)
@@ -444,7 +434,10 @@ uint8_t keyid[GNUTLS_OPENPGP_KEYID_SIZE];
  * This function will sign the given data using a signature algorithm
  * supported by the private key. Signature algorithms are always used
  * together with a hash functions.  Different hash functions may be
- * used for the RSA algorithm, but only SHA-1 for the DSA keys.
+ * used for the RSA algorithm, but only the SHA family for the DSA keys.
+ *
+ * Use gnutls_pubkey_get_preferred_hash_algorithm() to determine
+ * the hash algorithm.
  *
  * Returns: On success, %GNUTLS_E_SUCCESS is returned, otherwise a
  * negative error value.
@@ -504,7 +497,7 @@ cleanup:
  * together with a hash functions.  Different hash functions may be
  * used for the RSA algorithm, but only SHA-XXX for the DSA keys.
  *
- * Use gnutls_x509_crt_get_preferred_hash_algorithm() to determine
+ * Use gnutls_pubkey_get_preferred_hash_algorithm() to determine
  * the hash algorithm.
  *
  * Returns: On success, %GNUTLS_E_SUCCESS is returned, otherwise a
@@ -576,11 +569,9 @@ _gnutls_privkey_sign_hash (gnutls_privkey_t key,
       return gnutls_openpgp_privkey_sign_hash (key->key.openpgp,
                                                 hash, signature);
 #endif
-#ifdef ENABLE_PKCS11
     case GNUTLS_PRIVKEY_PKCS11:
       return _gnutls_pkcs11_privkey_sign_hash (key->key.pkcs11,
                                                hash, signature);
-#endif
     case GNUTLS_PRIVKEY_X509:
       return _gnutls_soft_sign (key->key.x509->pk_algorithm,
                                 key->key.x509->params,
@@ -627,12 +618,10 @@ gnutls_privkey_decrypt_data (gnutls_privkey_t key,
       return _gnutls_pkcs1_rsa_decrypt (plaintext, ciphertext,
                                         key->key.x509->params,
                                         key->key.x509->params_size, 2);
-#ifdef ENABLE_PKCS11
     case GNUTLS_PRIVKEY_PKCS11:
       return _gnutls_pkcs11_privkey_decrypt_data (key->key.pkcs11,
                                                  flags,
                                                  ciphertext, plaintext);
-#endif
     default:
       gnutls_assert ();
       return GNUTLS_E_INVALID_REQUEST;
