@@ -222,44 +222,18 @@ sign_tls_hash (gnutls_session_t session, gnutls_digest_algorithm_t hash_algo,
         if (!(cert->key_usage & GNUTLS_KEY_DIGITAL_SIGNATURE))
           {
             gnutls_assert ();
-            _gnutls_debug_log("Key usage violation was detected (ignored).\n");
+            return GNUTLS_E_KEY_USAGE_VIOLATION;
           }
 
       /* External signing. */
       if (!pkey)
         {
-          int ret;
-
           if (!session->internals.sign_func)
             return GNUTLS_E_INSUFFICIENT_CREDENTIALS;
 
-          if (!_gnutls_version_has_selectable_sighash (ver))
-            return (*session->internals.sign_func)
-              (session, session->internals.sign_func_userdata,
-               cert->cert_type, &cert->raw, hash_concat, signature);
-          else
-            {
-              gnutls_datum_t digest;
-
-              ret = _gnutls_set_datum(&digest, hash_concat->data, hash_concat->size);
-              if (ret < 0)
-                return gnutls_assert_val(ret);
-              
-              ret = pk_prepare_hash (gnutls_privkey_get_pk_algorithm(pkey, NULL), hash_algo, &digest);
-              if (ret < 0)
-                {
-                  gnutls_assert ();
-                  goto es_cleanup;
-                }
-
-              ret = (*session->internals.sign_func)
-                (session, session->internals.sign_func_userdata,
-                 cert->cert_type, &cert->raw, &digest, signature);
-es_cleanup:
-              gnutls_free(digest.data);
-              
-              return ret;
-            }
+          return (*session->internals.sign_func)
+            (session, session->internals.sign_func_userdata,
+             cert->cert_type, &cert->raw, hash_concat, signature);
         }
     }
 
@@ -270,7 +244,7 @@ es_cleanup:
 }
 
 static int
-verify_tls_hash (gnutls_session_t session, gnutls_protocol_t ver, gnutls_cert * cert,
+verify_tls_hash (gnutls_protocol_t ver, gnutls_cert * cert,
                     const gnutls_datum_t * hash_concat,
                     gnutls_datum_t * signature, size_t sha1pos,
                     gnutls_pk_algorithm_t pk_algo)
@@ -292,7 +266,7 @@ verify_tls_hash (gnutls_session_t session, gnutls_protocol_t ver, gnutls_cert *
     if (!(cert->key_usage & GNUTLS_KEY_DIGITAL_SIGNATURE))
       {
         gnutls_assert ();
-        _gnutls_debug_log("Key usage violation was detected (ignored).\n");
+        return GNUTLS_E_KEY_USAGE_VIOLATION;
       }
 
   if (pk_algo == GNUTLS_PK_UNKNOWN)
@@ -425,7 +399,7 @@ _gnutls_handshake_verify_data (gnutls_session_t session, gnutls_cert * cert,
       dconcat.size = _gnutls_hash_get_algo_len (hash_algo);
     }
 
-  ret = verify_tls_hash (session, ver, cert, &dconcat, signature,
+  ret = verify_tls_hash (ver, cert, &dconcat, signature,
                             dconcat.size -
                             _gnutls_hash_get_algo_len (hash_algo),
                             _gnutls_sign_get_pk_algorithm (algo));
@@ -490,7 +464,7 @@ _gnutls_handshake_verify_cert_vrfy12 (gnutls_session_t session,
   dconcat.size = _gnutls_hash_get_algo_len (hash_algo);
 
   ret =
-    verify_tls_hash (session, ver, cert, &dconcat, signature, 0,
+    verify_tls_hash (ver, cert, &dconcat, signature, 0,
                         cert->subject_pk_algorithm);
   if (ret < 0)
     {
@@ -558,18 +532,29 @@ _gnutls_handshake_verify_cert_vrfy (gnutls_session_t session,
       ret = _gnutls_generate_master (session, 1);
       if (ret < 0)
         {
-          gnutls_assert ();
-          return ret;
+          _gnutls_hash_deinit (&td_md5, NULL);
+          _gnutls_hash_deinit (&td_sha, NULL);
+          return gnutls_assert_val(ret);
         }
 
-      _gnutls_mac_deinit_ssl3_handshake (&td_md5, concat,
+      ret = _gnutls_mac_deinit_ssl3_handshake (&td_md5, concat,
                                          session->
                                          security_parameters.master_secret,
                                          GNUTLS_MASTER_SIZE);
-      _gnutls_mac_deinit_ssl3_handshake (&td_sha, &concat[16],
+      if (ret < 0)
+        {
+          _gnutls_hash_deinit (&td_sha, NULL);
+          return gnutls_assert_val(ret);
+        }
+
+      ret = _gnutls_mac_deinit_ssl3_handshake (&td_sha, &concat[16],
                                          session->
                                          security_parameters.master_secret,
                                          GNUTLS_MASTER_SIZE);
+      if (ret < 0)
+        {
+          return gnutls_assert_val(ret);
+        }
     }
   else
     {
@@ -581,7 +566,7 @@ _gnutls_handshake_verify_cert_vrfy (gnutls_session_t session,
   dconcat.size = 20 + 16;       /* md5+ sha */
 
   ret =
-    verify_tls_hash (session, ver, cert, &dconcat, signature, 16,
+    verify_tls_hash (ver, cert, &dconcat, signature, 16,
                         cert->subject_pk_algorithm);
   if (ret < 0)
     {
@@ -700,13 +685,16 @@ _gnutls_handshake_sign_cert_vrfy (gnutls_session_t session,
       if (ret < 0)
         {
           gnutls_assert ();
+          _gnutls_hash_deinit (&td_sha, NULL);
           return ret;
         }
 
-      _gnutls_mac_deinit_ssl3_handshake (&td_sha, &concat[16],
+      ret = _gnutls_mac_deinit_ssl3_handshake (&td_sha, &concat[16],
                                          session->
                                          security_parameters.master_secret,
                                          GNUTLS_MASTER_SIZE);
+      if (ret < 0)
+        return gnutls_assert_val(ret);
     }
   else
     _gnutls_hash_deinit (&td_sha, &concat[16]);
@@ -725,10 +713,14 @@ _gnutls_handshake_sign_cert_vrfy (gnutls_session_t session,
         }
 
       if (ver == GNUTLS_SSL3)
-        _gnutls_mac_deinit_ssl3_handshake (&td_md5, concat,
+        {
+          ret = _gnutls_mac_deinit_ssl3_handshake (&td_md5, concat,
                                            session->
                                            security_parameters.master_secret,
                                            GNUTLS_MASTER_SIZE);
+          if (ret < 0)
+            return gnutls_assert_val(ret);
+        }
       else
         _gnutls_hash_deinit (&td_md5, concat);
 
@@ -737,7 +729,7 @@ _gnutls_handshake_sign_cert_vrfy (gnutls_session_t session,
       break;
     case GNUTLS_PK_DSA:
       /* ensure 1024 bit DSA keys are used */
-      hash_algo = _gnutls_dsa_q_to_hash (cert->params[1], NULL);
+      hash_algo = _gnutls_dsa_q_to_hash (cert->params[1]);
       if (!_gnutls_version_has_selectable_sighash (ver) && hash_algo != GNUTLS_DIG_SHA1)
         return gnutls_assert_val(GNUTLS_E_INCOMPAT_DSA_KEY_WITH_TLS_PROTOCOL);
 
@@ -764,6 +756,22 @@ pk_hash_data (gnutls_pk_algorithm_t pk, gnutls_digest_algorithm_t hash,
 {
   int ret;
 
+  switch (pk)
+    {
+    case GNUTLS_PK_RSA:
+      break;
+    case GNUTLS_PK_DSA:
+      if (params && hash != _gnutls_dsa_q_to_hash (params[1]))
+        {
+          gnutls_assert ();
+          return GNUTLS_E_INVALID_REQUEST;
+        }
+      break;
+    default:
+      gnutls_assert ();
+      return GNUTLS_E_INVALID_REQUEST;
+    }
+
   digest->size = _gnutls_hash_get_algo_len (hash);
   digest->data = gnutls_malloc (digest->size);
   if (digest->data == NULL)
@@ -804,7 +812,7 @@ encode_ber_digest_info (gnutls_digest_algorithm_t hash,
   if (algo == NULL)
     {
       gnutls_assert ();
-      _gnutls_x509_log ("Hash algorithm: %d\n", hash);
+      _gnutls_debug_log ("Hash algorithm: %d has no OID\n", hash);
       return GNUTLS_E_UNKNOWN_PK_ALGORITHM;
     }
 
@@ -883,7 +891,7 @@ pk_prepare_hash (gnutls_pk_algorithm_t pk,
                  gnutls_digest_algorithm_t hash, gnutls_datum_t * digest)
 {
   int ret;
-  gnutls_datum old_digest = { digest->data, digest->size };
+  gnutls_datum_t old_digest = { digest->data, digest->size };
 
   switch (pk)
     {
