@@ -48,6 +48,7 @@
 #include <gnutls_rsa_export.h>
 #include <gnutls_extensions.h>
 #include <system.h>
+#include <gnutls/dtls.h>
 
 /* These should really be static, but src/tests.c calls them.  Make
    them public functions?  */
@@ -59,6 +60,8 @@ void
 _gnutls_session_cert_type_set (gnutls_session_t session,
                                gnutls_certificate_type_t ct)
 {
+  _gnutls_handshake_log("HSK[%p]: Selected certificate type %s (%d)\n", session,
+        gnutls_certificate_type_get_name(ct), ct);
   session->security_parameters.cert_type = ct;
 }
 
@@ -75,7 +78,11 @@ gnutls_cipher_algorithm_t
 gnutls_cipher_get (gnutls_session_t session)
 {
   record_parameters_st *record_params;
-  _gnutls_epoch_get (session, EPOCH_READ_CURRENT, &record_params);
+  int ret;
+  
+  ret = _gnutls_epoch_get (session, EPOCH_READ_CURRENT, &record_params);
+  if (ret < 0)
+    return gnutls_assert_val(GNUTLS_CIPHER_NULL);
 
   return record_params->cipher_algorithm;
 }
@@ -124,7 +131,11 @@ gnutls_mac_algorithm_t
 gnutls_mac_get (gnutls_session_t session)
 {
   record_parameters_st *record_params;
-  _gnutls_epoch_get (session, EPOCH_READ_CURRENT, &record_params);
+  int ret;
+  
+  ret = _gnutls_epoch_get (session, EPOCH_READ_CURRENT, &record_params);
+  if (ret < 0)
+    return gnutls_assert_val(GNUTLS_MAC_NULL);
 
   return record_params->mac_algorithm;
 }
@@ -142,7 +153,11 @@ gnutls_compression_method_t
 gnutls_compression_get (gnutls_session_t session)
 {
   record_parameters_st *record_params;
-  _gnutls_epoch_get (session, EPOCH_READ_CURRENT, &record_params);
+  int ret;
+  
+  ret = _gnutls_epoch_get (session, EPOCH_READ_CURRENT, &record_params);
+  if (ret < 0)
+    return gnutls_assert_val(GNUTLS_COMP_NULL);
 
   return record_params->compression_algorithm;
 }
@@ -231,9 +246,6 @@ _gnutls_handshake_internal_state_init (gnutls_session_t session)
   /* by default no selected certificate */
   session->internals.adv_version_major = 0;
   session->internals.adv_version_minor = 0;
-  session->internals.v2_hello = 0;
-  memset (&session->internals.handshake_header_buffer, 0,
-          sizeof (handshake_header_buffer_st));
   session->internals.direction = 0;
 
   /* use out of band data for the last
@@ -243,6 +255,10 @@ _gnutls_handshake_internal_state_init (gnutls_session_t session)
   session->internals.last_handshake_out = -1;
 
   session->internals.resumable = RESUME_TRUE;
+  
+  session->internals.dtls.hsk_read_seq = 0;
+  session->internals.dtls.hsk_write_seq = 0;
+  session->internals.dtls.handshake_start_time = time(0);
 }
 
 void
@@ -250,29 +266,31 @@ _gnutls_handshake_internal_state_clear (gnutls_session_t session)
 {
   _gnutls_handshake_internal_state_init (session);
 
-  _gnutls_free_datum (&session->internals.recv_buffer);
-
   deinit_internal_params (session);
-
+  
+  _gnutls_epoch_gc(session);
 }
 
 #define MIN_DH_BITS 727
 /**
  * gnutls_init:
- * @con_end: indicate if this session is to be used for server or client.
  * @session: is a pointer to a #gnutls_session_t structure.
+ * @flags: indicate if this session is to be used for server or client.
  *
  * This function initializes the current session to null. Every
  * session must be initialized before use, so internal structures can
  * be allocated.  This function allocates structures which can only
  * be free'd by calling gnutls_deinit().  Returns zero on success.
  *
- * @con_end can be one of %GNUTLS_CLIENT and %GNUTLS_SERVER.
+ * @flags can be one of %GNUTLS_CLIENT and %GNUTLS_SERVER. For a DTLS
+ * entity, the flags %GNUTLS_DATAGRAM and  %GNUTLS_NONBLOCK are
+ * also available. The latter flag will enable a non-blocking
+ * operation of the DTLS timers.
  *
  * Returns: %GNUTLS_E_SUCCESS on success, or an error code.
  **/
 int
-gnutls_init (gnutls_session_t * session, gnutls_connection_end_t con_end)
+gnutls_init (gnutls_session_t * session, unsigned int flags)
 {
   int ret;
   record_parameters_st *epoch;
@@ -293,22 +311,20 @@ gnutls_init (gnutls_session_t * session, gnutls_connection_end_t con_end)
 
   (*session)->security_parameters.epoch_next = 1;
 
-  (*session)->security_parameters.entity = con_end;
+  (*session)->security_parameters.entity = (flags&GNUTLS_SERVER?GNUTLS_SERVER:GNUTLS_CLIENT);
 
   /* the default certificate type for TLS */
   (*session)->security_parameters.cert_type = DEFAULT_CERT_TYPE;
 
   /* Initialize buffers */
-  _gnutls_buffer_init (&(*session)->internals.application_data_buffer);
-  _gnutls_buffer_init (&(*session)->internals.handshake_data_buffer);
   _gnutls_buffer_init (&(*session)->internals.handshake_hash_buffer);
-  _gnutls_buffer_init (&(*session)->internals.ia_data_buffer);
 
-  _mbuffer_init (&(*session)->internals.record_send_buffer);
-  _mbuffer_init (&(*session)->internals.record_recv_buffer);
+  _mbuffer_head_init (&(*session)->internals.record_buffer);
+  _mbuffer_head_init (&(*session)->internals.record_send_buffer);
+  _mbuffer_head_init (&(*session)->internals.record_recv_buffer);
 
-  _mbuffer_init (&(*session)->internals.handshake_send_buffer);
-  _gnutls_buffer_init (&(*session)->internals.handshake_recv_buffer);
+  _mbuffer_head_init (&(*session)->internals.handshake_send_buffer);
+  _gnutls_handshake_recv_buffer_init(*session);
 
   (*session)->key = gnutls_calloc (1, sizeof (struct gnutls_key_st));
   if ((*session)->key == NULL)
@@ -322,8 +338,6 @@ gnutls_init (gnutls_session_t * session, gnutls_connection_end_t con_end)
 
   gnutls_dh_set_prime_bits ((*session), MIN_DH_BITS);
 
-  gnutls_transport_set_lowat ((*session), DEFAULT_LOWAT);       /* the default for tcp */
-
   gnutls_handshake_set_max_packet_length ((*session),
                                           MAX_HANDSHAKE_PACKET_SIZE);
 
@@ -357,6 +371,25 @@ gnutls_init (gnutls_session_t * session, gnutls_connection_end_t con_end)
 #endif
   gnutls_transport_set_pull_function (*session, system_read);
   gnutls_transport_set_errno_function (*session, system_errno);
+  gnutls_transport_set_pull_timeout_function (*session, system_recv_timeout);
+
+  if (flags & GNUTLS_DATAGRAM)
+    {
+      (*session)->internals.dtls.mtu = DTLS_DEFAULT_MTU;
+      (*session)->internals.transport = GNUTLS_DGRAM;
+
+      (*session)->internals.dtls.retrans_timeout = 1000;
+      (*session)->internals.dtls.total_timeout = 60000;
+
+      (*session)->internals.dtls.record_sw_size = 0;
+    }
+  else
+    (*session)->internals.transport = GNUTLS_STREAM;
+  
+  if (flags & GNUTLS_NONBLOCK)
+    (*session)->internals.dtls.blocking = 0;
+  else
+    (*session)->internals.dtls.blocking = 1;
 
   return 0;
 }
@@ -400,12 +433,10 @@ gnutls_deinit (gnutls_session_t session)
         session->record_parameters[i] = NULL;
       }
 
-  _gnutls_buffer_clear (&session->internals.ia_data_buffer);
   _gnutls_buffer_clear (&session->internals.handshake_hash_buffer);
-  _gnutls_buffer_clear (&session->internals.handshake_data_buffer);
-  _gnutls_buffer_clear (&session->internals.application_data_buffer);
-  _mbuffer_clear (&session->internals.record_recv_buffer);
-  _mbuffer_clear (&session->internals.record_send_buffer);
+  _mbuffer_head_clear (&session->internals.record_buffer);
+  _mbuffer_head_clear (&session->internals.record_recv_buffer);
+  _mbuffer_head_clear (&session->internals.record_send_buffer);
 
   gnutls_credentials_clear (session);
   _gnutls_selected_certs_deinit (session);
@@ -1060,105 +1091,6 @@ gnutls_prf (gnutls_session_t session,
   return ret;
 }
 
-/*-
- * gnutls_session_get_client_random:
- * @session: is a #gnutls_session_t structure.
- *
- * Return a pointer to the 32-byte client random field used in the
- * session.  The pointer must not be modified or deallocated.
- *
- * If a client random value has not yet been established, the output
- * will be garbage; in particular, a %NULL return value should not be
- * expected.
- *
- * Returns: pointer to client random data.
- *
- * Deprecated in: 2.11.0
- -*/
-const void *
-gnutls_session_get_client_random (gnutls_session_t session)
-{
-  return (char *) session->security_parameters.client_random;
-}
-
-/*-
- * gnutls_session_get_server_random:
- * @session: is a #gnutls_session_t structure.
- *
- * Return a pointer to the 32-byte server random field used in the
- * session.  The pointer must not be modified or deallocated.
- *
- * If a server random value has not yet been established, the output
- * will be garbage; in particular, a %NULL return value should not be
- * expected.
- *
- * Returns: pointer to server random data.
- *
- * Deprecated in: 2.11.0
- -*/
-const void *
-gnutls_session_get_server_random (gnutls_session_t session)
-{
-  return (char *) session->security_parameters.server_random;
-}
-
-/*-
- * gnutls_session_get_master_secret:
- * @session: is a #gnutls_session_t structure.
- *
- * Return a pointer to the 48-byte master secret in the session.  The
- * pointer must not be modified or deallocated.
- *
- * If a master secret value has not yet been established, the output
- * will be garbage; in particular, a %NULL return value should not be
- * expected.
- *
- * Consider using gnutls_prf() rather than extracting the master
- * secret and use it to derive further data.
- *
- * Returns: pointer to master secret data.
- *
- * Deprecated in: 2.11.0
- -*/
-const void *
-gnutls_session_get_master_secret (gnutls_session_t session)
-{
-  return (char *) session->security_parameters.master_secret;
-}
-
-/*-
- * gnutls_session_set_finished_function:
- * @session: is a #gnutls_session_t structure.
- * @func: a #gnutls_finished_callback_func callback.
- *
- * Register a callback function for the session that will be called
- * when a TLS Finished message has been generated.  The function is
- * typically used to copy away the TLS finished message for later use
- * as a channel binding or similar purpose.
- *
- * The callback should follow this prototype:
- *
- * void callback (gnutls_session_t @session, const void *@finished, size_t @len);
- *
- * The @finished parameter will contain the binary TLS finished
- * message, and @len will contains its length.  For SSLv3 connections,
- * the @len parameter will be 36 and for TLS connections it will be
- * 12.
- *
- * It is recommended that the function returns quickly in order to not
- * delay the handshake.  Use the function to store a copy of the TLS
- * finished message for later use.
- *
- * Since: 2.6.0
- * Deprecated in: 2.11.0
- -*/
-void
-gnutls_session_set_finished_function (gnutls_session_t session,
-                                      gnutls_finished_callback_func func)
-{
-  session->internals.finished_func = func;
-}
-
 /**
  * gnutls_session_is_resumed:
  * @session: is a #gnutls_session_t structure.
@@ -1389,3 +1321,50 @@ gnutls_session_channel_binding (gnutls_session_t session,
 
   return 0;
 }
+
+/* returns overhead imposed by the record layer (encryption/compression)
+ * etc. It does include the record layer headers.
+ *
+ * It may return a negative value on error.
+ */
+int _gnutls_record_overhead_rt(gnutls_session_t session)
+{
+record_parameters_st *params;
+int total = 0, ret, iv_size;
+
+  if (session->internals.initial_negotiation_completed == 0)
+    return RECORD_HEADER_SIZE(session);
+
+  ret = _gnutls_epoch_get (session, EPOCH_WRITE_CURRENT, &params);
+  if (ret < 0)
+    return gnutls_assert_val(ret);
+
+  /* requires padding */
+  iv_size = _gnutls_cipher_get_iv_size(params->cipher_algorithm);
+  total += iv_size;
+
+  if (_gnutls_cipher_is_block (params->cipher_algorithm) == CIPHER_BLOCK)
+    {
+      if (!IS_DTLS(session))
+        total += MAX_PAD_SIZE;
+      else
+        total += iv_size; /* iv_size == block_size */
+    }
+  
+  if (params->mac_algorithm == GNUTLS_MAC_AEAD)
+    total += _gnutls_cipher_get_tag_size(params->cipher_algorithm);
+  else
+    {
+      ret = _gnutls_hmac_get_algo_len(params->mac_algorithm);
+      if (ret < 0)
+        return gnutls_assert_val(ret);
+      total+=ret;
+    }
+
+  if (params->compression_algorithm != GNUTLS_COMP_NULL)
+    total += EXTRA_COMP_SIZE;
+  
+  total += RECORD_HEADER_SIZE(session);
+  
+  return total;
+}
