@@ -92,16 +92,12 @@ extern "C"
  * @GNUTLS_X509_CRT_LIST_IMPORT_FAIL_IF_EXCEED: Fail if the
  *   certificates in the buffer are more than the space allocated for
  *   certificates. The error code will be %GNUTLS_E_SHORT_MEMORY_BUFFER.
- * @GNUTLS_X509_CRT_LIST_FAIL_IF_UNSORTED: Fail if the certificates
- *   in the buffer are not ordered starting from subject to issuer.
- *   The error code will be %GNUTLS_E_CERTIFICATE_LIST_UNSORTED.
  *
  * Enumeration of different certificate import flags.
  */
   typedef enum gnutls_certificate_import_flags
   {
-    GNUTLS_X509_CRT_LIST_IMPORT_FAIL_IF_EXCEED = 1,
-    GNUTLS_X509_CRT_LIST_FAIL_IF_UNSORTED = 2
+    GNUTLS_X509_CRT_LIST_IMPORT_FAIL_IF_EXCEED = 1
   } gnutls_certificate_import_flags;
 
   int gnutls_x509_crt_init (gnutls_x509_crt_t * cert);
@@ -109,6 +105,10 @@ extern "C"
   int gnutls_x509_crt_import (gnutls_x509_crt_t cert,
                               const gnutls_datum_t * data,
                               gnutls_x509_crt_fmt_t format);
+  int gnutls_x509_crt_list_import2 (gnutls_x509_crt_t ** certs,
+                             unsigned int * size,
+                             const gnutls_datum_t * data,
+                             gnutls_x509_crt_fmt_t format, unsigned int flags);
   int gnutls_x509_crt_list_import (gnutls_x509_crt_t * certs,
                                    unsigned int *cert_max,
                                    const gnutls_datum_t * data,
@@ -430,6 +430,16 @@ extern "C"
   int gnutls_x509_crl_check_issuer (gnutls_x509_crl_t crl,
                                     gnutls_x509_crt_t issuer);
 
+  int gnutls_x509_crl_list_import2 (gnutls_x509_crl_t ** crls,
+                             unsigned int * size,
+                             const gnutls_datum_t * data,
+                             gnutls_x509_crt_fmt_t format, unsigned int flags);
+
+  int gnutls_x509_crl_list_import (gnutls_x509_crl_t * crls,
+                                   unsigned int *crl_max,
+                                   const gnutls_datum_t * data,
+                                   gnutls_x509_crt_fmt_t format,
+                                   unsigned int flags);
 /* CRL writing.
  */
   int gnutls_x509_crl_set_version (gnutls_x509_crl_t crl,
@@ -711,8 +721,6 @@ extern "C"
                              gnutls_certificate_print_formats_t format,
                              gnutls_datum_t * out);
 
-  int gnutls_x509_crq_verify (gnutls_x509_crq_t crq, unsigned int flags);
-
   int gnutls_x509_crq_init (gnutls_x509_crq_t * crq);
   void gnutls_x509_crq_deinit (gnutls_x509_crq_t crq);
   int gnutls_x509_crq_import (gnutls_x509_crq_t crq,
@@ -820,6 +828,44 @@ extern "C"
                                             void *buf, size_t * sizeof_buf,
                                             unsigned int *critical);
 
+  typedef struct gnutls_x509_trust_list_st *gnutls_x509_trust_list_t;
+
+  int
+  gnutls_x509_trust_list_init (gnutls_x509_trust_list_t * list, unsigned int size);
+
+  void
+  gnutls_x509_trust_list_deinit (gnutls_x509_trust_list_t list, unsigned int all);
+
+  int
+  gnutls_x509_trust_list_add_cas (gnutls_x509_trust_list_t list, 
+    const gnutls_x509_crt_t * clist, int clist_size, unsigned int flags);
+
+#define GNUTLS_TL_VERIFY_CRL 1
+  int
+  gnutls_x509_trust_list_add_crls (gnutls_x509_trust_list_t list, 
+    const gnutls_x509_crl_t * crl_list, int crl_size, unsigned int flags,
+    unsigned int verification_flags);
+
+  typedef int gnutls_verify_output_function (
+    gnutls_x509_crt_t cert,
+    gnutls_x509_crt_t issuer, /* The issuer if verification failed 
+     * because of him. might be null.
+     */
+    gnutls_x509_crl_t crl, /* The CRL that caused verification failure 
+     * if any. Might be null. 
+     */
+    unsigned int verification_output);
+
+  int
+  gnutls_x509_trust_list_verify_crt (
+    gnutls_x509_trust_list_t list,
+    gnutls_x509_crt_t *cert_list,
+    unsigned int cert_list_size,
+    unsigned int flags,
+    unsigned int *verify,
+    gnutls_verify_output_function func);
+
+
 #ifdef __cplusplus
 }
 #endif
