@@ -1426,7 +1426,7 @@ int
 gnutls_openpgp_crt_get_pk_rsa_raw (gnutls_openpgp_crt_t crt,
                                    gnutls_datum_t * m, gnutls_datum_t * e)
 {
-  uint8_t keyid[GNUTLS_OPENPGP_KEYID_SIZE];
+  gnutls_openpgp_keyid_t keyid;
   int ret;
 
   ret = gnutls_openpgp_crt_get_key_id (crt, keyid);
@@ -1460,7 +1460,7 @@ gnutls_openpgp_crt_get_pk_dsa_raw (gnutls_openpgp_crt_t crt,
                                    gnutls_datum_t * p, gnutls_datum_t * q,
                                    gnutls_datum_t * g, gnutls_datum_t * y)
 {
-  uint8_t keyid[GNUTLS_OPENPGP_KEYID_SIZE];
+  gnutls_openpgp_keyid_t keyid;
   int ret;
 
   ret = gnutls_openpgp_crt_get_key_id (crt, keyid);
@@ -1494,7 +1494,7 @@ gnutls_openpgp_crt_get_subkey_pk_rsa_raw (gnutls_openpgp_crt_t crt,
                                           gnutls_datum_t * m,
                                           gnutls_datum_t * e)
 {
-  uint8_t keyid[GNUTLS_OPENPGP_KEYID_SIZE];
+  gnutls_openpgp_keyid_t keyid;
   int ret;
 
   ret = gnutls_openpgp_crt_get_subkey_id (crt, idx, keyid);
@@ -1532,7 +1532,7 @@ gnutls_openpgp_crt_get_subkey_pk_dsa_raw (gnutls_openpgp_crt_t crt,
                                           gnutls_datum_t * g,
                                           gnutls_datum_t * y)
 {
-  uint8_t keyid[GNUTLS_OPENPGP_KEYID_SIZE];
+  gnutls_openpgp_keyid_t keyid;
   int ret;
 
   ret = gnutls_openpgp_crt_get_subkey_id (crt, idx, keyid);
@@ -1600,7 +1600,7 @@ gnutls_openpgp_crt_set_preferred_key_id (gnutls_openpgp_crt_t key,
   ret = gnutls_openpgp_crt_get_subkey_idx (key, keyid);
   if (ret < 0)
     {
-      _gnutls_x509_log ("the requested subkey does not exist\n");
+      _gnutls_debug_log ("the requested subkey does not exist\n");
       gnutls_assert ();
       return ret;
     }
