@@ -75,55 +75,6 @@ gnutls_openpgp_privkey_deinit (gnutls_openpgp_privkey_t key)
   gnutls_free (key);
 }
 
-/*-
- * _gnutls_openpgp_privkey_cpy - This function copies a gnutls_openpgp_privkey_t structure
- * @dest: The structure where to copy
- * @src: The structure to be copied
- *
- * This function will copy an X.509 certificate structure.
- *
- * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise a
- *   negative error value.
- -*/
-int
-_gnutls_openpgp_privkey_cpy (gnutls_openpgp_privkey_t dest, gnutls_openpgp_privkey_t src)
-{
-  int ret;
-  size_t der_size=0;
-  opaque *der;
-  gnutls_datum_t tmp;
-
-  ret = gnutls_openpgp_privkey_export (src, GNUTLS_OPENPGP_FMT_RAW, NULL, 0, NULL, &der_size);
-  if (ret != GNUTLS_E_SHORT_MEMORY_BUFFER)
-    return gnutls_assert_val(ret);
-
-  der = gnutls_malloc (der_size);
-  if (der == NULL)
-    return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
-
-  ret = gnutls_openpgp_privkey_export (src, GNUTLS_OPENPGP_FMT_RAW, NULL, 0, der, &der_size);
-  if (ret < 0)
-    {
-      gnutls_assert ();
-      gnutls_free (der);
-      return ret;
-    }
-
-  tmp.data = der;
-  tmp.size = der_size;
-  ret = gnutls_openpgp_privkey_import (dest, &tmp, GNUTLS_OPENPGP_FMT_RAW, NULL, 0);
-
-  gnutls_free (der);
-
-  if (ret < 0)
-    return gnutls_assert_val(ret);
-
-  memcpy(dest->preferred_keyid, src->preferred_keyid, GNUTLS_OPENPGP_KEYID_SIZE);
-  dest->preferred_set = src->preferred_set;
-
-  return 0;
-}
-
 /**
  * gnutls_openpgp_privkey_sec_param:
  * @key: a key structure
@@ -317,7 +268,7 @@ _gnutls_openpgp_get_algo (int cdk_algo)
     algo = GNUTLS_PK_DSA;
   else
     {
-      _gnutls_x509_log ("Unknown OpenPGP algorithm %d\n", cdk_algo);
+      _gnutls_debug_log ("Unknown OpenPGP algorithm %d\n", cdk_algo);
       algo = GNUTLS_PK_UNKNOWN;
     }
 
@@ -1081,7 +1032,7 @@ gnutls_openpgp_privkey_export_rsa_raw (gnutls_openpgp_privkey_t pkey,
                                        gnutls_datum_t * d, gnutls_datum_t * p,
                                        gnutls_datum_t * q, gnutls_datum_t * u)
 {
-  uint8_t keyid[GNUTLS_OPENPGP_KEYID_SIZE];
+  gnutls_openpgp_keyid_t keyid;
   int ret;
 
   ret = gnutls_openpgp_privkey_get_key_id (pkey, keyid);
@@ -1117,7 +1068,7 @@ gnutls_openpgp_privkey_export_dsa_raw (gnutls_openpgp_privkey_t pkey,
                                        gnutls_datum_t * g, gnutls_datum_t * y,
                                        gnutls_datum_t * x)
 {
-  uint8_t keyid[GNUTLS_OPENPGP_KEYID_SIZE];
+  gnutls_openpgp_keyid_t keyid;
   int ret;
 
   ret = gnutls_openpgp_privkey_get_key_id (pkey, keyid);
@@ -1159,7 +1110,7 @@ gnutls_openpgp_privkey_export_subkey_rsa_raw (gnutls_openpgp_privkey_t pkey,
                                               gnutls_datum_t * q,
                                               gnutls_datum_t * u)
 {
-  uint8_t keyid[GNUTLS_OPENPGP_KEYID_SIZE];
+  gnutls_openpgp_keyid_t keyid;
   int ret;
 
   ret = gnutls_openpgp_privkey_get_subkey_id (pkey, idx, keyid);
@@ -1199,7 +1150,7 @@ gnutls_openpgp_privkey_export_subkey_dsa_raw (gnutls_openpgp_privkey_t pkey,
                                               gnutls_datum_t * y,
                                               gnutls_datum_t * x)
 {
-  uint8_t keyid[GNUTLS_OPENPGP_KEYID_SIZE];
+  gnutls_openpgp_keyid_t keyid;
   int ret;
 
   ret = gnutls_openpgp_privkey_get_subkey_id (pkey, idx, keyid);
@@ -1267,7 +1218,7 @@ gnutls_openpgp_privkey_set_preferred_key_id (gnutls_openpgp_privkey_t key,
   ret = gnutls_openpgp_privkey_get_subkey_idx (key, keyid);
   if (ret < 0)
     {
-      _gnutls_x509_log ("the requested subkey does not exist\n");
+      _gnutls_debug_log ("the requested subkey does not exist\n");
       gnutls_assert ();
       return ret;
     }
@@ -1302,7 +1253,7 @@ gnutls_openpgp_privkey_sign_hash (gnutls_openpgp_privkey_t key,
   bigint_t params[MAX_PRIV_PARAMS_SIZE];
   int params_size = MAX_PRIV_PARAMS_SIZE;
   int pk_algorithm;
-  uint8_t keyid[GNUTLS_OPENPGP_KEYID_SIZE];
+  gnutls_openpgp_keyid_t keyid;
 
   if (key == NULL)
     {
@@ -1377,7 +1328,7 @@ _gnutls_openpgp_privkey_decrypt_data (gnutls_openpgp_privkey_t key,
   bigint_t params[MAX_PRIV_PARAMS_SIZE];
   int params_size = MAX_PRIV_PARAMS_SIZE;
   int pk_algorithm;
-  uint8_t keyid[GNUTLS_OPENPGP_KEYID_SIZE];
+  gnutls_openpgp_keyid_t keyid;
 
   if (key == NULL)
     {
