@@ -34,6 +34,7 @@
 #include <x509_b64.h>
 #include "x509_int.h"
 #include <common.h>
+#include <time.h>
 
 struct oid2string
 {
@@ -257,7 +258,7 @@ _gnutls_x509_oid_data2string (const char *oid, void *value,
                           asn1_err)) != ASN1_SUCCESS)
     {
       gnutls_assert ();
-      _gnutls_x509_log ("asn1_der_decoding: %s:%s\n", str, asn1_err);
+      _gnutls_debug_log ("asn1_der_decoding: %s:%s\n", str, asn1_err);
       asn1_delete_structure (&tmpasn);
       return _gnutls_asn2err (result);
     }
@@ -1101,9 +1102,12 @@ _gnutls_x509_write_value (ASN1_TYPE c, const char *root,
                           const gnutls_datum_t * data, int str)
 {
   int result;
+  int asize;
   ASN1_TYPE c2 = ASN1_TYPE_EMPTY;
   gnutls_datum_t val = { NULL, 0 };
 
+  asize = data->size + 16;
+
   if (str)
     {
       /* Convert it to OCTET STRING
@@ -1319,7 +1323,7 @@ _gnutls_x509_get_pk_algorithm (ASN1_TYPE src, const char *src_name,
   algo = _gnutls_x509_oid2pk_algorithm (oid);
   if (algo == GNUTLS_PK_UNKNOWN)
     {
-      _gnutls_x509_log
+      _gnutls_debug_log
         ("%s: unknown public key algorithm: %s\n", __func__, oid);
     }
 
