@@ -139,7 +139,7 @@ int _gnutls_x509_get_signed_data (ASN1_TYPE src, const char *src_name,
 int _gnutls_x509_get_signature (ASN1_TYPE src, const char *src_name,
                                 gnutls_datum_t * signature);
 
-gnutls_digest_algorithm_t _gnutls_dsa_q_to_hash (bigint_t q, unsigned int* hash_len);
+gnutls_digest_algorithm_t _gnutls_dsa_q_to_hash (bigint_t q);
 
 int _gnutls_get_asn_mpis (ASN1_TYPE asn, const char *root,
                           bigint_t * params, int *params_size);
@@ -157,4 +157,7 @@ int pubkey_verify_sig (const gnutls_datum_t * tbs,
                        gnutls_pk_algorithm_t pk, bigint_t * issuer_params,
                        int issuer_params_size);
 
+int
+check_if_same_cert (gnutls_x509_crt_t cert1, gnutls_x509_crt_t cert2);
+
 #endif
