@@ -120,7 +120,7 @@ check_schema (const char *oid)
   if (strcmp (oid, PKCS12_PBE_RC2_40_SHA1_OID) == 0)
     return PKCS12_RC2_40_SHA1;
 
-  _gnutls_x509_log ("PKCS encryption schema OID '%s' is unsupported.\n", oid);
+  _gnutls_debug_log ("PKCS encryption schema OID '%s' is unsupported.\n", oid);
 
   return GNUTLS_E_UNKNOWN_CIPHER_TYPE;
 }
@@ -438,7 +438,7 @@ _gnutls_pkcs_flags_to_schema (unsigned int flags)
   else
     {
       gnutls_assert ();
-      _gnutls_x509_log
+      _gnutls_debug_log
         ("Selecting default encryption PKCS12_3DES_SHA1 (flags: %u).\n",
          flags);
       schema = PKCS12_3DES_SHA1;
@@ -1038,11 +1038,14 @@ _decode_pkcs8_dsa_key (ASN1_TYPE pkcs8_asn, gnutls_x509_privkey_t pkey)
   _gnutls_mpi_powm (pkey->params[3], pkey->params[2], pkey->params[4],
                     pkey->params[0]);
 
-  ret = _gnutls_asn1_encode_dsa (&pkey->key, pkey->params);
-  if (ret < 0)
+  if (!pkey->crippled)
     {
-      gnutls_assert ();
-      goto error;
+      ret = _gnutls_asn1_encode_dsa (&pkey->key, pkey->params);
+      if (ret < 0)
+        {
+          gnutls_assert ();
+          goto error;
+        }
     }
 
   pkey->params_size = DSA_PRIVATE_PARAMS;
@@ -1103,7 +1106,7 @@ decode_private_key_info (const gnutls_datum_t * der,
   else
     {
       gnutls_assert ();
-      _gnutls_x509_log
+      _gnutls_debug_log
         ("PKCS #8 private key OID '%s' is unsupported.\n", oid);
       result = GNUTLS_E_UNKNOWN_PK_ALGORITHM;
       goto error;
@@ -1270,7 +1273,7 @@ read_pbkdf2_params (ASN1_TYPE pbes2_asn,
   if (strcmp (oid, PBKDF2_OID) != 0)
     {
       gnutls_assert ();
-      _gnutls_x509_log
+      _gnutls_debug_log
         ("PKCS #8 key derivation OID '%s' is unsupported.\n", oid);
       return _gnutls_asn2err (result);
     }
@@ -1346,7 +1349,7 @@ read_pbkdf2_params (ASN1_TYPE pbes2_asn,
   /* We don't read the PRF. We only use the default.
    */
 
-  result = 0;
+  return 0;
 
 error:
   asn1_delete_structure (&pbkdf2_asn);
@@ -1470,7 +1473,7 @@ oid2cipher (const char *oid, gnutls_cipher_algorithm_t * algo)
       return 0;
     }
 
-  _gnutls_x509_log ("PKCS #8 encryption OID '%s' is unsupported.\n", oid);
+  _gnutls_debug_log ("PKCS #8 encryption OID '%s' is unsupported.\n", oid);
   return GNUTLS_E_UNKNOWN_CIPHER_TYPE;
 }
 
@@ -1554,8 +1557,8 @@ read_pbe_enc_params (ASN1_TYPE pbes2_asn,
       goto error;
     }
   _gnutls_hard_log ("IV.size: %d\n", params->iv_size);
-  
-  result = 0;
+
+  return 0;
 
 error:
   asn1_delete_structure (&pbe_asn);
