@@ -39,12 +39,6 @@
 #include "x509_int.h"
 #include <common.h>
 
-static int _gnutls_verify_certificate2 (gnutls_x509_crt_t cert,
-                                        const gnutls_x509_crt_t * trusted_cas,
-                                        int tcas_size, unsigned int flags,
-                                        unsigned int *output,
-                                        gnutls_x509_crt_t * issuer);
-
 static int is_crl_issuer (gnutls_x509_crl_t crl,
                           gnutls_x509_crt_t issuer_cert);
 
@@ -54,7 +48,7 @@ static int _gnutls_verify_crl2 (gnutls_x509_crl_t crl,
                                 unsigned int *output);
 
 /* Checks if two certs are identical.  Return 0 on match. */
-static int
+int
 check_if_same_cert (gnutls_x509_crt_t cert1, gnutls_x509_crt_t cert2)
 {
   gnutls_datum_t cert1bin = { NULL, 0 }, cert2bin =
@@ -255,12 +249,49 @@ cleanup:
 
 }
 
+/* Checks if the DN of two certificates is the same.
+ * Returns 1 if they match and zero if they don't match. Otherwise
+ * a negative value is returned to indicate error.
+ */
+int
+_gnutls_is_same_dn (gnutls_x509_crt_t cert1, gnutls_x509_crt_t cert2)
+{
+  gnutls_datum_t dn1 = { NULL, 0 }, dn2 =
+  {
+  NULL, 0};
+  int ret;
 
+  ret = gnutls_x509_crt_get_raw_dn (cert1, &dn1);
+  if (ret < 0)
+    {
+      gnutls_assert ();
+      goto cleanup;
+    }
+
+  ret = gnutls_x509_crt_get_raw_dn (cert2, &dn2);
+  if (ret < 0)
+    {
+      gnutls_assert ();
+      goto cleanup;
+    }
+
+  ret = _gnutls_x509_compare_raw_dn (&dn1, &dn2);
+
+cleanup:
+  _gnutls_free_datum (&dn1);
+  _gnutls_free_datum (&dn2);
+  return ret;
+}
+
+/* Finds an issuer of the certificate. If multiple issuers
+ * are present, returns one that is activated and not expired.
+ */
 static inline gnutls_x509_crt_t
 find_issuer (gnutls_x509_crt_t cert,
              const gnutls_x509_crt_t * trusted_cas, int tcas_size)
 {
-  int i;
+int i;
+gnutls_x509_crt_t issuer = NULL;
 
   /* this is serial search. 
    */
@@ -268,165 +299,25 @@ find_issuer (gnutls_x509_crt_t cert,
   for (i = 0; i < tcas_size; i++)
     {
       if (is_issuer (cert, trusted_cas[i]) == 1)
-        return trusted_cas[i];
-    }
-
-  gnutls_assert ();
-  return NULL;
-}
-
-
-
-/* 
- * Verifies the given certificate again a certificate list of
- * trusted CAs.
- *
- * Returns only 0 or 1. If 1 it means that the certificate 
- * was successfuly verified.
- *
- * 'flags': an OR of the gnutls_certificate_verify_flags enumeration.
- *
- * Output will hold some extra information about the verification
- * procedure. Issuer will hold the actual issuer from the trusted list.
- */
-static int
-_gnutls_verify_certificate2 (gnutls_x509_crt_t cert,
-                             const gnutls_x509_crt_t * trusted_cas,
-                             int tcas_size, unsigned int flags,
-                             unsigned int *output,
-                             gnutls_x509_crt_t * _issuer)
-{
-  gnutls_datum_t cert_signed_data = { NULL, 0 };
-  gnutls_datum_t cert_signature = { NULL, 0 };
-  gnutls_x509_crt_t issuer = NULL;
-  int issuer_version, result;
-
-  if (output)
-    *output = 0;
-
-  if (tcas_size >= 1)
-    issuer = find_issuer (cert, trusted_cas, tcas_size);
-  else
-    {
-      gnutls_assert ();
-      if (output)
-        *output |= GNUTLS_CERT_SIGNER_NOT_FOUND | GNUTLS_CERT_INVALID;
-      return 0;
-    }
-
-  /* issuer is not in trusted certificate
-   * authorities.
-   */
-  if (issuer == NULL)
-    {
-      if (output)
-        *output |= GNUTLS_CERT_SIGNER_NOT_FOUND | GNUTLS_CERT_INVALID;
-      gnutls_assert ();
-      return 0;
-    }
-
-  if (_issuer != NULL)
-    *_issuer = issuer;
-
-  issuer_version = gnutls_x509_crt_get_version (issuer);
-  if (issuer_version < 0)
-    {
-      gnutls_assert ();
-      return issuer_version;
-    }
-
-  if (!(flags & GNUTLS_VERIFY_DISABLE_CA_SIGN) &&
-      ((flags & GNUTLS_VERIFY_DO_NOT_ALLOW_X509_V1_CA_CRT)
-       || issuer_version != 1))
-    {
-      if (check_if_ca (cert, issuer, flags) == 0)
         {
-          gnutls_assert ();
-          if (output)
-            *output |= GNUTLS_CERT_SIGNER_NOT_CA | GNUTLS_CERT_INVALID;
-          return 0;
+          if (issuer == NULL) 
+            {
+              issuer = trusted_cas[i];
+            }
+          else
+            {
+              time_t now = time(0);
+
+              if (now < gnutls_x509_crt_get_expiration_time(trusted_cas[i]) && 
+                now >= gnutls_x509_crt_get_activation_time(trusted_cas[i]))
+                {
+                  issuer = trusted_cas[i];
+                }
+            }
         }
     }
 
-  result =
-    _gnutls_x509_get_signed_data (cert->cert, "tbsCertificate",
-                                  &cert_signed_data);
-  if (result < 0)
-    {
-      gnutls_assert ();
-      goto cleanup;
-    }
-
-  result =
-    _gnutls_x509_get_signature (cert->cert, "signature", &cert_signature);
-  if (result < 0)
-    {
-      gnutls_assert ();
-      goto cleanup;
-    }
-
-  result =
-    _gnutls_x509_verify_signature (&cert_signed_data, NULL, &cert_signature,
-                                   issuer);
-  if (result == GNUTLS_E_PK_SIG_VERIFY_FAILED)
-    {
-      gnutls_assert ();
-      /* error. ignore it */
-      if (output)
-        *output |= GNUTLS_CERT_INVALID;
-      result = 0;
-    }
-  else if (result < 0)
-    {
-      gnutls_assert();
-      goto cleanup;
-    }
-
-  /* If the certificate is not self signed check if the algorithms
-   * used are secure. If the certificate is self signed it doesn't
-   * really matter.
-   */
-  if (is_issuer (cert, cert) == 0)
-    {
-      int sigalg;
-
-      sigalg = gnutls_x509_crt_get_signature_algorithm (cert);
-
-      if (((sigalg == GNUTLS_SIGN_RSA_MD2) &&
-           !(flags & GNUTLS_VERIFY_ALLOW_SIGN_RSA_MD2)) ||
-          ((sigalg == GNUTLS_SIGN_RSA_MD5) &&
-           !(flags & GNUTLS_VERIFY_ALLOW_SIGN_RSA_MD5)))
-        {
-          if (output)
-            *output |= GNUTLS_CERT_INSECURE_ALGORITHM | GNUTLS_CERT_INVALID;
-          result = 0;
-        }
-    }
-
-cleanup:
-  _gnutls_free_datum (&cert_signed_data);
-  _gnutls_free_datum (&cert_signature);
-
-  return result;
-}
-
-/**
- * gnutls_x509_crt_check_issuer:
- * @cert: is the certificate to be checked
- * @issuer: is the certificate of a possible issuer
- *
- * This function will check if the given certificate was issued by the
- * given issuer.
- *
- * Returns: It will return true (1) if the given certificate is issued
- *   by the given issuer, and false (0) if not.  A negative value is
- *   returned in case of an error.
- **/
-int
-gnutls_x509_crt_check_issuer (gnutls_x509_crt_t cert,
-                              gnutls_x509_crt_t issuer)
-{
-  return is_issuer (cert, issuer);
+  return issuer;
 }
 
 static unsigned int
@@ -454,6 +345,193 @@ check_time (gnutls_x509_crt_t crt, time_t now)
   return 0;
 }
 
+/* 
+ * Verifies the given certificate again a certificate list of
+ * trusted CAs.
+ *
+ * Returns only 0 or 1. If 1 it means that the certificate 
+ * was successfuly verified.
+ *
+ * 'flags': an OR of the gnutls_certificate_verify_flags enumeration.
+ *
+ * Output will hold some extra information about the verification
+ * procedure. Issuer will hold the actual issuer from the trusted list.
+ */
+static int
+_gnutls_verify_certificate2 (gnutls_x509_crt_t cert,
+                             const gnutls_x509_crt_t * trusted_cas,
+                             int tcas_size, unsigned int flags,
+                             unsigned int *output,
+                             gnutls_x509_crt_t * _issuer,
+                             time_t now,
+                             gnutls_verify_output_function func)
+{
+  gnutls_datum_t cert_signed_data = { NULL, 0 };
+  gnutls_datum_t cert_signature = { NULL, 0 };
+  gnutls_x509_crt_t issuer = NULL;
+  int issuer_version, result;
+  unsigned int out = 0;
+
+  if (output)
+    *output = 0;
+
+  if (tcas_size >= 1)
+    issuer = find_issuer (cert, trusted_cas, tcas_size);
+  else
+    {
+      gnutls_assert ();
+      out = GNUTLS_CERT_SIGNER_NOT_FOUND | GNUTLS_CERT_INVALID;
+      if (output)
+        *output |= out;
+      result = 0;
+      goto cleanup;
+    }
+
+  /* issuer is not in trusted certificate
+   * authorities.
+   */
+  if (issuer == NULL)
+    {
+      out = GNUTLS_CERT_SIGNER_NOT_FOUND | GNUTLS_CERT_INVALID;
+      if (output)
+        *output |= out;
+      gnutls_assert ();
+      result = 0;
+      goto cleanup;
+    }
+
+  if (_issuer != NULL)
+    *_issuer = issuer;
+
+  issuer_version = gnutls_x509_crt_get_version (issuer);
+  if (issuer_version < 0)
+    {
+      gnutls_assert ();
+      return issuer_version;
+    }
+
+  if (!(flags & GNUTLS_VERIFY_DISABLE_CA_SIGN) &&
+      ((flags & GNUTLS_VERIFY_DO_NOT_ALLOW_X509_V1_CA_CRT)
+       || issuer_version != 1))
+    {
+      if (check_if_ca (cert, issuer, flags) == 0)
+        {
+          gnutls_assert ();
+          out = GNUTLS_CERT_SIGNER_NOT_CA | GNUTLS_CERT_INVALID;
+          if (output)
+            *output |= out;
+          result = 0;
+          goto cleanup;
+        }
+    }
+
+  result =
+    _gnutls_x509_get_signed_data (cert->cert, "tbsCertificate",
+                                  &cert_signed_data);
+  if (result < 0)
+    {
+      gnutls_assert ();
+      goto cleanup;
+    }
+
+  result =
+    _gnutls_x509_get_signature (cert->cert, "signature", &cert_signature);
+  if (result < 0)
+    {
+      gnutls_assert ();
+      goto cleanup;
+    }
+
+  result =
+    _gnutls_x509_verify_signature (&cert_signed_data, NULL, &cert_signature,
+                                   issuer);
+  if (result == GNUTLS_E_PK_SIG_VERIFY_FAILED)
+    {
+      gnutls_assert ();
+      out |= GNUTLS_CERT_INVALID;
+      /* error. ignore it */
+      if (output)
+        *output |= out;
+      result = 0;
+    }
+  else if (result < 0)
+    {
+      gnutls_assert();
+      goto cleanup;
+    }
+
+  /* If the certificate is not self signed check if the algorithms
+   * used are secure. If the certificate is self signed it doesn't
+   * really matter.
+   */
+  if (is_issuer (cert, cert) == 0)
+    {
+      int sigalg;
+
+      sigalg = gnutls_x509_crt_get_signature_algorithm (cert);
+
+      if (((sigalg == GNUTLS_SIGN_RSA_MD2) &&
+           !(flags & GNUTLS_VERIFY_ALLOW_SIGN_RSA_MD2)) ||
+          ((sigalg == GNUTLS_SIGN_RSA_MD5) &&
+           !(flags & GNUTLS_VERIFY_ALLOW_SIGN_RSA_MD5)))
+        {
+          out = GNUTLS_CERT_INSECURE_ALGORITHM | GNUTLS_CERT_INVALID;
+          if (output)
+            *output |= out;
+          result = 0;
+        }
+    }
+
+  /* Check activation/expiration times
+   */
+  if (!(flags & GNUTLS_VERIFY_DISABLE_TIME_CHECKS))
+    {
+      /* check the time of the issuer first */
+      if (!(flags & GNUTLS_VERIFY_DISABLE_TRUSTED_TIME_CHECKS))
+        {
+          out |= check_time (issuer, now);
+          if (out != 0)
+            {
+              result = 0;
+              if (output) *output |= out;
+            }
+        }
+
+      out |= check_time (cert, now);
+      if (out != 0)
+        {
+          result = 0;
+          if (output) *output |= out;
+        }
+    }
+
+cleanup:
+  if (result >= 0 && func) func(cert, issuer, NULL, out);
+  _gnutls_free_datum (&cert_signed_data);
+  _gnutls_free_datum (&cert_signature);
+
+  return result;
+}
+
+/**
+ * gnutls_x509_crt_check_issuer:
+ * @cert: is the certificate to be checked
+ * @issuer: is the certificate of a possible issuer
+ *
+ * This function will check if the given certificate was issued by the
+ * given issuer.
+ *
+ * Returns: It will return true (1) if the given certificate is issued
+ *   by the given issuer, and false (0) if not.  A negative value is
+ *   returned in case of an error.
+ **/
+int
+gnutls_x509_crt_check_issuer (gnutls_x509_crt_t cert,
+                              gnutls_x509_crt_t issuer)
+{
+  return is_issuer (cert, issuer);
+}
+
 /* Verify X.509 certificate chain.
  *
  * Note that the return value is an OR of GNUTLS_CERT_* elements.
@@ -461,17 +539,17 @@ check_time (gnutls_x509_crt_t crt, time_t now)
  * This function verifies a X.509 certificate list. The certificate
  * list should lead to a trusted certificate in order to be trusted.
  */
-static unsigned int
+unsigned int
 _gnutls_x509_verify_certificate (const gnutls_x509_crt_t * certificate_list,
                                  int clist_size,
                                  const gnutls_x509_crt_t * trusted_cas,
                                  int tcas_size,
-                                 const gnutls_x509_crl_t * CRLs,
-                                 int crls_size, unsigned int flags)
+                                 unsigned int flags, 
+                                 gnutls_verify_output_function func)
 {
   int i = 0, ret;
   unsigned int status = 0, output;
-  time_t now = gnutls_time (0);
+  time_t now = time (0);
   gnutls_x509_crt_t issuer = NULL;
 
   if (clist_size > 1)
@@ -519,9 +597,12 @@ _gnutls_x509_verify_certificate (const gnutls_x509_crt_t * certificate_list,
                   status |= check_time (trusted_cas[j], now);
                   if (status != 0)
                     {
+                      if (func) func(certificate_list[i], trusted_cas[j], NULL, status);
                       return status;
                     }
                 }
+
+              if (func) func(certificate_list[i], trusted_cas[j], NULL, status);
               clist_size = i;
               break;
             }
@@ -530,9 +611,11 @@ _gnutls_x509_verify_certificate (const gnutls_x509_crt_t * certificate_list,
     }
 
   if (clist_size == 0)
-    /* The certificate is already present in the trusted certificate list.
-     * Nothing to verify. */
-    return status;
+    {
+      /* The certificate is already present in the trusted certificate list.
+       * Nothing to verify. */
+      return status;
+    }
 
   /* Verify the last certificate in the certificate path
    * against the trusted CA certificate list.
@@ -540,9 +623,10 @@ _gnutls_x509_verify_certificate (const gnutls_x509_crt_t * certificate_list,
    * If no CAs are present returns CERT_INVALID. Thus works
    * in self signed etc certificates.
    */
+  output = 0;
   ret = _gnutls_verify_certificate2 (certificate_list[clist_size - 1],
                                      trusted_cas, tcas_size, flags, &output,
-                                     &issuer);
+                                     &issuer, now, func);
   if (ret == 0)
     {
       /* if the last certificate in the certificate
@@ -555,57 +639,11 @@ _gnutls_x509_verify_certificate (const gnutls_x509_crt_t * certificate_list,
       return status;
     }
 
-  /* Check for revoked certificates in the chain
-   */
-#ifdef ENABLE_PKI
-  for (i = 0; i < clist_size; i++)
-    {
-      ret = gnutls_x509_crt_check_revocation (certificate_list[i],
-                                              CRLs, crls_size);
-      if (ret == 1)
-        {                       /* revoked */
-          status |= GNUTLS_CERT_REVOKED;
-          status |= GNUTLS_CERT_INVALID;
-          return status;
-        }
-    }
-#endif
-
-
-  /* Check activation/expiration times
-   */
-  if (!(flags & GNUTLS_VERIFY_DISABLE_TIME_CHECKS))
-    {
-      /* check the time of the issuer first */
-      if (!(flags & GNUTLS_VERIFY_DISABLE_TRUSTED_TIME_CHECKS))
-        {
-          if (issuer == NULL)
-            {
-              gnutls_assert ();
-              return GNUTLS_E_INTERNAL_ERROR;
-            }
-
-          status |= check_time (issuer, now);
-          if (status != 0)
-            {
-              return status;
-            }
-        }
-
-      for (i = 0; i < clist_size; i++)
-        {
-          status |= check_time (certificate_list[i], now);
-          if (status != 0)
-            {
-              return status;
-            }
-        }
-    }
-
   /* Verify the certificate path (chain)
    */
   for (i = clist_size - 1; i > 0; i--)
     {
+      output = 0;
       if (i - 1 < 0)
         break;
 
@@ -617,8 +655,9 @@ _gnutls_x509_verify_certificate (const gnutls_x509_crt_t * certificate_list,
       if ((ret =
            _gnutls_verify_certificate2 (certificate_list[i - 1],
                                         &certificate_list[i], 1, flags,
-                                        NULL, NULL)) == 0)
+                                        &output, NULL, now, func)) == 0)
         {
+          status |= output;
           status |= GNUTLS_CERT_INVALID;
           return status;
         }
@@ -672,7 +711,7 @@ decode_ber_digest_info (const gnutls_datum_t * info,
   if (*hash == GNUTLS_MAC_UNKNOWN)
     {
 
-      _gnutls_x509_log ("verify.c: HASH OID: %s\n", str);
+      _gnutls_debug_log ("verify.c: HASH OID: %s\n", str);
 
       gnutls_assert ();
       asn1_delete_structure (&dinfo);
@@ -798,19 +837,16 @@ dsa_verify_sig (const gnutls_datum_t * text,
   gnutls_datum_t digest;
   digest_hd_st hd;
   gnutls_digest_algorithm_t algo;
-  unsigned int hash_len;
 
-  algo = _gnutls_dsa_q_to_hash (params[1], &hash_len);
+  algo = _gnutls_dsa_q_to_hash (params[1]);
   if (hash)
     {
       /* SHA1 or better allowed */
-      if (!hash->data || hash->size < hash_len)
+      if (!hash->data || hash->size != _gnutls_hash_get_algo_len(algo))
         {
           gnutls_assert();
           _gnutls_debug_log("Hash size (%d) does not correspond to hash %s", (int)hash->size, gnutls_mac_get_name(algo));
-          
-          if (hash->size != 20)
-            return GNUTLS_E_PK_SIG_VERIFY_FAILED;
+          return GNUTLS_E_INVALID_REQUEST;
         }
       digest = *hash;
     }
@@ -879,23 +915,20 @@ pubkey_verify_sig (const gnutls_datum_t * tbs,
 }
 
 gnutls_digest_algorithm_t
-_gnutls_dsa_q_to_hash (bigint_t q, unsigned int* hash_len)
+_gnutls_dsa_q_to_hash (bigint_t q)
 {
   int bits = _gnutls_mpi_get_nbits (q);
 
   if (bits <= 160)
     {
-      if (hash_len) *hash_len = 20;
       return GNUTLS_DIG_SHA1;
     }
   else if (bits <= 224)
     {
-      if (hash_len) *hash_len = 28;
-      return GNUTLS_DIG_SHA256;
+      return GNUTLS_DIG_SHA224;
     }
   else
     {
-      if (hash_len) *hash_len = 32;
       return GNUTLS_DIG_SHA256;
     }
 }
@@ -907,7 +940,7 @@ _gnutls_dsa_q_to_hash (bigint_t q, unsigned int* hash_len)
 int
 _gnutls_x509_verify_algorithm (gnutls_mac_algorithm_t * hash,
                                const gnutls_datum_t * signature,
-                               gnutls_pk_algorithm pk,
+                               gnutls_pk_algorithm_t pk,
                                bigint_t * issuer_params,
                                unsigned int issuer_params_size)
 {
@@ -921,7 +954,7 @@ _gnutls_x509_verify_algorithm (gnutls_mac_algorithm_t * hash,
     case GNUTLS_PK_DSA:
 
       if (hash)
-        *hash = _gnutls_dsa_q_to_hash (issuer_params[1], NULL);
+        *hash = _gnutls_dsa_q_to_hash (issuer_params[1]);
 
       ret = 0;
       break;
@@ -1085,6 +1118,8 @@ gnutls_x509_crt_list_verify (const gnutls_x509_crt_t * cert_list,
                              int CRL_list_length, unsigned int flags,
                              unsigned int *verify)
 {
+int i, ret;
+
   if (cert_list == NULL || cert_list_length == 0)
     return GNUTLS_E_NO_CERTIFICATE_FOUND;
 
@@ -1092,8 +1127,23 @@ gnutls_x509_crt_list_verify (const gnutls_x509_crt_t * cert_list,
    */
   *verify =
     _gnutls_x509_verify_certificate (cert_list, cert_list_length,
-                                     CA_list, CA_list_length, CRL_list,
-                                     CRL_list_length, flags);
+                                     CA_list, CA_list_length, 
+                                     flags, NULL);
+
+  /* Check for revoked certificates in the chain. 
+   */
+#ifdef ENABLE_PKI
+  for (i = 0; i < cert_list_length; i++)
+    {
+      ret = gnutls_x509_crt_check_revocation (cert_list[i],
+                                              CRL_list, CRL_list_length);
+      if (ret == 1)
+        {                       /* revoked */
+          *verify |= GNUTLS_CERT_REVOKED;
+          *verify |= GNUTLS_CERT_INVALID;
+        }
+    }
+#endif
 
   return 0;
 }
@@ -1122,7 +1172,8 @@ gnutls_x509_crt_verify (gnutls_x509_crt_t cert,
    */
   *verify =
     _gnutls_x509_verify_certificate (&cert, 1,
-                                     CA_list, CA_list_length, NULL, 0, flags);
+                                     CA_list, CA_list_length, 
+                                     flags, NULL);
   return 0;
 }
 
