@@ -84,7 +84,7 @@ int
 _gnutls_x509_crt_cpy (gnutls_x509_crt_t dest, gnutls_x509_crt_t src)
 {
   int ret;
-  size_t der_size = 0;
+  size_t der_size;
   opaque *der;
   gnutls_datum_t tmp;
 
@@ -123,6 +123,7 @@ _gnutls_x509_crt_cpy (gnutls_x509_crt_t dest, gnutls_x509_crt_t src)
     }
 
   return 0;
+
 }
 
 /**
@@ -247,52 +248,6 @@ cleanup:
   return result;
 }
 
-static int check_if_sorted(gnutls_x509_crt_t * crt, int nr)
-{
-char prev_dn[MAX_DN];
-char dn[MAX_DN];
-size_t prev_dn_size, dn_size;
-int i, ret;
-
-  /* check if the X.509 list is ordered */
-  if (nr > 1)
-    {
-
-      for (i=0;i<nr;i++)
-        {
-          if (i>0)
-            {
-              dn_size = sizeof(dn);
-              ret = gnutls_x509_crt_get_dn(crt[i], dn, &dn_size);
-              if (ret < 0)
-                {
-                  ret = gnutls_assert_val(ret);
-                  goto cleanup;
-                }
-              
-              if (dn_size != prev_dn_size || memcmp(dn, prev_dn, dn_size) != 0)
-                {
-                  ret = gnutls_assert_val(GNUTLS_E_CERTIFICATE_LIST_UNSORTED);
-                  goto cleanup;
-                }
-            }
-
-          prev_dn_size = sizeof(prev_dn);
-          ret = gnutls_x509_crt_get_issuer_dn(crt[i], prev_dn, &prev_dn_size);
-          if (ret < 0)
-            {
-              ret = gnutls_assert_val(ret);
-              goto cleanup;
-            }
-        }
-    }
-
-  ret = 0;
-
-cleanup:
-  return ret;
-}
-
 
 /**
  * gnutls_x509_crt_get_issuer_dn:
@@ -1048,7 +1003,7 @@ _gnutls_parse_general_name (ASN1_TYPE src, const char *src_name,
               size_t orig_name_size = *name_size;
 
               result = asn1_create_element
-                (_gnutls_get_pkix (), "PKIX1.UTF8String", &c2);
+                (_gnutls_get_pkix (), "PKIX1.XmppAddr", &c2);
               if (result != ASN1_SUCCESS)
                 {
                   gnutls_assert ();
@@ -2422,25 +2377,17 @@ gnutls_x509_crt_get_key_id (gnutls_x509_crt_t crt, unsigned int flags,
 
 #ifdef ENABLE_PKI
 
-/**
- * gnutls_x509_crt_check_revocation:
- * @cert: should contain a #gnutls_x509_crt_t structure
- * @crl_list: should contain a list of gnutls_x509_crl_t structures
- * @crl_list_length: the length of the crl_list
- *
- * This function will return check if the given certificate is
- * revoked.  It is assumed that the CRLs have been verified before.
- *
- * Returns: 0 if the certificate is NOT revoked, and 1 if it is.  A
- * negative value is returned on error.
- **/
+/* This is exactly as gnutls_x509_crt_check_revocation() except that
+ * it calls func.
+ */
 int
-gnutls_x509_crt_check_revocation (gnutls_x509_crt_t cert,
+_gnutls_x509_crt_check_revocation (gnutls_x509_crt_t cert,
                                   const gnutls_x509_crl_t * crl_list,
-                                  int crl_list_length)
+                                  int crl_list_length,
+                                  gnutls_verify_output_function func)
 {
-  opaque serial[64];
-  opaque cert_serial[64];
+  opaque serial[128];
+  opaque cert_serial[128];
   size_t serial_size, cert_serial_size;
   int ncerts, ret, i, j;
   gnutls_datum_t dn1, dn2;
@@ -2520,15 +2467,38 @@ gnutls_x509_crt_check_revocation (gnutls_x509_crt_t cert,
               if (memcmp (serial, cert_serial, serial_size) == 0)
                 {
                   /* serials match */
+                  if (func) func(cert, NULL, crl_list[j], GNUTLS_CERT_REVOKED|GNUTLS_CERT_INVALID);
                   return 1;     /* revoked! */
                 }
             }
         }
+      if (func) func(cert, NULL, crl_list[j], 0);
 
     }
   return 0;                     /* not revoked. */
 }
 
+
+/**
+ * gnutls_x509_crt_check_revocation:
+ * @cert: should contain a #gnutls_x509_crt_t structure
+ * @crl_list: should contain a list of gnutls_x509_crl_t structures
+ * @crl_list_length: the length of the crl_list
+ *
+ * This function will return check if the given certificate is
+ * revoked.  It is assumed that the CRLs have been verified before.
+ *
+ * Returns: 0 if the certificate is NOT revoked, and 1 if it is.  A
+ * negative value is returned on error.
+ **/
+int
+gnutls_x509_crt_check_revocation (gnutls_x509_crt_t cert,
+                                  const gnutls_x509_crl_t * crl_list,
+                                  int crl_list_length)
+{
+  return _gnutls_x509_crt_check_revocation(cert, crl_list, crl_list_length, NULL);
+}
+
 /**
  * gnutls_x509_crt_get_verify_algorithm:
  * @crt: Holds the certificate
@@ -3108,6 +3078,63 @@ cleanup:
 }
 
 #endif
+/**
+ * gnutls_x509_crt_list_import2:
+ * @certs: The structures to store the parsed certificate. Must not be initialized.
+ * @size: It will contain the size of the list.
+ * @data: The PEM encoded certificate.
+ * @format: One of DER or PEM.
+ * @flags: must be zero or an OR'd sequence of gnutls_certificate_import_flags.
+ *
+ * This function will convert the given PEM encoded certificate list
+ * to the native gnutls_x509_crt_t format. The output will be stored
+ * in @certs.  They will be automatically initialized.
+ *
+ * If the Certificate is PEM encoded it should have a header of "X509
+ * CERTIFICATE", or "CERTIFICATE".
+ *
+ * Returns: the number of certificates read or a negative error value.
+ **/
+int
+gnutls_x509_crt_list_import2 (gnutls_x509_crt_t ** certs,
+                             unsigned int * size,
+                             const gnutls_datum_t * data,
+                             gnutls_x509_crt_fmt_t format, unsigned int flags)
+{
+unsigned int init = 1024;
+int ret;
+
+  *certs = gnutls_malloc(sizeof(gnutls_x509_crt_t)*init);
+  if (*certs == NULL)
+    {
+      gnutls_assert();
+      return GNUTLS_E_MEMORY_ERROR;
+    }
+
+  ret = gnutls_x509_crt_list_import(*certs, &init, data, format, GNUTLS_X509_CRT_LIST_IMPORT_FAIL_IF_EXCEED);
+  if (ret == GNUTLS_E_SHORT_MEMORY_BUFFER)
+    {
+      *certs = gnutls_realloc_fast(*certs, sizeof(gnutls_x509_crt_t)*init);
+      if (*certs == NULL)
+        {
+          gnutls_assert();
+          return GNUTLS_E_MEMORY_ERROR;
+        }
+      
+      ret = gnutls_x509_crt_list_import(*certs, &init, data, format, flags);
+    }
+
+  if (ret < 0)
+    {
+      gnutls_free(*certs);
+      *certs = NULL;
+      return ret;
+    }
+
+  *size = init;
+  return 0;
+}
+
 
 /**
  * gnutls_x509_crt_list_import:
@@ -3121,12 +3148,6 @@ cleanup:
  * to the native gnutls_x509_crt_t format. The output will be stored
  * in @certs.  They will be automatically initialized.
  *
- * The flag %GNUTLS_X509_CRT_LIST_IMPORT_FAIL_IF_EXCEED will cause
- * import to fail if the certificates in the provided buffer are more
- * than the available structures. The %GNUTLS_X509_CRT_LIST_FAIL_IF_UNSORTED
- * flag will cause the function to fail if the provided list is not
- * sorted from subject to issuer.
- *
  * If the Certificate is PEM encoded it should have a header of "X509
  * CERTIFICATE", or "CERTIFICATE".
  *
@@ -3181,7 +3202,10 @@ gnutls_x509_crt_list_import (gnutls_x509_crt_t * certs,
                   PEM_CERT_SEP2, sizeof (PEM_CERT_SEP2) - 1);
 
   if (ptr == NULL)
-    return gnutls_assert_val(GNUTLS_E_NO_CERTIFICATE_FOUND);
+    {
+      gnutls_assert ();
+      return GNUTLS_E_BASE64_DECODING_ERROR;
+    }
 
   count = 0;
 
@@ -3243,16 +3267,6 @@ gnutls_x509_crt_list_import (gnutls_x509_crt_t * certs,
 
   *cert_max = count;
 
-  if (flags & GNUTLS_X509_CRT_LIST_FAIL_IF_UNSORTED)
-    {
-      ret = check_if_sorted(certs, *cert_max);
-      if (ret < 0)
-        {
-          gnutls_assert();
-          goto error;
-        }
-    }
-
   if (nocopy == 0)
     return count;
   else
