@@ -46,6 +46,7 @@
 #define HASH_OID_MD5 "1.2.840.113549.2.5"
 #define HASH_OID_MD2 "1.2.840.113549.2.2"
 #define HASH_OID_RMD160 "1.3.36.3.2.1"
+#define HASH_OID_SHA224 "2.16.840.1.101.3.4.2.4"
 #define HASH_OID_SHA256 "2.16.840.1.101.3.4.2.1"
 #define HASH_OID_SHA384 "2.16.840.1.101.3.4.2.2"
 #define HASH_OID_SHA512 "2.16.840.1.101.3.4.2.3"
@@ -119,6 +120,12 @@ typedef struct gnutls_x509_privkey_int
 
   gnutls_pk_algorithm_t pk_algorithm;
 
+  /* The crippled keys will not use the ASN1_TYPE key.  The encoding
+   * will only be performed at the export phase, to optimize copying
+   * etc. Cannot be used with the exported API (used internally only).
+   */
+  int crippled;
+
   ASN1_TYPE key;
 } gnutls_x509_privkey_int;
 
@@ -391,4 +398,21 @@ int _gnutls_x509_crq_set_extension (gnutls_x509_crq_t crq,
                                     const gnutls_datum_t * ext_data,
                                     unsigned int critical);
 
+unsigned int
+_gnutls_x509_verify_certificate (const gnutls_x509_crt_t * certificate_list,
+                                 int clist_size,
+                                 const gnutls_x509_crt_t * trusted_cas,
+                                 int tcas_size,
+                                 unsigned int flags,
+                                 gnutls_verify_output_function func);
+
+int
+_gnutls_is_same_dn (gnutls_x509_crt_t cert1, gnutls_x509_crt_t cert2);
+
+int
+_gnutls_x509_crt_check_revocation (gnutls_x509_crt_t cert,
+                                  const gnutls_x509_crl_t * crl_list,
+                                  int crl_list_length,
+                                  gnutls_verify_output_function func);
+
 #endif
