@@ -1,6 +1,5 @@
 /*
- * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010,
- *   2011 Free Software Foundation, Inc.
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
  *
  * This file is part of GnuTLS.
  *
@@ -22,12 +21,15 @@
 #include <config.h>
 
 #include <gnutls/gnutls.h>
+#include <gnutls/extra.h>
 #include <gnutls/x509.h>
 #include <gnutls/openpgp.h>
 #include <gnutls/pkcs12.h>
 #include <gnutls/pkcs11.h>
 #include <gnutls/abstract.h>
 
+#include <gcrypt.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -46,12 +48,9 @@
 #include <version-etc.h>
 
 #include <certtool-cfg.h>
-#include <p11common.h>
 #include "certtool-gaa.h"
 #include "certtool-common.h"
 
-#define SIGN_HASH GNUTLS_DIG_SHA1
-
 static void print_crl_info (gnutls_x509_crl_t crl, FILE * out);
 void pkcs7_info (void);
 void crq_info (void);
@@ -59,9 +58,9 @@ void smime_to_pkcs7 (void);
 void pkcs12_info (void);
 void generate_pkcs12 (common_info_st *);
 void generate_pkcs8 (common_info_st *);
-void verify_chain (void);
+static void verify_chain (void);
 void verify_crl (common_info_st * cinfo);
-void pubkey_info (gnutls_x509_crt crt, common_info_st *);
+void pubkey_info (gnutls_x509_crt_t crt, common_info_st *);
 void pgp_privkey_info (void);
 void pgp_ring_info (void);
 void certificate_info (int, common_info_st *);
@@ -73,6 +72,7 @@ void generate_self_signed (common_info_st *);
 void generate_request (common_info_st *);
 static void print_certificate_info (gnutls_x509_crt_t crt, FILE * out,
                                     unsigned int all);
+static void verify_certificate (common_info_st * cinfo);
 
 static void print_hex_datum (gnutls_datum_t * dat);
 
@@ -363,13 +363,12 @@ generate_private_key (void)
 
 
 static gnutls_x509_crt_t
-generate_certificate (gnutls_privkey_t * ret_key,
+generate_certificate (gnutls_x509_privkey_t * ret_key,
                       gnutls_x509_crt_t ca_crt, int proxy,
                       common_info_st * cinfo)
 {
   gnutls_x509_crt_t crt;
-  gnutls_privkey_t key = NULL;
-  gnutls_pubkey_t pubkey;
+  gnutls_x509_privkey_t key = NULL;
   size_t size;
   int ret;
   int client;
@@ -388,8 +387,6 @@ generate_certificate (gnutls_privkey_t * ret_key,
     {
 
       key = load_private_key (1, cinfo);
-      
-      pubkey = load_public_key_or_import (1, key, cinfo);
 
       if (!batch)
         fprintf (stderr,
@@ -426,7 +423,7 @@ generate_certificate (gnutls_privkey_t * ret_key,
           get_pkcs9_email_crt_set (crt);
         }
 
-      result = gnutls_x509_crt_set_pubkey (crt, pubkey);
+      result = gnutls_x509_crt_set_key (crt, key);
       if (result < 0)
         error (EXIT_FAILURE, 0, "set_key: %s", gnutls_strerror (result));
     }
@@ -771,7 +768,7 @@ generate_crl (gnutls_x509_crt_t ca_crt, common_info_st * cinfo)
 }
 
 static gnutls_digest_algorithm_t
-get_dig (gnutls_x509_crt crt)
+get_dig (gnutls_x509_crt_t crt)
 {
   gnutls_digest_algorithm_t dig;
   gnutls_pubkey_t pubkey;
@@ -807,7 +804,7 @@ void
 generate_self_signed (common_info_st * cinfo)
 {
   gnutls_x509_crt_t crt;
-  gnutls_privkey_t key;
+  gnutls_x509_privkey_t key;
   size_t size;
   int result;
   const char *uri;
@@ -834,7 +831,7 @@ generate_self_signed (common_info_st * cinfo)
 
   fprintf (stderr, "\n\nSigning certificate...\n");
 
-  result = gnutls_x509_crt_privkey_sign (crt, crt, key, get_dig (crt), 0);
+  result = gnutls_x509_crt_sign2 (crt, crt, key, get_dig (crt), 0);
   if (result < 0)
     error (EXIT_FAILURE, 0, "crt_sign: %s", gnutls_strerror (result));
 
@@ -846,17 +843,17 @@ generate_self_signed (common_info_st * cinfo)
   fwrite (buffer, 1, size, outfile);
 
   gnutls_x509_crt_deinit (crt);
-  gnutls_privkey_deinit (key);
+  gnutls_x509_privkey_deinit (key);
 }
 
 static void
 generate_signed_certificate (common_info_st * cinfo)
 {
   gnutls_x509_crt_t crt;
-  gnutls_privkey_t key;
+  gnutls_x509_privkey_t key;
   size_t size;
   int result;
-  gnutls_privkey_t ca_key;
+  gnutls_x509_privkey_t ca_key;
   gnutls_x509_crt_t ca_crt;
 
   fprintf (stderr, "Generating a signed certificate...\n");
@@ -876,7 +873,7 @@ generate_signed_certificate (common_info_st * cinfo)
 
   fprintf (stderr, "\n\nSigning certificate...\n");
 
-  result = gnutls_x509_crt_privkey_sign (crt, ca_crt, ca_key, get_dig (ca_crt), 0);
+  result = gnutls_x509_crt_sign2 (crt, ca_crt, ca_key, get_dig (ca_crt), 0);
   if (result < 0)
     error (EXIT_FAILURE, 0, "crt_sign: %s", gnutls_strerror (result));
 
@@ -888,15 +885,14 @@ generate_signed_certificate (common_info_st * cinfo)
   fwrite (buffer, 1, size, outfile);
 
   gnutls_x509_crt_deinit (crt);
-  gnutls_privkey_deinit (key);
-  gnutls_privkey_deinit(ca_key);
+  gnutls_x509_privkey_deinit (key);
 }
 
 static void
 generate_proxy_certificate (common_info_st * cinfo)
 {
   gnutls_x509_crt_t crt, eecrt;
-  gnutls_privkey_t key, eekey;
+  gnutls_x509_privkey_t key, eekey;
   size_t size;
   int result;
 
@@ -911,7 +907,7 @@ generate_proxy_certificate (common_info_st * cinfo)
 
   fprintf (stderr, "\n\nSigning certificate...\n");
 
-  result = gnutls_x509_crt_privkey_sign (crt, eecrt, eekey, get_dig (eecrt), 0);
+  result = gnutls_x509_crt_sign2 (crt, eecrt, eekey, get_dig (eecrt), 0);
   if (result < 0)
     error (EXIT_FAILURE, 0, "crt_sign: %s", gnutls_strerror (result));
 
@@ -922,10 +918,8 @@ generate_proxy_certificate (common_info_st * cinfo)
 
   fwrite (buffer, 1, size, outfile);
 
-  gnutls_x509_crt_deinit (eecrt);
   gnutls_x509_crt_deinit (crt);
-  gnutls_privkey_deinit (key);
-  gnutls_privkey_deinit (eekey);
+  gnutls_x509_privkey_deinit (key);
 }
 
 static void
@@ -933,7 +927,8 @@ generate_signed_crl (common_info_st * cinfo)
 {
   gnutls_x509_crl_t crl;
   int result;
-  gnutls_privkey_t ca_key;
+  gnutls_x509_privkey_t ca_key;
+  gnutls_privkey_t ca_pkey;
   gnutls_x509_crt_t ca_crt;
 
   fprintf (stderr, "Generating a signed CRL...\n");
@@ -942,14 +937,23 @@ generate_signed_crl (common_info_st * cinfo)
   ca_crt = load_ca_cert (cinfo);
   crl = generate_crl (ca_crt, cinfo);
 
+  result = gnutls_privkey_init(&ca_pkey);
+  if (result < 0)
+    error (EXIT_FAILURE, 0, "privkey_init: %s", gnutls_strerror (result));
+
+  result = gnutls_privkey_import_x509(ca_pkey, ca_key, 0);
+  if (result < 0)
+    error (EXIT_FAILURE, 0, "privkey_init: %s", gnutls_strerror (result));
+
   fprintf (stderr, "\n");
-  result = gnutls_x509_crl_privkey_sign(crl, ca_crt, ca_key, SIGN_HASH, 0);
+
+  result = gnutls_x509_crl_privkey_sign(crl, ca_crt, ca_pkey, GNUTLS_DIG_SHA1, 0);
   if (result < 0)
     error (EXIT_FAILURE, 0, "crl_privkey_sign: %s", gnutls_strerror (result));
 
   print_crl_info (crl, stderr);
 
-  gnutls_privkey_deinit( ca_key);
+  gnutls_privkey_deinit( ca_pkey);
   gnutls_x509_crl_deinit (crl);
 }
 
@@ -959,7 +963,7 @@ update_signed_certificate (common_info_st * cinfo)
   gnutls_x509_crt_t crt;
   size_t size;
   int result;
-  gnutls_privkey_t ca_key;
+  gnutls_x509_privkey_t ca_key;
   gnutls_x509_crt_t ca_crt;
   int days;
   time_t tim = time (NULL);
@@ -982,7 +986,7 @@ update_signed_certificate (common_info_st * cinfo)
 
   fprintf (stderr, "\n\nSigning certificate...\n");
 
-  result = gnutls_x509_crt_privkey_sign (crt, ca_crt, ca_key, get_dig (ca_crt), 0);
+  result = gnutls_x509_crt_sign2 (crt, ca_crt, ca_key, get_dig (ca_crt), 0);
   if (result < 0)
     error (EXIT_FAILURE, 0, "crt_sign: %s", gnutls_strerror (result));
 
@@ -1077,9 +1081,8 @@ gaa_parser (int argc, char **argv)
   if ((ret = gnutls_global_init ()) < 0)
     error (EXIT_FAILURE, 0, "global_init: %s", gnutls_strerror (ret));
 
-#ifdef ENABLE_PKCS11
-  pkcs11_common();
-#endif
+  if ((ret = gnutls_global_init_extra ()) < 0)
+    error (EXIT_FAILURE, 0, "global_init_extra: %s", gnutls_strerror (ret));
 
   memset (&cinfo, 0, sizeof (cinfo));
   cinfo.privkey = info.privkey;
@@ -1114,6 +1117,9 @@ gaa_parser (int argc, char **argv)
     case ACTION_VERIFY_CHAIN:
       verify_chain ();
       break;
+    case ACTION_VERIFY:
+      verify_certificate (&cinfo);
+      break;
     case ACTION_PRIVKEY_INFO:
       privkey_info ();
       break;
@@ -1176,9 +1182,7 @@ gaa_parser (int argc, char **argv)
     }
   fclose (outfile);
 
-#ifdef ENABLE_PKCS11
   gnutls_pkcs11_deinit ();
-#endif
   gnutls_global_deinit ();
 }
 
@@ -1314,7 +1318,7 @@ void
 pgp_privkey_info (void)
 {
   gnutls_openpgp_privkey_t key;
-  unsigned char keyid[GNUTLS_OPENPGP_KEYID_SIZE];
+  gnutls_openpgp_keyid_t keyid;
   size_t size;
   int ret, i, subkeys;
   gnutls_datum_t pem;
@@ -1764,8 +1768,7 @@ void
 generate_request (common_info_st * cinfo)
 {
   gnutls_x509_crq_t crq;
-  gnutls_x509_privkey_t xkey;
-  gnutls_pubkey_t pubkey;
+  gnutls_x509_privkey_t key;
   gnutls_privkey_t pkey;
   int ret, ca_status, path_len;
   const char *pass;
@@ -1777,26 +1780,23 @@ generate_request (common_info_st * cinfo)
   if (ret < 0)
     error (EXIT_FAILURE, 0, "crq_init: %s", gnutls_strerror (ret));
 
+  ret = gnutls_privkey_init (&pkey);
+  if (ret < 0)
+    error (EXIT_FAILURE, 0, "privkey_init: %s", gnutls_strerror (ret));
 
   /* Load the private key.
    */
-  pkey = load_private_key (0, cinfo);
-  if (!pkey)
+  key = load_private_key (0, cinfo);
+  if (!key)
     {
-      ret = gnutls_privkey_init (&pkey);
-      if (ret < 0)
-        error (EXIT_FAILURE, 0, "privkey_init: %s", gnutls_strerror (ret));
+      key = generate_private_key_int ();
 
-      xkey = generate_private_key_int ();
-
-      print_private_key (xkey);
-
-      ret = gnutls_privkey_import_x509(pkey, xkey, GNUTLS_PRIVKEY_IMPORT_AUTO_RELEASE);
-      if (ret < 0)
-        error (EXIT_FAILURE, 0, "privkey_import_x509: %s", gnutls_strerror (ret));
+      print_private_key (key);
     }
 
-  pubkey = load_public_key_or_import (1, pkey, cinfo);
+  ret = gnutls_privkey_import_x509(pkey, key, 0);
+  if (ret < 0)
+    error (EXIT_FAILURE, 0, "privkey_import_x509: %s", gnutls_strerror (ret));
 
   /* Set the DN.
    */
@@ -1915,11 +1915,11 @@ generate_request (common_info_st * cinfo)
         }
     }
 
-  ret = gnutls_x509_crq_set_pubkey (crq, pubkey);
+  ret = gnutls_x509_crq_set_key (crq, key);
   if (ret < 0)
     error (EXIT_FAILURE, 0, "set_key: %s", gnutls_strerror (ret));
 
-  ret = gnutls_x509_crq_privkey_sign (crq, pkey, SIGN_HASH, 0);
+  ret = gnutls_x509_crq_privkey_sign (crq, pkey, GNUTLS_DIG_SHA1, 0);
   if (ret < 0)
     error (EXIT_FAILURE, 0, "sign: %s", gnutls_strerror (ret));
 
@@ -1927,261 +1927,177 @@ generate_request (common_info_st * cinfo)
 
   gnutls_x509_crq_deinit (crq);
   gnutls_privkey_deinit( pkey);
-  gnutls_pubkey_deinit( pubkey);
+  gnutls_x509_privkey_deinit (key);
 
 }
 
-static void print_verification_res (gnutls_x509_crt_t crt,
-                                    gnutls_x509_crt_t issuer,
-                                    gnutls_x509_crl_t * crl_list,
-                                    int crl_list_size, unsigned int flags);
+static void print_verification_res (FILE* outfile, unsigned int output);
 
-#define CERT_SEP "-----BEGIN CERT"
-#define CRL_SEP "-----BEGIN X509 CRL"
-static int
-_verify_x509_mem (const void *cert, int cert_size)
+static int detailed_verification(gnutls_x509_crt_t cert,
+    gnutls_x509_crt_t issuer, gnutls_x509_crl_t crl, 
+    unsigned int verification_output)
 {
-  const char *ptr;
-  int ret, i;
   char name[512];
+  char tmp[255];
   char issuer_name[512];
   size_t name_size;
   size_t issuer_name_size;
-  gnutls_datum_t tmp;
-  gnutls_x509_crt_t *x509_cert_list = NULL;
-  gnutls_x509_crl_t *x509_crl_list = NULL;
-  int x509_ncerts, x509_ncrls;
+  int ret;
 
 
-  /* Decode the CA certificate
-   */
-
-  /* Decode the CRL list
-   */
-  ptr = cert;
-
-  i = 1;
-
-  if (strstr (ptr, CRL_SEP) != NULL)    /* if CRLs exist */
-    do
-      {
-        x509_crl_list =
-          (gnutls_x509_crl_t *) realloc (x509_crl_list,
-                                         i * sizeof (gnutls_x509_crl_t));
-        if (x509_crl_list == NULL)
-          error (EXIT_FAILURE, 0, "memory error");
-
-        tmp.data = (char *) ptr;
-        tmp.size = cert_size;
-        tmp.size -=
-          (unsigned int) ((unsigned char *) ptr - (unsigned char *) cert);
-
-        ret = gnutls_x509_crl_init (&x509_crl_list[i - 1]);
-        if (ret < 0)
-          error (EXIT_FAILURE, 0, "error parsing CRL[%d]: %s", i,
-                 gnutls_strerror (ret));
-
-        ret = gnutls_x509_crl_import (x509_crl_list[i - 1], &tmp,
-                                      GNUTLS_X509_FMT_PEM);
-        if (ret < 0)
-          error (EXIT_FAILURE, 0, "error parsing CRL[%d]: %s", i,
-                 gnutls_strerror (ret));
-
-        /* now we move ptr after the pem header */
-        ptr = strstr (ptr, CRL_SEP);
-        if (ptr != NULL)
-          ptr++;
-
-        i++;
-      }
-    while ((ptr = strstr (ptr, CRL_SEP)) != NULL);
-
-  x509_ncrls = i - 1;
-
-
-  /* Decode the certificate chain. 
-   */
-  ptr = cert;
-
-  i = 1;
-
-  do
-    {
-      x509_cert_list =
-        (gnutls_x509_crt_t *) realloc (x509_cert_list,
-                                       i * sizeof (gnutls_x509_crt_t));
-      if (x509_cert_list == NULL)
-        error (EXIT_FAILURE, 0, "memory error");
-
-
-      tmp.data = (char *) ptr;
-      tmp.size = cert_size;
-      tmp.size -=
-        (unsigned int) ((unsigned char *) ptr - (unsigned char *) cert);
-
-      ret = gnutls_x509_crt_init (&x509_cert_list[i - 1]);
-      if (ret < 0)
-        error (EXIT_FAILURE, 0, "error parsing certificate[%d]: %s", i,
-               gnutls_strerror (ret));
-
-      ret =
-        gnutls_x509_crt_import (x509_cert_list[i - 1], &tmp,
-                                GNUTLS_X509_FMT_PEM);
-      if (ret < 0)
-        error (EXIT_FAILURE, 0, "error parsing certificate[%d]: %s", i,
-               gnutls_strerror (ret));
-
-
-      if (i - 1 != 0)
-        {
-          /* verify the previous certificate using this one 
-           * as CA.
-           */
-
-          name_size = sizeof (name);
-          ret =
-            gnutls_x509_crt_get_dn (x509_cert_list[i - 2], name, &name_size);
-          if (ret < 0)
-            error (EXIT_FAILURE, 0, "get_dn: %s", gnutls_strerror (ret));
-
-          fprintf (outfile, "Certificate[%d]: %s\n", i - 2, name);
-
-          /* print issuer 
-           */
-          issuer_name_size = sizeof (issuer_name);
-          ret =
-            gnutls_x509_crt_get_issuer_dn (x509_cert_list[i - 2],
-                                           issuer_name, &issuer_name_size);
-          if (ret < 0)
-            error (EXIT_FAILURE, 0, "get_issuer_dn: %s",
-                   gnutls_strerror (ret));
-
-          fprintf (outfile, "\tIssued by: %s\n", issuer_name);
-
-          /* Get the Issuer's name
-           */
-          name_size = sizeof (name);
-          ret =
-            gnutls_x509_crt_get_dn (x509_cert_list[i - 1], name, &name_size);
-          if (ret < 0)
-            error (EXIT_FAILURE, 0, "get_dn: %s", gnutls_strerror (ret));
-
-          fprintf (outfile, "\tVerifying against certificate[%d].\n", i - 1);
-
-          if (strcmp (issuer_name, name) != 0)
-            {
-              fprintf (stderr, "Error: Issuer's name: %s\n", name);
-              error (EXIT_FAILURE, 0,
-                     "issuer name does not match the next certificate");
-            }
-
-          fprintf (outfile, "\tVerification output: ");
-          print_verification_res (x509_cert_list[i - 2],
-                                  x509_cert_list[i - 1], x509_crl_list,
-                                  x509_ncrls,
-                                  GNUTLS_VERIFY_DO_NOT_ALLOW_SAME);
-          fprintf (outfile, ".\n\n");
-
-        }
-
-
-      /* now we move ptr after the pem header 
-       */
-      ptr = strstr (ptr, CERT_SEP);
-      if (ptr != NULL)
-        ptr++;
-
-      i++;
-    }
-  while ((ptr = strstr (ptr, CERT_SEP)) != NULL);
-
-  x509_ncerts = i - 1;
-
-  /* The last certificate in the list will be used as
-   * a CA (should be self signed).
-   */
-  name_size = sizeof (name);
-  ret = gnutls_x509_crt_get_dn (x509_cert_list[x509_ncerts - 1], name,
-                                &name_size);
-  if (ret < 0)
-    error (EXIT_FAILURE, 0, "get_dn: %s", gnutls_strerror (ret));
-
-  fprintf (outfile, "Certificate[%d]: %s\n", x509_ncerts - 1, name);
-
-  /* print issuer 
-   */
   issuer_name_size = sizeof (issuer_name);
   ret =
-    gnutls_x509_crt_get_issuer_dn (x509_cert_list[x509_ncerts - 1],
-                                   issuer_name, &issuer_name_size);
+    gnutls_x509_crt_get_issuer_dn (cert, issuer_name, &issuer_name_size);
   if (ret < 0)
-    error (EXIT_FAILURE, 0, "get_issuer_dn: %s", gnutls_strerror (ret));
+    error (EXIT_FAILURE, 0, "gnutls_x509_crt_get_issuer_dn: %s", gnutls_strerror (ret));
 
-  fprintf (outfile, "\tIssued by: %s\n", name);
+  name_size = sizeof (name);
+  ret =
+    gnutls_x509_crt_get_dn (cert, name, &name_size);
+  if (ret < 0)
+    error (EXIT_FAILURE, 0, "gnutls_x509_crt_get_dn: %s", gnutls_strerror (ret));
 
-  if (strcmp (issuer_name, name) != 0)
-    error (EXIT_FAILURE, 0, "the last certificate is not self signed");
+  fprintf (outfile, "\tSubject: %s\n", name);
+  fprintf (outfile, "\tIssuer: %s\n", issuer_name);
 
-  fprintf (outfile, "\tVerification output: ");
-  print_verification_res (x509_cert_list[x509_ncerts - 1],
-                          x509_cert_list[x509_ncerts - 1], x509_crl_list,
-                          /* we add GNUTLS_VERIFY_ALLOW_X509_V1_CA_CRT since it is
-                           * self signed. */
-                          x509_ncrls,
-                          GNUTLS_VERIFY_DO_NOT_ALLOW_SAME |
-                          GNUTLS_VERIFY_ALLOW_X509_V1_CA_CRT);
+  if (issuer != NULL)
+    {
+      issuer_name_size = sizeof (issuer_name);
+      ret =
+        gnutls_x509_crt_get_dn (issuer, issuer_name, &issuer_name_size);
+      if (ret < 0)
+        error (EXIT_FAILURE, 0, "gnutls_x509_crt_get_issuer_dn: %s", gnutls_strerror (ret));
+
+      fprintf (outfile, "\tChecked against: %s\n", issuer_name);
+    }
+
+  if (crl != NULL)
+    {
+      gnutls_datum_t data;
+
+      issuer_name_size = sizeof (issuer_name);
+      ret =
+        gnutls_x509_crl_get_issuer_dn (crl, issuer_name, &issuer_name_size);
+      if (ret < 0)
+        error (EXIT_FAILURE, 0, "gnutls_x509_crl_get_issuer_dn: %s", gnutls_strerror (ret));
+
+      name_size = sizeof(tmp);
+      ret = gnutls_x509_crl_get_number(crl, tmp, &name_size, NULL);
+      if (ret < 0)
+        strcpy(name, "unnumbered");
+      else
+        {
+          data.data = tmp;
+          data.size = name_size;
+
+          name_size = sizeof(name);
+          ret = gnutls_hex_encode(&data, name, &name_size);
+          if (ret < 0)
+            error (EXIT_FAILURE, 0, "gnutls_hex_encode: %s", gnutls_strerror (ret));
+        }
+      fprintf (outfile, "\tChecked against CRL[%s] of: %s\n", name, issuer_name);
+    }
+
+  fprintf (outfile, "\tOutput: ");
+  print_verification_res(outfile, verification_output);
+
+  fputs(".\n\n", outfile);
+
+  return 0;
+}
+
+/* Will verify a certificate chain. If no CA certificates
+ * are provided, then the last certificate in the certificate
+ * chain is used as a CA.
+ */
+static int
+_verify_x509_mem (const void *cert, int cert_size, const void* ca, int ca_size)
+{
+  int ret;
+  gnutls_datum_t tmp;
+  gnutls_x509_crt_t *x509_cert_list = NULL;
+  gnutls_x509_crt_t *x509_ca_list = NULL;
+  gnutls_x509_crl_t *x509_crl_list = NULL;
+  unsigned int x509_ncerts, x509_ncrls = 0, x509_ncas = 0;
+  gnutls_x509_trust_list_t list;
+  unsigned int output;
+
+  ret = gnutls_x509_trust_list_init(&list, 0);
+  if (ret < 0)
+     error (EXIT_FAILURE, 0, "gnutls_x509_trust_list_init: %s", 
+                 gnutls_strerror (ret));
+
+  if (ca == NULL)
+    {
+      tmp.data = (void*)cert;
+      tmp.size = cert_size;
+    }
+  else
+    {
+      tmp.data = (void*)ca;
+      tmp.size = ca_size;
+
+      /* Load CAs */
+      ret = gnutls_x509_crt_list_import2( &x509_ca_list, &x509_ncas, &tmp, 
+        GNUTLS_X509_FMT_PEM, 0);
+      if (ret < 0 || x509_ncas < 1)
+         error (EXIT_FAILURE, 0, "error parsing CAs: %s", 
+                     gnutls_strerror (ret));
+    }
+
+  ret = gnutls_x509_crl_list_import2( &x509_crl_list, &x509_ncrls, &tmp, 
+    GNUTLS_X509_FMT_PEM, 0);
+  if (ret < 0)
+    {
+      x509_crl_list = NULL;
+      x509_ncrls = 0;
+    }
+
+  tmp.data = (void*)cert;
+  tmp.size = cert_size;
+
+  /* ignore errors. CRLs might not be given */
+  ret = gnutls_x509_crt_list_import2( &x509_cert_list, &x509_ncerts, &tmp, 
+    GNUTLS_X509_FMT_PEM, 0);
+  if (ret < 0 || x509_ncerts < 1)
+     error (EXIT_FAILURE, 0, "error parsing CRTs: %s", 
+                 gnutls_strerror (ret));
+
+  if (ca == NULL)
+    {
+      x509_ca_list = &x509_cert_list[x509_ncerts - 1];
+      x509_ncas = 1;
+    }
+
+  fprintf(stdout, "Loaded %d certificates, %d CAs and %d CRLs\n\n", 
+    x509_ncerts, x509_ncas, x509_ncrls);
+
+  ret = gnutls_x509_trust_list_add_cas(list, x509_ca_list, x509_ncas, 0);
+  if (ret < 0)
+     error (EXIT_FAILURE, 0, "gnutls_x509_trust_add_cas: %s", 
+                 gnutls_strerror (ret));
+
+  ret = gnutls_x509_trust_list_add_crls(list, x509_crl_list, x509_ncrls, 0, 0);
+  if (ret < 0)
+     error (EXIT_FAILURE, 0, "gnutls_x509_trust_add_crls: %s", 
+                 gnutls_strerror (ret));
+
+  gnutls_free(x509_crl_list);
+
+  ret = gnutls_x509_trust_list_verify_crt (list, x509_cert_list, x509_ncerts,
+    GNUTLS_VERIFY_DO_NOT_ALLOW_SAME|GNUTLS_VERIFY_ALLOW_X509_V1_CA_CRT, &output,
+    detailed_verification);
+  if (ret < 0)
+     error (EXIT_FAILURE, 0, "gnutls_x509_trusted_list_verify_crt: %s", 
+                 gnutls_strerror (ret));
+
+  fprintf (outfile, "Chain verification output: ");
+  print_verification_res(outfile, output);
 
   fprintf (outfile, ".\n\n");
 
-  /* Verify using internal algorithm too. */
-  {
-    int verify_status;
-
-    ret = gnutls_x509_crt_list_verify (x509_cert_list, x509_ncerts,
-                                       &x509_cert_list[x509_ncerts - 1], 1,
-                                       x509_crl_list,
-                                       x509_ncrls,
-                                       GNUTLS_VERIFY_ALLOW_X509_V1_CA_CRT |
-                                       GNUTLS_VERIFY_DO_NOT_ALLOW_SAME,
-                                       &verify_status);
-    if (ret < 0)
-      error (EXIT_FAILURE, 0, "gnutls_x509_crt_list_verify: %s",
-             gnutls_strerror (ret));
-
-    fprintf (outfile, "Chain verification output: ");
-
-    if (verify_status & GNUTLS_CERT_INVALID)
-      {
-        fprintf (outfile, "Not verified");
-      }
-    else
-      {
-        fprintf (outfile, "Verified");
-      }
-
-    if (verify_status & GNUTLS_CERT_SIGNER_NOT_CA)
-      {
-        fprintf (outfile, ", ");
-        fprintf (outfile, "Issuer is not a CA");
-      }
-
-    if (verify_status & GNUTLS_CERT_INSECURE_ALGORITHM)
-      {
-        fprintf (outfile, ", ");
-        fprintf (outfile, "Insecure algorithm");
-      }
-
-    fprintf (outfile, ".\n");
-  }
-
-  for (i = 0; i < x509_ncerts; i++)
-    gnutls_x509_crt_deinit (x509_cert_list[i]);
-
-  for (i = 0; i < x509_ncrls; i++)
-    gnutls_x509_crl_deinit (x509_crl_list[i]);
-
-  free (x509_cert_list);
-  free (x509_crl_list);
+  gnutls_free(x509_cert_list);
+  gnutls_x509_trust_list_deinit(list, 1);
 
   if (ret < 0)
     error (EXIT_FAILURE, 0, "verification error: %s", gnutls_strerror (ret));
@@ -2190,18 +2106,9 @@ _verify_x509_mem (const void *cert, int cert_size)
 }
 
 static void
-print_verification_res (gnutls_x509_crt_t crt,
-                        gnutls_x509_crt_t issuer,
-                        gnutls_x509_crl_t * crl_list, int crl_list_size,
-                        unsigned int flags)
+print_verification_res (FILE* outfile, unsigned int output)
 {
-  unsigned int output;
   int comma = 0;
-  int ret;
-
-  ret = gnutls_x509_crt_verify (crt, &issuer, 1, flags, &output);
-  if (ret < 0)
-    error (EXIT_FAILURE, 0, "verification error: %s", gnutls_strerror (ret));
 
   if (output & GNUTLS_CERT_INVALID)
     {
@@ -2246,20 +2153,16 @@ print_verification_res (gnutls_x509_crt_t crt,
       comma = 1;
     }
 
-  ret = gnutls_x509_crt_check_revocation (crt, crl_list, crl_list_size);
-  if (ret < 0)
-    error (EXIT_FAILURE, 0, "revocation check: %s", gnutls_strerror (ret));
-
-  if (ret == 1)
-    {                           /* revoked */
+  if (output & GNUTLS_CERT_REVOKED)
+    {
       if (comma)
         fprintf (outfile, ", ");
-      comma = 1;
       fprintf (outfile, "Revoked");
+      comma = 1;
     }
 }
 
-void
+static void
 verify_chain (void)
 {
   char *buf;
@@ -2271,7 +2174,36 @@ verify_chain (void)
 
   buf[size] = 0;
 
-  _verify_x509_mem (buf, size);
+  _verify_x509_mem (buf, size, NULL, 0);
+
+}
+
+static void
+verify_certificate (common_info_st * cinfo)
+{
+  char *cert;
+  char *cas;
+  size_t cert_size, ca_size;
+  FILE * ca_file = fopen(cinfo->ca, "r");
+  
+  if (ca_file == NULL)
+    error (EXIT_FAILURE, errno, "opening CA file");
+
+  cert = fread_file (infile, &cert_size);
+  if (cert == NULL)
+    error (EXIT_FAILURE, errno, "reading certificate chain");
+
+  cert[cert_size] = 0;
+
+  cas = fread_file (ca_file, &ca_size);
+  if (cas == NULL)
+    error (EXIT_FAILURE, errno, "reading CA list");
+
+  cas[ca_size] = 0;
+  fclose(ca_file);
+
+  _verify_x509_mem (cert, cert_size, cas, ca_size);
+
 
 }
 
@@ -2379,7 +2311,7 @@ generate_pkcs8 (common_info_st * cinfo)
 
   fprintf (stderr, "Generating a PKCS #8 key structure...\n");
 
-  key = load_x509_private_key (1, cinfo);
+  key = load_private_key (1, cinfo);
 
   if (info.pass)
     password = info.pass;
@@ -2431,7 +2363,7 @@ generate_pkcs12 (common_info_st * cinfo)
 
   fprintf (stderr, "Generating a PKCS #12 structure...\n");
 
-  key = load_x509_private_key (0, cinfo);
+  key = load_private_key (0, cinfo);
   crts = load_cert_list (0, &ncrts, cinfo);
 
   name = get_pkcs12_key_name ();
@@ -2927,7 +2859,7 @@ print_key_usage (FILE * outfile, unsigned int usage)
 }
 
 void
-pubkey_info (gnutls_x509_crt crt, common_info_st * cinfo)
+pubkey_info (gnutls_x509_crt_t crt, common_info_st * cinfo)
 {
   gnutls_pubkey_t pubkey;
   unsigned int bits, usage;
