@@ -34,6 +34,7 @@
 #include <error.h>
 
 #include <gnutls/gnutls.h>
+#include <gnutls/dtls.h>
 #include <gnutls/extra.h>
 #include <gnutls/x509.h>
 #include <gnutls/openpgp.h>
@@ -53,7 +54,7 @@
 #define MAX_BUF 4096
 
 /* global stuff here */
-int resume, starttls, insecure, rehandshake;
+int resume, starttls, insecure, rehandshake, udp, mtu;
 const char *hostname = NULL;
 char *service;
 int record_max_size;
@@ -170,11 +171,10 @@ load_keys (void)
 {
   unsigned int crt_num;
   int ret;
-  gnutls_datum_t data = { NULL, 0 };
+  gnutls_datum_t data;
 
   if (x509_certfile != NULL && x509_keyfile != NULL)
     {
-#ifdef ENABLE_PKCS11
       if (strncmp (x509_certfile, "pkcs11:", 7) == 0)
         {
           crt_num = 1;
@@ -196,7 +196,6 @@ load_keys (void)
           x509_crt_size = 1;
         }
       else
-#endif /* ENABLE_PKCS11 */
         {
 
           data = load_file (x509_certfile);
@@ -230,11 +229,10 @@ load_keys (void)
             }
           x509_crt_size = ret;
         }
-      fprintf (stderr, "Processed %d client certificates...\n", ret);
+      fprintf (stdout, "Processed %d client certificates...\n", ret);
 
       unload_file (data);
 
-#ifdef ENABLE_PKCS11
       if (strncmp (x509_keyfile, "pkcs11:", 7) == 0)
         {
           gnutls_pkcs11_privkey_init (&pkcs11_key);
@@ -249,7 +247,6 @@ load_keys (void)
             }
         }
       else
-#endif /* ENABLE_PKCS11 */
         {
           data = load_file (x509_keyfile);
           if (data.data == NULL)
@@ -272,7 +269,7 @@ load_keys (void)
           unload_file (data);
         }
 
-      fprintf (stderr, "Processed %d client X.509 certificates...\n",
+      fprintf (stdout, "Processed %d client X.509 certificates...\n",
                x509_crt_size);
     }
 #ifdef ENABLE_OPENPGP
@@ -299,7 +296,6 @@ load_keys (void)
 
       unload_file (data);
 
-#ifdef ENABLE_PKCS11
       if (strncmp (pgp_keyfile, "pkcs11:", 7) == 0)
         {
           gnutls_pkcs11_privkey_init (&pkcs11_key);
@@ -313,7 +309,6 @@ load_keys (void)
             }
         }
       else
-#endif /* ENABLE_PKCS11 */
         {
 
           data = load_file (pgp_keyfile);
@@ -342,7 +337,7 @@ load_keys (void)
 
       if (info.pgp_subkey != NULL)
         {
-          unsigned char keyid[GNUTLS_OPENPGP_KEYID_SIZE];
+          gnutls_openpgp_keyid_t keyid;
 
           if (strcasecmp (info.pgp_subkey, "auto") == 0)
             {
@@ -371,7 +366,7 @@ load_keys (void)
             }
         }
 
-      fprintf (stderr, "Processed 1 client PGP certificate...\n");
+      fprintf (stdout, "Processed 1 client PGP certificate...\n");
     }
 #endif
 
@@ -557,7 +552,15 @@ init_tls_session (const char *hostname)
 
   gnutls_session_t session;
 
-  gnutls_init (&session, GNUTLS_CLIENT);
+  if (udp)
+    {
+      gnutls_init (&session, GNUTLS_CLIENT|GNUTLS_DATAGRAM);
+      if (mtu)
+        gnutls_dtls_set_mtu(session, mtu);
+    }
+  else
+    gnutls_init (&session, GNUTLS_CLIENT);
+
 
   if (gnutls_priority_set_direct (session, info.priorities, &err) < 0)
     {
@@ -624,7 +627,7 @@ handle_error (socket_st * hd, int err)
   int alert, ret;
   const char *err_type, *str;
 
-  if (err >= 0)
+  if (err >= 0 || err == GNUTLS_E_AGAIN || err == GNUTLS_E_INTERRUPTED)
     return 0;
 
   if (gnutls_error_is_fatal (err) == 0)
@@ -651,12 +654,6 @@ handle_error (socket_st * hd, int err)
       if (str == NULL)
         str = str_unknown;
       printf ("*** Received alert [%d]: %s\n", alert, str);
-
-      /* In SRP if the alert is MISSING_SRP_USERNAME,
-       * we should read the username/password and
-       * call gnutls_handshake(). This is not implemented
-       * here.
-       */
     }
 
   check_rehandshake (hd, err);
@@ -666,13 +663,11 @@ handle_error (socket_st * hd, int err)
 
 int starttls_alarmed = 0;
 
-#ifndef _WIN32
 static void
 starttls_alarm (int signum)
 {
   starttls_alarmed = 1;
 }
-#endif
 
 static void
 tls_log_func (int level, const char *str)
@@ -680,58 +675,19 @@ tls_log_func (int level, const char *str)
   fprintf (stderr, "|<%d>| %s", level, str);
 }
 
-#define IN_KEYBOARD 1
-#define IN_NET 2
-#define IN_NONE 0
-/* returns IN_KEYBOARD for keyboard input and IN_NET for network input
- */
-static int check_net_or_keyboard_input(socket_st* hd)
-{
-  int maxfd;
-  fd_set rset;
-  int err;
-  struct timeval tv;
-
-  do
-    {
-      FD_ZERO (&rset);
-      FD_SET (fileno (stdin), &rset);
-      FD_SET (hd->fd, &rset);
-
-      maxfd = MAX (fileno (stdin), hd->fd);
-      tv.tv_sec = 0;
-      tv.tv_usec = 500 * 1000;
-
-      if (hd->secure == 1)
-        if (gnutls_record_check_pending(hd->session))
-          return IN_NET;
-
-      err = select (maxfd + 1, &rset, NULL, NULL, &tv);
-      if (err < 0)
-        continue;
-
-      if (FD_ISSET (hd->fd, &rset))
-        return IN_NET;
-
-
-      if (FD_ISSET (fileno (stdin), &rset))
-        return IN_KEYBOARD;
-    }
-  while(err == 0);
-  
-  return IN_NONE;
-}
-
 int
 main (int argc, char **argv)
 {
-  int ret;
-  int ii, i, inp;
+  int err, ret;
+  int ii, i;
   char buffer[MAX_BUF + 1];
   char *session_data = NULL;
   char *session_id = NULL;
   size_t session_data_size;
   size_t session_id_size = 0;
+  fd_set rset;
+  int maxfd;
+  struct timeval tv;
   int user_term = 0, retval = 0;
   socket_st hd;
   ssize_t bytes;
@@ -750,9 +706,7 @@ main (int argc, char **argv)
       exit (1);
     }
 
-#ifdef ENABLE_PKCS11
   pkcs11_common ();
-#endif
   gaa_parser (argc, argv);
   if (hostname == NULL)
     {
@@ -892,9 +846,19 @@ after_handshake:
             }
         }
 
-      inp = check_net_or_keyboard_input(&hd);
+      FD_ZERO (&rset);
+      FD_SET (fileno (stdin), &rset);
+      FD_SET (hd.fd, &rset);
 
-      if (inp == IN_NET)
+      maxfd = MAX (fileno (stdin), hd.fd);
+      tv.tv_sec = 3;
+      tv.tv_usec = 0;
+
+      err = select (maxfd + 1, &rset, NULL, NULL, &tv);
+      if (err < 0)
+        continue;
+
+      if (FD_ISSET (hd.fd, &rset))
         {
           memset (buffer, 0, MAX_BUF + 1);
           ret = socket_recv (&hd, buffer, MAX_BUF);
@@ -926,7 +890,7 @@ after_handshake:
             break;
         }
 
-      if (inp == IN_KEYBOARD)
+      if (FD_ISSET (fileno (stdin), &rset))
         {
           if ((bytes = read (fileno (stdin), buffer, MAX_BUF - 1)) <= 0)
             {
@@ -1020,6 +984,8 @@ gaa_parser (int argc, char **argv)
   resume = info.resume;
   rehandshake = info.rehandshake;
   insecure = info.insecure;
+  udp = info.udp;
+  mtu = info.mtu;
   service = info.port;
   record_max_size = info.record_size;
   fingerprint = info.fingerprint;
@@ -1121,7 +1087,6 @@ do_handshake (socket_st * socket)
 
 
       socket->secure = 1;
-
     }
   else
     {
@@ -1168,10 +1133,11 @@ psk_callback (gnutls_session_t session, char **username, gnutls_datum_t * key)
     {
       char *tmp = NULL;
       size_t n;
+      ssize_t len;
 
       printf ("Enter PSK identity: ");
       fflush (stdout);
-      getline (&tmp, &n, stdin);
+      len = getline (&tmp, &n, stdin);
 
       if (tmp == NULL)
         {
@@ -1318,7 +1284,8 @@ init_global_tls_stuff (void)
                    gnutls_strerror (ret));
         }
     }
-  gnutls_psk_set_client_credentials_function (psk_cred, psk_callback);
+  else
+    gnutls_psk_set_client_credentials_function (psk_cred, psk_callback);
 #endif
 
 #ifdef ENABLE_ANON
@@ -1437,7 +1404,7 @@ socket_open (socket_st * hd, const char *hostname, const char *service)
   printf ("Resolving '%s'...\n", hostname);
   /* get server name */
   memset (&hints, 0, sizeof (hints));
-  hints.ai_socktype = SOCK_STREAM;
+  hints.ai_socktype = udp ? SOCK_DGRAM : SOCK_STREAM;
   if ((err = getaddrinfo (hostname, service, &hints, &res)))
     {
       fprintf (stderr, "Cannot resolve %s:%s: %s\n", hostname, service,
@@ -1470,6 +1437,18 @@ socket_open (socket_st * hd, const char *hostname, const char *service)
       exit (1);
     }
 
+#ifdef IP_DONTFRAG
+  if (hints.ai_socktype == SOCK_DGRAM)
+    {
+      int yes = 1;
+      if (setsockopt (sd, IPPROTO_IP, IP_DONTFRAG,
+                          (const void *) &yes, sizeof (yes)) < 0)
+        {
+          perror ("setsockopt(IP_DF) failed");
+        }
+    }
+#endif
+
   hd->secure = 0;
   hd->fd = sd;
   hd->hostname = strdup (hostname);
