@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009, 2010, 2011 Free Software Foundation, Inc.
+ * Copyright (C) 2009, 2010 Free Software Foundation, Inc.
  *
  * Author: Simon Josefsson
  *
@@ -33,8 +33,8 @@
 #include <stdio.h>
 #include <stdarg.h>
 #include <stdlib.h>
-#include <string.h>
 
+#include <gcrypt.h>
 #include <gnutls/gnutls.h>
 #include <gnutls/x509.h>
 
